#!/bin/bash
# asset.sh / 2021.09.20

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

# -----------------------------------------------------------------------------------
# --- Init locking Files (Flags) ....
# -----------------------------------------------------------------------------------

# --- "CronFlag": -------------------------------------------------------------------
# >>> Set by Deamon (AvrLogger.py) in every "Trap-Signal handler" to indicate a Signal was resieved.
# >>> Is queried in CronJob (AvrShedule.py) before a "new" signal is sent to the Deamon.
CrnFlag="${ETC}/tmp/CrnFlag"

# --- "AvrLock": --------------------------------------------------------------------
# >>> Set by Daemon (AvrLogger.py) in "init.Init.py": Is always set when an initialization
# >>> or config update in is running. Will be deleted after successful initialization or
# >>> config update.
AvrLock="${ETC}/tmp/AvrLock"

# --- "SensorLock": -----------------------------------------------------------------
# >>> Set by Daemon (AvrLogger.py) in "Main[endless]loop": Is always set when sensor values
# >>> are read in or processed. Will be deleted when the daemon waits for the next "LoopTick".
SenLock="${ETC}/tmp/SenLock"

# --- "SvgLock": --------------------------------------------------------------------
# >>> Set by Deamon (AvrLogger.py) in "SVG.Svg.py": Set when the SVG file is being updated
# >>> and deleted immediately afterwards.
SvgLock="${ETC}/tmp/SvgLock"

# --- "CgiUpdateFlag": --------------------------------------------------------------
# >>> Is set in different places "only" when the daemon (AvrLogger.py) is running, to signal
# >>> that an update has to be processed. The CGI-Userinterface (AvrLogger.cgi) uses this
# >>> flag to detect if a "Busy message" should be displayed. Will be deleted in the daemon
# >>> (AvrLogger.py) at the end of "Init.py" and is also deleted in the
# >>> "start-stop-status" script called with "STOP".
CgiUpdate="${ETC}/tmp/CgiUpdate"

# --- "HeartbeatFlag": --------------------------------------------------------------
# >>> Set by Daemon (AvrLogger.py) in different places and is a kind of "watchdog"
Heartbeat="${ETC}/tmp/Heartbeat"

# --- "ShmLock": --------------------------------------------------------------------
# >>> Is always set if an "shared memory" is present and the daemon is to be stopped.
# >>> Will be deleted in "DissolveShm" & "PrepareShm" after the "shared memory" has been
# >> prepared  or dissolved.
ShmLock="${ETC}/log/ShmLock"

# --- "FlowFlag": -------------------------------------------------------------------
# >>> Set and deleted by "Add.Flow.py"
# >>> Indicates that currently data is being written to "AvrLogger.dat"
FlowFlag="${ETC}/tmp/FlowFlag"

# --- "VirtualDSM": -----------------------------------------------------------------
# >>> Is set during installation and signals a virtual environment.
VirtualDSM="${ETC}/log/VirtualDSM"
ExcludeSCEMD="${ETC}/log/ExcludeSCEMD"

# --- "ExcludeSHM": -----------------------------------------------------------------
# >>> Is set during installation and signals the use of Share Memory.
ExcludeSHM="${ETC}/log/ExcludeSHM"

# --- "AvrDisabled": -----------------------------------------------------------------
# >>> Controlled via the GUI and determines whether the Daemon may be started or not.
AvrDisabled="${ETC}/log/disabled"

# -----------------------------------------------------------------------------------
CronTab="/etc/cron.d/crontab"
sysHostName="/etc/hostname"
ScheduleConf="${ETC}/config/log.conf"

# -----------------------------------------------------------------------------------
UpgradePKG="${ETC}/log/UpgradePKG"
UpgradeConf="${ETC}/log/UpgradeConf"
NewsAvrPKG="${ETC}/log/NewsAvrPKG"
RestoreConf="${ETC}/log/RestoreConf"
MemorySize="${ETC}/log/MemorySize"

# --- "SpeedBusy": ------------------------------------------------------------------
# >>> Is set by the function "SpeedTest.Ookla.py" and signals that a Speedtest is being performed.
SpeedBusy="${ETC}/tmp/SpeedBusy"
SynoCheck="${ETC}/tmp/synoCheck"

# --- "cpuScore": -------------------------------------------------------------------
# >>> CPU performance Score for determining the TimeOut value.
# >>> Is determined during installation or an update.
cpuScore="${ETC}/log/cpuScore"

# -----------------------------------------------------------------------------------
# --- Some ANSI escape code's (Colors)
Red=$'\033[1;31m'
Gre=$'\033[1;32m'
Blu=$'\033[1;34m'
Yel=$'\033[1;33m'
NC=$'\033[0m'
