#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import os
import sys
import time

import errno
from   traceback import format_exc as Trace

sys.path.append( os.path.dirname(os.path.abspath(sys.argv[0])) + "/lib")

import asset
import basics   as io
import lib

# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
pMain = sys.modules[ __name__ ]

Debug       = False
AvrSmart    = False
TestMail    = False # Do not clear "ERROR!-Log

# -----------------------------------------------------------------------------------
# --- Redirect stderr to "ERROR" as early as possible to really log all errors ...
if not sys.stdin.isatty() :
   sys.stderr = open( asset.ERROR, 'a')

# -----------------------------------------------------------------------------------
# --- Helper function for getting the current exception in a try/except block
# -----------------------------------------------------------------------------------
def getException() :

   return sys.exc_info()[1]

# -----------------------------------------------------------------------------------
# --- Function : "Cron"
# -----------------------------------------------------------------------------------
def func_Cron ( sArg ) :

   import Cron
   ScheduleStatus = Cron.checkTime()

   # --------------------------------------------------------------------------------
   # --- Discover all parent instances of Deamon ...
   Status = lib.GetPPidOf( f'{asset.SpkDaemon}.py' )

   # --------------------------------------------------------------------------------
   # --- Wait until only one instance of Deamon is running or until TimeOut
   TimeOut = int( time.time() ) + pMain.iTimeOut
   while len( Status ) > 1 :
      # -----------------------------------------------------------------------------
      if int( time.time() ) >= TimeOut :
         PID = lib.getCMD( 'ps f -opid,cmd -C python3' ).strip().split('\n')
         lib.Error( asset.msg['errCronDouble'] )

         for sID in sys.argv :
            lib.Error( f'ARG : {sID}' )

         for sID in PID :
            lib.Error( f'PID : {sID}' )

         return

      # -----------------------------------------------------------------------------
      time.sleep(0.500)
      Status = lib.GetPPidOf( f'{asset.SpkDaemon}.py' )

   # --------------------------------------------------------------------------------
   # --- Wait until Daemon is in Idle or until TimeOut ...
   if Status :
      io.ioRm( asset.f_Heartbeat )
      TimeOut = int( time.time() ) + pMain.iTimeOut

      while os.path.exists( asset.f_AvrLock ) or os.path.exists( asset.f_SenLock ) or os.path.exists( asset.f_SvgLock ) or os.path.exists( asset.f_CrnFlag ) :
         if int( time.time() ) >= TimeOut :
            lib.Error( asset.msg['errCronIdleTime'] )
            return

         time.sleep( 0.010 )

         if os.path.exists( asset.f_Heartbeat ) :
            TimeOut = int( time.time() ) + pMain.iTimeOut
            io.ioRm( asset.f_Heartbeat )

   else :
      # -----------------------------------------------------------------------------
      # --- If the Daemon is not running and the argument is not "start" ...
      if ScheduleStatus >= 0 and sArg in [ 'archive', 'update' ] and os.path.exists( asset.avrPidFile ) :
         lib.Error( asset.msg['errCronQuit'] )

         if lib.GetShm() :
            io.touch( asset.f_ShmLock )
            ShmFlag = True
         else :
            ShmFlag = False

         io.touch( asset.f_CgiUpdate )
         # --------------------------------------------------------------------------
         # --- Run / Start the Daemon ...
         os.system( ' '.join(( asset.script, 'cron', 'start', '&' )) )

         # --------------------------------------------------------------------------
         # --- Wait until "shared memory" is prepared  or dissolved ...
         if ShmFlag :
            TimeOut = int( time.time() ) + pMain.iTimeOut
            while os.path.exists( asset.f_ShmLock ) :
               if int( time.time() ) >= TimeOut :
                  lib.Error( asset.msg['errCronRestart'] )
                  return

               time.sleep( 0.010 )

         # --------------------------------------------------------------------------
         io.ioRm( asset.f_Heartbeat )
         TimeOut = int( time.time() ) + pMain.iTimeOut

         while os.path.exists( asset.f_CgiUpdate ) :
            if int( time.time() ) >= TimeOut :
               lib.Error( asset.msg['errCronRestart'] )
               sys.exit()
   
            time.sleep( 0.010 )

            if os.path.exists( asset.f_Heartbeat ) :
               TimeOut = int( time.time() ) + pMain.iTimeOut
               io.ioRm( asset.f_Heartbeat )

   # --------------------------------------------------------------------------------
   # --- Execute argument ... 
   # --------------------------------------------------------------------------------
   while lib.switch( sArg ):
      # -----------------------------------------------------------------------------
      if lib.case( 'check' ) :
         # --------------------------------------------------------------------------
         if not Status and ScheduleStatus >= 0 :
            # -----------------------------------------------------------------------
            # --- Start Daemon (AvrLogger) & stop hibernation ...
            lib.Error( asset.msg['errCronStop'] )
            os.system( ' '.join(( asset.script, 'cron', 'start', '&' )) )
            break
         # --------------------------------------------------------------------------
         if Status and ScheduleStatus == -1 :
            # -----------------------------------------------------------------------
            # --- Check "ArchiveMode" is "Stop" ... 
            if asset.AvrConf["SvgArchiveMode"] == "Stop" :
               io.touch( asset.f_CrnFlag )
               TimeOut = int( time.time() ) + pMain.iTimeOut
               os.kill( int( Status[0] ), 30 ) # archive plot ...
               # --- Wait for archiving to complete or until timeout ...
               while os.path.exists( asset.f_CrnFlag ) :
                  if int( time.time() ) >= TimeOut :
                     lib.Error( asset.msg['errCronIdleTime'] )
                     return

                  time.sleep( 0.010 )

            # -----------------------------------------------------------------------
            # --- Stop Daemon (AvrLogger) & start hibernation ...
            lib.Error( asset.msg['errCronStart'] )
            os.system( ' '.join(( asset.script, 'cron', 'stop', '&' )) )
            break
      # -----------------------------------------------------------------------------
      if lib.case( 'update' ) :
         if Status and ScheduleStatus >= 0 :
            # -----------------------------------------------------------------------
            # --- 16 : SvgTick has expired, update plot ...
            os.kill( int( Status[0] ), 16 )
         break
      # -----------------------------------------------------------------------------
      if lib.case( 'archive' ) :
         if Status and ScheduleStatus >= 0 :
            # -----------------------------------------------------------------------
            # --- 30 : Archive timer has expired, archive plot ...
            os.kill( int( Status[0] ), 30 )
         break
      # --- End of "switch / case" --------------------------------------------------
      break

# -----------------------------------------------------------------------------------
# --- Function : "Ookla" - Speedtest
# -----------------------------------------------------------------------------------
def func_Ookla ( sArg ) :

   # --------------------------------------------------------------------------------
   # --- Performe Speedtest "only" if Daemon is running ...
   if not lib.GetPPidOf( f'{asset.SpkDaemon}.py' ) and not pMain.Debug :
      return

   # --------------------------------------------------------------------------------
   # --- Only "one" instance of "Ookla" allowed,
   # --- therefore ... discover all parent instances of "Ookla" ...
   ListPID = lib.GetPPidOf( 'Ookla.py' )

   if len( ListPID ) > 1 :                # --- If more than one instance was found ...
      for pid in ListPID :                # --- Walk through all found instances ...
         if int( os.getpid() ) != pid :   # --- Kill (stop) this instance ...
            os.kill( pid, 9 )

   # --------------------------------------------------------------------------------
   # --- Performe Speedtest ...
   os.system( ' '.join(( asset.OoklaScript, '&' )) )

# -----------------------------------------------------------------------------------
# --- Function : "Report"
# -----------------------------------------------------------------------------------
def func_Report ( sArg ) :

   import Status

   # --------------------------------------------------------------------------------
   Status.init()                          # "Always" Init Mail-Values first
   Status.StatusMail( pMain.TestMail )    # Build Status-Report

# -----------------------------------------------------------------------------------
# --- Function-Array : Do not move !!
# -----------------------------------------------------------------------------------
iFunctions =   {
                  'cron'      : func_Cron   ,
                  'ookla'     : func_Ookla  ,
                  'report'    : func_Report ,
               }

# -----------------------------------------------------------------------------------
# --- AvrLogger Cron-Scheduler : start / stop / speed / report
# -----------------------------------------------------------------------------------
if __name__ == '__main__' :

   sFunc = False
   sCall = False

   # --------------------------------------------------------------------------------
   # --- Check if an argument has been passed ...
   for arg in sys.argv[1:] :
      while lib.switch( arg ) :
         # --------------------------------------------------------------------------
         if lib.case( 'debug' ) :
            Debug = True
            break
         # --------------------------------------------------------------------------
         if lib.case( 'TestMail' ) :
            TestMail = True
            break
         # --------------------------------------------------------------------------
         if lib.case( 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun' ) :
            break
         # --------------------------------------------------------------------------
         if lib.case( 'check', 'start', 'stopp', 'stop' ) :
            sCall = 'cron'
            sFunc = 'check'
            break
         # --------------------------------------------------------------------------
         if lib.case( 'update' ) :
            sCall = 'cron'
            sFunc = 'update'
            break
         # --------------------------------------------------------------------------
         if lib.case( 'archive' ) :
            sCall = 'cron'
            sFunc = 'archive'
            break
         # --------------------------------------------------------------------------
         if lib.case( 'speed' ) :
            sCall = 'ookla'
            sFunc = 'speed'
            break
         # --------------------------------------------------------------------------
         if lib.case( 'mail' ) :
            sCall = 'report'
            sFunc = 'mail'
            break
         # --- Default --------------------------------------------------------------
         sCall = ''
         sFunc = ''
         break

   # --------------------------------------------------------------------------------
   # --- Exit if "Package - Disabled-Flag" is set ...
   if not os.path.exists( asset.f_AvrDisabled ) or pMain.Debug :
      # -----------------------------------------------------------------------------
      sFunctions = iFunctions.get( sCall, False )
      if not sFunctions :
         lib.Error( f'CRN: Unknown ARG "{arg}" has passed ...' )
         lib.Error(  'CRN: Execution was canceled' )
      else :
         if not bool( asset.msg     ) : asset.msg     = lib.InitLanguage( 'admin', asset.avrLangFile )
         if not bool( asset.AvrConf ) : asset.AvrConf = lib.ConfigRead( asset.avrConfigFile )

         pMain.iTimeOut = int( io.ioRead( asset.avrCpuScore ) )

         sFunctions( sFunc )

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
