# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import os
import sys
import time
import json
import re
import urllib.parse as     urlparse
from   urllib.parse import urlencode
from   urllib.parse import quote
from   html         import escape

from   textwrap     import dedent, indent

import asset
import lib

import basics  as io

# -----------------------------------------------------------------------------------
# --- Changing 'print' to use 'flush=True' as default :
# import functools
# print = functools.partial( print, flush=True )
# -----------------------------------------------------------------------------------

# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
iMainSubWidth     = 200    # Width of the Sub-Menu
iMainOutWidth     = 782    # Width of the Out-Page
SubMenuWidth      = iMainSubWidth - 60

iMainHeight       = 562   # Height of the Main-Menu
iMainSaveHeight   = 38
sWhiteSpace       = '&nbsp;'
sTab              = '   '

Box_Color = 65 # 62
Box_Input = 40

#  Column_1=130
Column_1  = 165
Column_1a = 150
Column_1b = 130
Column_1c = 120
Column_1e = 100
Column_1d =  90
Column_2  =  75
Column_2a =  80
Column_2b =  50
Column_2c =  30
Box_1     =  70
Box_1a    =  85
Box_1b    =  55
Box_1c    =  35
Box_2     = 146
Box_3     = 175
Box_4     = 450

# ------------------------------------------------------------------------------
# --- Print a Time-Window
# ------------------------------------------------------------------------------
def TimeBox( sPos, oWidth, iStart, iEnd ) :

   iSwap = False
   fProd = oWidth / 1440

   Hour, Minute = iStart.split( ':', 1 )
   iStart = int( Hour ) * 60 + int( Minute )

   Hour, Minute = iEnd.split( ':', 1 )
   iEnd   = int( Hour ) * 60 + int( Minute )

   if iEnd < iStart :
      iSwap = True
      iStart, iEnd = iEnd, iStart

   iWidth = int( ( iEnd - iStart ) * fProd + 0.5 )
   iStart = int( iStart * fProd + 0.5 )


   if iSwap : oClass, iClass = 'TimeBoxRed', 'TimeBoxGreen'
   else     : oClass, iClass = 'TimeBoxGreen', 'TimeBoxRed'

   print (f'<div class="TimeBoxOut {oClass}" style="width: {oWidth}px; left : {sPos}px;">' )
   print (f'   <div class="TimeBoxIn {iClass}" style="width: {iWidth}px; left : {iStart}px;">' )
   print ( '   </div>' )
   print ( '</div>' )

# -----------------------------------------------------------------------------------
# --- Transfere "Nice"-Path to "Full"-Path
# -----------------------------------------------------------------------------------
def GetFullPath( sOut, isFile=True ) :

   sFull = asset.cgiInput[sOut].strip().strip( '/' )
   sName = sFull.split( '/' )[-1]

   if isFile : sFull = lib.FindeShare( sFull.rsplit( '/', 1)[0] )
   else      : sFull = lib.FindeShare( sFull )

   if sFull :
      if isFile : asset.cgiInput[sOut] = os.path.join(               *sFull, sName )
      else      : asset.cgiInput[sOut] = os.path.join(               *sFull        )
   else     :
      if isFile : asset.cgiInput[sOut] = os.path.join( asset.avrPresetFiles, sName )
      else      : asset.cgiInput[sOut] = asset.avrPresetFiles

# -----------------------------------------------------------------------------------
# --- Output search result formated ...
# -----------------------------------------------------------------------------------
def printLogQuery( iRet, sDate=False, sTime=False, sKey=False, iMax=100 ) :

   sIndent = '        '
   tmpFile = ''
   sLogTxt = ''
   DateMask, TimeMask = LogMask( sDate=sDate, sTime=sTime, sKey=sKey )

   # --------------------------------------------------------------------------------
   for sLine in iRet :
      try    : sFile, iLine, sText = sLine.split( ':', 2 )
      except : continue

      if sFile != tmpFile :
         if tmpFile : sLogTxt += Colorize( '<br>File  : {0}<br>'.format( sFile ), 'Gold', True )
         else       : sLogTxt += Colorize(     'File  : {0}<br>'.format( sFile ), 'Gold', True )

      sLogTxt += Colorize( '{0:>5}'.format( iLine ), 'Lime' ) + " : "

      iFirst = False
      iCount = 0
      sText  = sText.strip()
      while len( sText ) > iMax :

         iCount += 1

         for iID in range( iMax, 1, -1 ) :
            if sText[iID] in [ ' ', '/', ',', ] :
               sPart = sText[:iID+1 ].strip()
               sText = sText[ iID+1:].strip()
               break

         # --------------------------------------------------------------------------
         if sKey and sKey in sPart : sPart = sPart.replace( sKey, Colorize( sKey, 'Red', True ) )

         # --------------------------------------------------------------------------
         if not iFirst :
            if DateMask :
               Match = getColorToken( sPart, DateMask )
               if Match : sPart = sPart.replace( Match, Colorize( Match, 'DodgerBlue', True ) )
            if TimeMask :
               Match = getColorToken( sPart, TimeMask )
               if Match : sPart = sPart.replace( Match, Colorize( Match, 'MediumVioletRed', True ) )

         # --------------------------------------------------------------------------
         if iFirst : sLogTxt += '{0}{1}<br>'.format( sIndent, sPart )
         else      : sLogTxt +=    '{0}<br>'.format( sPart )
            

         iFirst = True
         if iCount >= 20 : break

      # -----------------------------------------------------------------------------
      if not iFirst :
         if DateMask :
            Match = getColorToken( sText, DateMask )
            if Match : sText = sText.replace( Match, Colorize( Match, 'DodgerBlue', True ) )
         if TimeMask :
            Match = getColorToken( sText, TimeMask )
            if Match : sText = sText.replace( Match, Colorize( Match, 'MediumVioletRed', True ) )

      # -----------------------------------------------------------------------------
      if sKey and sKey in sText : sText = sText.replace( sKey, Colorize( sKey, 'Red', True ) )
      if iFirst : sLogTxt += '{0}{1}<br>'.format( sIndent, sText )
      else      : sLogTxt +=    '{0}<br>'.format( sText )

      tmpFile = sFile

   # --------------------------------------------------------------------------------
   return sLogTxt

# -----------------------------------------------------------------------------------
# --- Search for a keyword in all log files. Optionally limit the search to date and time ...
# -----------------------------------------------------------------------------------
def LogMask( sDate=False, sTime=False, sKey=False ) :

   sTerm = lib.dictGrepX( sDate=sDate, sTime=sTime, sKey=sKey )

   DateMask = False
   TimeMask = False

   # --------------------------------------------------------------------------------
   if not sDate and not sTime :
      # --- Just search string :
      return DateMask, TimeMask

   # --------------------------------------------------------------------------------
   if sDate :
      DateMask = "^{year}.?{month}.?{day}".format( **sTerm )

   # --------------------------------------------------------------------------------
   if sTime :
      if sTerm[ 'minute' ] == '--' :
         # --- Time only hour
         TimeMask = "{hour}:.{0}".format( '{2}:.{2}', **sTerm )
      else :
         # --- Time : hour and minute
         if len ( sTerm[ 'minute' ] ) == 1 :
            TimeMask = "{hour}:{minute}.{0}".format( '{1}:.{2}', **sTerm )
         else :
            TimeMask = "{hour}:{minute}:.{0}".format( '{2}', **sTerm )

   # --------------------------------------------------------------------------------
   return DateMask, TimeMask

# -----------------------------------------------------------------------------------
# ---
# -----------------------------------------------------------------------------------
def getColorToken( sLine, sReg ) :

   sRegX = re.compile( sReg )
   sRegX = sRegX.search( sLine )

   if sRegX : return sRegX.group()
   else     : return False

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
def Print ( *args, **kwargs ) :

   print( *args, file=sys.stderr, **kwargs )

# ------------------------------------------------------------------------------
# --- Main Menu-Icon :
# ------------------------------------------------------------------------------
def menuIcon ( Icon, sText, sPage, opt=False, iTab=7 ) :

   wTab = sTab * iTab
   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]
         
   if opt : opt = ' MenuIconActive'
   else   : opt = ''


   Out =     [  f'{wTab}<button class="InlineLeft MenuIcon{opt}" type="submit" name="page" value="{sPage}" title="{sText}">\n' ]
   Out.append(  f'{wTab}   <img class="MenuIcon" src="images/{Icon}"/>\n' )
   Out.append(  f'{wTab}</button>' )
   print ( ''.join( Out ) )

# ------------------------------------------------------------------------------
# --- Sub-Menu : Icon with Text :
# ------------------------------------------------------------------------------
def menuBtn( sName, sTitel, Width, Icon, url, lWidth=False, opt=False, onclick='', iTab=7, sCgi='AvrConfig' ) :

   wTab = sTab * iTab
   if lWidth : Label   ( '&nbsp;', lWidth )
   if opt : opt = ' subMenuIconActive'
   else   : opt = ''

   if onclick : sButton = f'<button class="subMenuIcon{opt}" href="{sCgi}.cgi?page={onclick}>'
   else       : sButton = f'<button class="subMenuIcon{opt}" type="submit" name="page" value="{url}">'

   Out =     [  f'{wTab}{sButton}\n' ]
   Out.append(  f'{wTab}   <img class="subMenuIcon" src="images/{Icon}"/>\n' )
   Out.append(  f'{wTab}   <a class="subMenuIcon" style="width: {Width}px;" title="{sTitel}" >{sName}</a>\n' )
   Out.append(  f'{wTab}</button>' )
   print ( ''.join( Out ) )

# ------------------------------------------------------------------------------
# --- Prints a single "Link" Button : Will open a new Browserwindow / -Tab
# ------------------------------------------------------------------------------
def lnkButton( sKey, sText, sLink, iSize=80, iTab=7 ) :

   wTab = sTab * iTab
   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]

   print (f'{wTab}<button style="width: {iSize}px;" onclick="window.open(\'{escape( sLink )}\');" type="button">{sText}</button>' )
   Sign ( sKey )

# ------------------------------------------------------------------------------
# --- Prints a single "Action" Button : Will open a new Window
# ------------------------------------------------------------------------------
def popButton( sKey, sText, sRet, iWidth=800, iHight=500, iClose=True, sCgi='AvrConfig', iSize=80, iTab=7 ):

   wTab = sTab * iTab
   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]

   if iClose : iClose = 'true'
   else      : iClose = 'false'

   print (f'{wTab}<button class="SingleButton" style="width: {iSize}px;" href="{sCgi}.cgi?page={quote( sRet )}&PopUp=Yes" onclick="return openWindow( this,{iWidth},{iHight},window,{iClose} )">{sText}</button>' )
   Sign ( sKey )

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
def SectionOpen( sText, iIndex=0, iTab=7 ) :

   wTab = sTab * iTab
   if 'sec_' + sText in asset.msg: sText = asset.msg['sec_' + sText ]

   opt  = asset.Out.get( 'AcdActive', False )
   name = f'AcdReturn_{iIndex}'

   if opt and opt == iIndex :
      opt   = 'trigger_active'
      style = ''
      value = 'Yes'
      src   = 'images/AccOpen.png'
   else :
      opt   = ''
      style = ' style="display: none;"'
      value = 'No'
      src   = 'images/AccClose.png'

   Out =     [  f'{wTab}<div class="accordion"> <!-- SectionOpen (Accordion) -->\n' ]
   Out.append(  f'{wTab}   <div class="AccOpenClose trigger {opt}">\n' )
   Out.append(  f'{wTab}      <img class="accordion" src="{src}">\n' )
   Out.append(  f'{wTab}      <p class="accordion">{sText}</p>\n' )
   Out.append(  f'{wTab}   </div>\n' )
   Out.append(  f'{wTab}   <div class="toggle_container"{style}>\n' )
   Out.append(  f'{wTab}      <input class="acc_ret" type="hidden" name="{name}" value="{value}"/>' )
   print ( ''.join( Out ) )

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
def SectionClose( iTab=7 ) :

   wTab = sTab * iTab

   Out  =    [  f'{wTab}   </div> <!-- SectionClose (Accordion) -->\n' ]
   Out.append(  f'{wTab}</div>' )
   print ( ''.join( Out ) )

# -----------------------------------------------------------------------------------
# HTML (Web) colors :
# ->> https://en.wikipedia.org/wiki/Web_colors
def Colorize( sText, sColor='White', iBold=False ):

   if iBold : return f'<span style="color: {sColor}; font-weight: 600;">{sText}</span>'
   else     : return f'<span style="color: {sColor};">{sText}</span>'

# -----------------------------------------------------------------------------------
def divOpenBlk ( iWidth, iHeight, sClass='BlackMonoShell', iTab=7 ) :

   wTab = sTab * iTab

#  Out  = f'{wTab}<div class="{sClass}"><!-- divOpenBlk -->\n'
#  Out += f'{wTab}   <pre class="{sClass}" style="height: {iHeight}px;">'
   Out =     [  f'{wTab}<div class="{sClass}"><!-- divOpenBlk -->\n' ]
   Out.append(  f'{wTab}   <pre class="{sClass}" style="height: {iHeight}px;">' )
   print ( ''.join( Out ) )

# -----------------------------------------------------------------------------------
def divCloseBlk ( iTab=7 ) :

   wTab = sTab * iTab

   Out =     [  f'{wTab}   </pre>\n' ]
   Out.append(  f'{wTab}</div><!-- divCloseBlk -->' )
   print ( ''.join( Out ) )

# ------------------------------------------------------------------------------
# --- Menu : href Icon
# ------------------------------------------------------------------------------
def hrefIcon ( sIcon, sText, sCgi, sPage, opt=False, iTab=7 ) :

   wTab = sTab * iTab
   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]

   if opt : opt = ' Menu_Active'
   else   : opt = ''

   print (f'{wTab}<a href="{sCgi}.cgi?page={sPage}"><img class="Menu_Href{opt}" src="images/{sIcon}" title="{sText}"/></a>' )

# ------------------------------------------------------------------------------
# --- Prints a single "Action" Button
# ------------------------------------------------------------------------------
def pButton( sKey, sText, sRet, opt=False, iSize=80, iTab=7 ):

   wTab = sTab * iTab
   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]

   print (f'{wTab}<button class="SingleButton" style="width: {iSize}px;" name="page" value="{sRet}">{sText}</button>' )
   if opt: Preview ( opt )

# ------------------------------------------------------------------------------
# --- Display a note icon for SVG Prevue
# ------------------------------------------------------------------------------
def Preview( sSvg, iTab=7 ):

   wTab = sTab * iTab

   Out =     [  f'{wTab}<div class="Preview">\n' ]
   Out.append(  f'{wTab}   <img class="PreviewSign" src="images/AvrLogger_16.png"/>\n' )
   Out.append(  f'{wTab}   <div class="PreviewSvg">\n' )
   Out.append(  f'{wTab}      <img class="PreviewSvg" src="images/{sSvg}"/>\n' )
   Out.append(  f'{wTab}   </div>\n' )
   Out.append(  f'{wTab}</div>' )
   print ( ''.join( Out ) )

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
def Hide( sKey, iTab=7 ) :

   wTab = sTab * iTab

   print (f'{wTab}<input type="text" style="display: none;" name="{sKey}" value="{asset.Out[sKey]}"/>' )

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
def refBtn( sText, Width, Icon, sUrl, lWidth=False, opt=False, sPage='Config', iTab=7 ) :

   wTab = sTab * iTab
   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]

   if lWidth : Label   ( '&nbsp;', lWidth )
   if opt : opt = ' subMenuIconActive'
   else   : opt = ''

   Out =     [  f'{wTab}<button class="subMenuIcon{opt}" type="submit" name="page" value="{sUrl}">\n' ]
   Out.append(  f'{wTab}   <img class="subMenuIcon" src="images/{Icon}"/>\n' )
   Out.append(  f'{wTab}   <a class="subMenuIcon" style="width: {Width}px;">{sText}</a>\n' )
   Out.append(  f'{wTab}</button>' )
   print ( ''.join( Out ) )

# ------------------------------------------------------------------------------
# ---
# ------------------------------------------------------------------------------
def hddAlert( sDev, sDict, sName, GuiPush, GuiMail ):

   asset.Out[ 'hddAlert_DsmOn_'  + sDev ] = sDict[ 'DsmOn'  ]
   asset.Out[ 'hddAlert_PushOn_' + sDev ] = sDict[ 'PushOn' ]
   asset.Out[ 'hddAlert_MailOn_' + sDev ] = sDict[ 'MailOn' ]

   RowOpen ()
   Label                ( sName,                              Column_1 )
   CheckBox             ( 'hddAlert_DsmOn_'  + sDev,  'DSM:', 45 )
   if GuiPush: CheckBox ( 'hddAlert_PushOn_' + sDev, 'Push:', 45 )
   if GuiMail: CheckBox ( 'hddAlert_MailOn_' + sDev, 'Mail:', 45 )
   RowClose()

# ------------------------------------------------------------------------------
# ---
# ------------------------------------------------------------------------------
def Alert( sKey, sText, GuiPush, GuiMail ):

   # if asset.Out['Plot' + sKey + 'On'] != 'Yes' : return

   RowOpen ()
   Label                ( sText,                               Column_1 )
   CheckBox             ( f'Alert{sKey}DsmOn',  '&emsp;DSM',   45 )
   if GuiPush: CheckBox ( f'Alert{sKey}PushOn', '&emsp;Push',  45 )
   if GuiMail: CheckBox ( f'Alert{sKey}MailOn', '&emsp;Mail',  45 )
   CheckBox             ( f'Alert{sKey}PtcOn',  '&emsp;Prot.', 45 )
   Input                ( f'Alert{sKey}Max',    '&emsp;Max :', 45, 45 )
   RowClose()
 
# ------------------------------------------------------------------------------
# --- Prints a On / Off-Switch with a Description & Plot-Label / Plot-Color
# ------------------------------------------------------------------------------
def Options( sKey, sText ):

   if not asset.Out[ f'Plot{sKey}On'] : return

   RowOpen   ()
   CheckBox  ( f'Plot{sKey}On',      sText,                                                             145             )
   Input     ( f'Plot{sKey}Label',   'ID',                                                               15,  Box_1a    )
   Color     ( f'Plot{sKey}Color',   '',                                                                  0,  Box_Color )
   SelectGUI ( f'Plot{sKey}Style',   'Style:',         asset.msg['PlotLineStyleGUI'],  asset.Out['PlotLineStyleRAW'],  32,  115       )
   Select    ( f'Plot{sKey}Stroke',  'PlotLineStroke', '10 12 14 16 18 20 25 30 35 40 45 50',            38,   50       )
   Select    ( f'Plot{sKey}Opacity', 'Opacity:',       '0.50 0.60 0.70 0.80 0.90 1.00',                  48,   60       )
   RowClose  ()

# ------------------------------------------------------------------------------
# --- Print a ReadOnly Input-Field with Description
# ------------------------------------------------------------------------------
def RoBox( sKey, sText, dWidth, iWidth, opt=False, iAlert=False, iTab=7, sPage=False ) :

   wTab = sTab * iTab
   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]

   if opt : sClass = 'BoxTextColorRed'    # 'BoxTextColorYellow' # 'RoBoxRed'
   else   : sClass = 'BoxTextColorGray'   # 'RoBoxGray'

   if sText : print ( PrintLabel( sText, dWidth, iTab=iTab ) )

   if sPage : print (f'<a href="AvrConfig.cgi?page={sPage}">' )
   print (f'{wTab}<p class="BoxText {sClass}" style="width: {iWidth}px;">{asset.Out[sKey]}</p>' )
   if sPage : print ( '</a>' )

   if iAlert : Sign ( sKey, opt=True, sHelp=iAlert )
   else      : Sign ( sKey )

# ------------------------------------------------------------------------------
# --- Print a "Blue strong" InfoText ...
# ------------------------------------------------------------------------------
def BlueText( sKey, sText, dWidth, iWidth, opt=False, iAlert=False, iTab=7 ) :

   wTab = sTab * iTab
   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]

   if sText : print ( PrintLabel( sText, dWidth, iTab=iTab ) )

   print (f'{wTab}<p class="InfoTextBlue" style="width: {iWidth}px;">{asset.Out[sKey]}</p>' )
   if iAlert : Sign ( sKey, opt=True, sHelp=iAlert )
   else      : Sign ( sKey )

# ------------------------------------------------------------------------------
# --- Print a Label-Text
# ------------------------------------------------------------------------------
def Label( sText, iWidth, opt='', sAlign='Left', iTab=7 ):

   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]

   if sText : print ( PrintLabel( sText, iWidth ,sAlign, iTab=iTab ) )

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
def PrintLabel( sText, iWidth, sAlign='', iTab=7 ) :

   wTab = sTab * iTab

   if sText.endswith( ':' ) :
      sText = sText.strip()[0:-1]
      sRet  =   f'{wTab}<p class="Label {sAlign}" style="width: {iWidth}px;">{sText}</p>\n'
      sRet +=   f'{wTab}<p class="Label Colon">:</p>'
   else :
      sRet  =   f'{wTab}<p class="Label {sAlign}" style="width: {iWidth}px;">{sText}</p>'

   return sRet

# ------------------------------------------------------------------------------
# --- Call-Back function "onChange" ...
# ------------------------------------------------------------------------------
def OnChange( sPage='', sMore='', opt=False, iTab=3 ):

   wTab = sTab * iTab

   if opt : print (f'{wTab}<input type="hidden" name="page" value="{opt}"/>' )
   else   : print (f'{wTab}<input type="hidden" name="page" value="{sPage}-{sMore}"/>' )

# ------------------------------------------------------------------------------
# --- Display a Icon
# ------------------------------------------------------------------------------
def Icon( sIcon, sHelp=False, iTab=7 ):

   wTab = sTab * iTab

   if sHelp :
      if sHelp in asset.msg: sHelp = asset.msg[ sHelp ]
      sHelp = f' onmouseover="Tip(\'{escape( sHelp )}\')" onmouseout="UnTip()"'
   else     :
      sHelp = ''

   print (f'{wTab}<img class="SmartSign Inline" src="images/{sIcon}"{sHelp}/>' )

# ------------------------------------------------------------------------------
# --- Display a note icon if help is available
# ------------------------------------------------------------------------------
def Sign( sText, opt=False, sHelp=False, iTab=7 ):

   wTab = sTab * iTab
   if not sText in asset.msg and not sHelp or asset.Out['HintModeOn'] != 'Yes' : return

   if not sHelp and sText in asset.msg: sText = asset.msg[ sText ]
   else                               : sText = sHelp

   if opt : sSrc = 'images/Warning.png'
   else   : sSrc = 'images/BlueHint.png'

   print (f'{wTab}<img class="ToolTipSign" src="{sSrc}" onmouseover="Tip(\'{escape( sText)}\')" onmouseout="UnTip()"/>' )

# ------------------------------------------------------------------------------
# --- Trenne einen String ...
# ------------------------------------------------------------------------------
def Split( sText ):

   if '"' in sText : return sText.strip('"').split('" "')
   else            : return sText.split(' ')

# ------------------------------------------------------------------------------
# --- Print a User friendly Select-Box (Combo-Box)
# ------------------------------------------------------------------------------
def SelectGUI( sKey, sText, sGui, sRaw, dWidth, sWidth, opt=False, iOff='', Row=False ):

   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]

   if opt: onChange = 'onChange="this.form.submit()"'
   else:   onChange = ''

   if iOff : iOff  = f' margin-left: {iOff}px;'

   Item = dict()
   Item[ asset.Out[ sKey ] ]='selected'

   sel = Split ( sRaw )
   GUI = Split ( sGui )

   if sText :
      if sText.endswith( ':' ) :
         iColon = True
         sText = sText.strip()[0:-1]
      else :
         iColon = False
      out           = f'                           <p class="Label" style="width: {dWidth}px;">{sText}</p>\n'
      if iColon :
         out       += f'                           <p class="Label Colon">:</p>\n'
   else :
      out           =  ''

   out             += f'                           <select class="SelectBox" style="width: {sWidth}px;" name="{sKey}" {onChange}>\n'

   for i in range(0, len( sel )):
      try    : out += f'                              <option value="{sel[i]}" {Item[sel[i]]}>{GUI[i].replace("   ", "&nbsp;&nbsp;&nbsp;")}</option>\n'
      except : out += f'                              <option value="{sel[i]}">{GUI[i].replace("   ", "&nbsp;&nbsp;&nbsp;")}</option>\n'

   out             += f'                           </select>'
   print ( out )

   Sign ( sKey )

# ------------------------------------------------------------------------------
# --- Print a Select-Box (Combo-Box)
# ------------------------------------------------------------------------------
def Select( sKey, sText, sList, dWidth, sWidth, opt=False, iOff='', Row=False ):
   Out = asset.Out
   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]

   if opt: onChange = 'onChange="this.form.submit()"'
   else:   onChange = ''

   if iOff : iOff   = f'margin-left: {iOff}px;'

   Item                     = dict()
   Item[asset.Out[ sKey ] ] ='selected'
   sel                      = Split ( sList )

   if Row : RowOpen()

   if sText :
      if sText.endswith( ':' ) :
         iColon = True
         sText = sText.strip()[0:-1]
      else :
         iColon = False
      out = f'                           <p class="Label" style="width: {dWidth}px;">{sText}</p>\n'
      if iColon :
         out += f'                           <p class="Label Colon">:</p>\n'
   else :
      out  =  ''


   out += f'                           <select class="SelectBox" style="width: {sWidth}px;" name="{sKey}" {onChange}>\n'

   for i in range(0, len( sel )):
      try    : out += f'                              <option value="{sel[i]}" {Item[sel[i]]}>{sel[i]}</option>\n'
      except : out += f'                              <option value="{sel[i]}">{sel[i]}</option>\n'

   out +=  '                           </select>'
   print ( out )

   Sign ( sKey )

   if Row : RowClose()

# ------------------------------------------------------------------------------
# --- Print a Input-Field with Description
# ------------------------------------------------------------------------------
def Input( sKey, sText, dWidth, sWidth, opt=None, sHelp=False, iTab=7 ):

   wTab = sTab * iTab
   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]

   if sText : print ( PrintLabel( sText, dWidth, iTab=iTab ) )

   print (f'{wTab}<input class="Input" type="text" style="width: {sWidth}px;" placeholder="{asset.Out[sKey]}" name="{sKey}" value="{asset.Out[sKey]}"/>' )
   if opt: Sign ( opt, True )
   else:   Sign ( sKey, sHelp=sHelp )

# ------------------------------------------------------------------------------
# --- Print a Input-Field with Description and "onChange"
# ------------------------------------------------------------------------------
def InputOc( sKey, sText, dWidth, iWidth, opt=False, iTab=7 ) :

   wTab = sTab * iTab
   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]

   if sText : print ( PrintLabel( sText, dWidth, iTab=iTab ) )

   print (f'{wTab}<input class="Input" type="text" style="width: {iWidth}px;" placeholder="{asset.Out[sKey]}" name="{sKey}" value="{asset.Out[sKey]}" onChange="this.form.submit()"/>' )
   if opt: Sign ( opt, True )
   else  : Sign ( sKey )

# ------------------------------------------------------------------------------
# --- Print a Pasword Input-Field with Description
# ------------------------------------------------------------------------------
def Password( sKey, sText, dWidth, iWidth, sHelp=False, iTab=7 ):

   wTab = sTab * iTab
   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]

   if sText : print ( PrintLabel( sText, dWidth, iTab=iTab ) )

   print (f'{wTab}<input class="Input" type="password" autocomplete="off" style="width: {iWidth}px;" placeholder="{asset.Out[sKey]}" name="{sKey}" value="{asset.Out[sKey]}"/>' )
   Sign ( sKey, sHelp=sHelp )

# ------------------------------------------------------------------------------
# --- Prints a single "Action" Button
# ------------------------------------------------------------------------------
def Button( sKey, sText, sRet, opt=False, iSize=80, iTab=7 ):

   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]

   wTab = sTab * iTab
   print (f'{wTab}<button class="SingleButton" style="width: {iSize}px;" name="page" value="{sRet}">{sText}</button>' )

   if opt: Sign ( opt, True )
   else:   Sign ( sKey )

# ------------------------------------------------------------------------------
# --- Prints a single Button in the right corner below the Page
# ------------------------------------------------------------------------------
def Button1( sButton, vButton, iSize=80, iTab=7 ):

   wTab = sTab * iTab

   Out  = f'{wTab}<div style="float: right;">\n'
   Out += f'{wTab}   <button style="width: {iSize}px;" name="page" value="{vButton}">{sButton}</button>\n'
   Out += f'{wTab}</div>'
   print ( Out )

# ------------------------------------------------------------------------------
# --- Prints a single Button in the "left" corner below the Page
# ------------------------------------------------------------------------------
def Button1_l( sButton, vButton, sClass='GrayButton', iSize=80, iTab=7 ):

   wTab = sTab * iTab

   print (f'{wTab}<button class="{sClass}" style="width: {iSize}px;" name="page" value="{vButton}">{sButton}</button>' )

# ------------------------------------------------------------------------------
# --- Prints 2 Button in the right corner below the Page
# --- Button 1 = Button-Text  for "save"
# --- Button 1 = Return-Value for "save"
# --- Button 2 = Button-Text  for "undo"
# --- Button 2 = Return-Value for "undo"
# ------------------------------------------------------------------------------
def Button2( sButton1, vButton1, sButton2, vButton2, iSize=80, iTab=7 ):

   wTab = sTab * iTab

   Out =     [  f'{wTab}<div class="ButtonFooter InlineRight">\n' ]
   Out.append(  f'{wTab}   <p class="ButtonFooter"></p>\n' )
   Out.append(  f'{wTab}   <button class="BlueButton" style="width: {iSize}px;" name="page" value="{vButton1}">{sButton1}</button>\n' )
   Out.append(  f'{wTab}   <button class="GrayButton" style="width: {iSize}px;" name="page" value="{vButton2}">{sButton2}</button>\n' )
   Out.append(  f'{wTab}</div>' )
   print ( ''.join( Out ) )

# ------------------------------------------------------------------------------
# --- Display "Save" & "Undo" Button's at Bottom ...
# ------------------------------------------------------------------------------
def SaveUndo( PageName, sFunc, iSize=80, iHight=40, iWidth=797, iTab=7 ) :

   vButton1 = f'{PageName}-{sFunc}+save'
   vButton2 = f'{PageName}-{sFunc}'
   wTab = sTab * iTab

   Out =     [  f'{wTab}<div class="ContentRow"> <!-- SaveUndo -->\n' ]
   Out.append(  f'{wTab}   <div class="BlueBorder InlineLeft" style="width: {iWidth}px; height: {iHight}px">\n' )
   Out.append(  f'{wTab}      <div class="ButtonFooter InlineRight">\n' )
   Out.append(  f'{wTab}         <p class="ButtonFooter"></p>\n' )
   Out.append(  f'{wTab}         <button class="BlueButton InlineRight" style="width: {iSize}px;" name="page" value="{vButton1}">Save</button>\n' )
   Out.append(  f'{wTab}         <button class="GrayButton InlineRight" style="width: {iSize}px;" name="page" value="{vButton2}">Undo</button>\n' )
   Out.append(  f'{wTab}      </div>\n' )
   Out.append(  f'{wTab}   </div>\n' )
   Out.append(  f'{wTab}</div>' )
   print ( ''.join( Out ) )

# ------------------------------------------------------------------------------
# --- Print a Color-Picker Input-Field with Description
# ------------------------------------------------------------------------------
def Color( sKey, sText, dWidth, iWidth, iTab=7, Row=False ):

   wTab = sTab * iTab
   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]
   if Row   : RowOpen()
   if sText : print ( PrintLabel( sText, dWidth, iTab=iTab ) )

   
   print(f'{wTab}<input style="width: {iWidth}px;" name="{sKey}" value="{asset.Out[sKey]}" class="jscolor">' )
   Sign ( sKey )
   if Row : RowClose()

# ------------------------------------------------------------------------------
# --- Print a single CheckBox
# ------------------------------------------------------------------------------
def CheckBox ( sKey, sText, iWidth, opt=False, sHelp=False, iTab=7, Row=False ) :

   wTab = sTab * iTab
   if 'dec_' + sText in asset.msg: sText = asset.msg['dec_' + sText ]

   if opt: onChange = 'onChange="this.form.submit()"'
   else:   onChange = ''

   if 'Yes' in asset.Out[ sKey ] : sChecked = 'checked'
   else                          : sChecked = ''

   if Row : RowOpen()
   if sText : print ( PrintLabel( sText, iWidth ) )

   Out =     [  f'{wTab}<label class="CheckBox">\n' ]
   Out.append(  f'{wTab}   <input type="hidden" name="{sKey}" value="Switch"/>\n' )
   Out.append(  f'{wTab}   <input class="CheckBox" type="checkbox" {sChecked} name="{sKey}" value="Yes" {onChange}>\n' )
   Out.append(  f'{wTab}   <span class="CheckBoxMark"></span>\n' )
   Out.append(  f'{wTab}</label>' )
   print ( ''.join( Out ) )

   Sign ( sKey, sHelp=sHelp )
   if Row : RowClose()

# ------------------------------------------------------------------------------
def RowOpen( sClass='Row', iTab=7 ) :

   wTab = sTab * iTab
   print (f'{wTab}<div class="{sClass} InlineLeft">' )

# ------------------------------------------------------------------------------
def RowClose( iTab=7 ):

   wTab = sTab * iTab
   print (f'{wTab}</div>' )

# ------------------------------------------------------------------------------
# --- Prints a Chapter / section heading in DSM-Blue
# ------------------------------------------------------------------------------
def Section( sText, iOff='', iTab=7 ):

   wTab = sTab * iTab
   if 'sec_' + sText in asset.msg: sText = asset.msg['sec_' + sText ]

   print (f'{wTab}<p class="Section">{sText}</p>' )

# ------------------------------------------------------------------------------
def PageRefresh( page, sCgi='AvrConfig' ):

   print ( f'<meta http-equiv="refresh" content="0; url={sCgi}.cgi?page={escape( page )}">', flush=True )

# -----------------------------------------------------------------------------------
# --- Call (refresh) a Page (url) with additional Parameters ...
# -----------------------------------------------------------------------------------
def CallPage( sDict, sCgi='AvrConfig' ):

   url   = f'{sCgi}.cgi?'
   parts = list( urlparse.urlparse( url ) )

   query = dict( urlparse.parse_qsl( parts[4] ) )
   query.update( sDict )

   parts[4] = urlencode( query )

   url = urlparse.urlunparse( parts )

   print ( f'<meta http-equiv="refresh" content="0; url={url}">', flush=True )

# --------------------------------------------------------------------------
# --- Display a Message DSM like ---
# --------------------------------------------------------------------------
def jBusy( sText, iTab=1, iFlush=False ):

   wTab = sTab * iTab
   if sText in asset.msg: sText = asset.msg[ sText ]

   Out =     [  f'{wTab}<div id="rtfBusyLoad" class="Message">\n' ]
   Out.append(  f'{wTab}   <table class="Message">\n' )
   Out.append(  f'{wTab}      <tbody class="Message">\n' )
   Out.append(  f'{wTab}         <tr>\n' )
   Out.append(  f'{wTab}            <th class="MessageTab1">\n' )
   Out.append(  f'{wTab}               <img class="Message" src="images/busy.gif">\n' )
   Out.append(  f'{wTab}            </th>\n' )
   Out.append(  f'{wTab}            <th class="MessageTab2">\n' )
   Out.append(  f'{wTab}               <p class="Message">{sText}</p>\n' )
   Out.append(  f'{wTab}            </th>\n' )
   Out.append(  f'{wTab}         </tr>\n' )
   Out.append(  f'{wTab}      </tbody>\n' )
   Out.append(  f'{wTab}   </table>\n' )
   Out.append(  f'{wTab}</div>' )
   print ( ''.join( Out ), flush=iFlush )

# --------------------------------------------------------------------------
# --- Display a Message DSM like ---
# --------------------------------------------------------------------------
def Busy( sText, iTab=7 ):

   wTab = sTab * iTab
   if sText in asset.msg: sText = asset.msg[ sText ]

   Out =     [  f'{wTab}<div class="Message">\n' ]
   Out.append(  f'{wTab}   <div>\n' )
   Out.append(  f'{wTab}      <center>\n' )
   Out.append(  f'{wTab}         <table id="system_msg" style="width: 350px; table-align: center;">\n' )
   Out.append(  f'{wTab}            <tr>\n' )
   Out.append(  f'{wTab}               <th style="width: 20%;"><img src="images/busy.gif" style="float: left; height: 30px;"></th>\n' )
   Out.append(  f'{wTab}               <th style="width: 80%;"><p class="center"><span style="color: #424242; font-weight: normal;">{sText}</span></p></th>\n' )
   Out.append(  f'{wTab}            </tr>\n' )
   Out.append(  f'{wTab}         </table>\n' )
   Out.append(  f'{wTab}      </center>\n' )
   Out.append(  f'{wTab}   </div>\n' )
   Out.append(  f'{wTab}</div>' )
   print ( ''.join( Out ) )

# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
def rtfBusyLoadWait() :

   print ( '<script>rtfBusyLoadWait();</script>' )

# --------------------------------------------------------------------------
# --- Init CGI HTML-Page ...
# --------------------------------------------------------------------------
def OpenPage( sTitel=asset.SpkName, sMethod='POST', sCgi='AvrConfig' ) :

   Out  =    [   'Content-type: text/html\n' ]
   Out.append(   '\n' )
   Out.append(   '<!DOCTYPE html>\n' )
   Out.append(   '   <html xmlns="http://www.w3.org/1999/xhtml">\n' )
   Out.append(   '      <head>\n' )
   Out.append(  f'         <title> -- {sTitel} -- </title>\n' )
   Out.append(   '         <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>\n' )
   Out.append(   '         <meta name="viewport" content="width=device-width, initial-scale=1">\n' )
   Out.append(  f'         <link rel="shortcut icon" href="images/{asset.SpkName}_32.png" type="image/x-icon"/>\n' )
   Out.append(  f'         <link rel="stylesheet" type="text/css" href="css/Normalize.css?v={asset.cssVersion}"/>\n' )
   Out.append(  f'         <link rel="stylesheet" type="text/css" href="css/Style.css?v={asset.cssVersion}"/>\n' )
   Out.append(  f'         <link rel="stylesheet" type="text/css" href="css/Scrollbars.css?v={asset.cssVersion}"/>\n' )
   Out.append(  f'         <script src="js/AvrLib.js?v={asset.cssVersion}"></script>\n' )
   Out.append(  f'         <script src="js/jQuery.js?v={asset.cssVersion}"></script>\n' )
   Out.append(  f'         <script src="js/jscolor.js?v={asset.cssVersion}"></script>\n' )
   Out.append(  f'         <script src="js/Scrollbar.js?v={asset.cssVersion}"></script>\n' )
   Out.append(   '         <script>jQuery(document).ready(function(){{ jQuery(\'.ScrollBar\').scrollbar(); }});</script>\n' )
   Out.append(   '      </head>\n' )
   Out.append(   '      <body class="Chart">\n' )
   Out.append(   '         <script>$(document).ready( rtfAccordion );</script>\n' )
   Out.append(   '         <script>$(document).ready( rtfBusyLoadReady );</script>\n' )
   Out.append(   '         <script src="js/Tooltip.js"></script>\n' )
   Out.append(  f'         <form action="{sCgi}.cgi" method="{sMethod}" autocomplete="on">' )
   print ( ''.join( Out ) )

# -----------------------------------------------------------------------------------
def MainConfigOpen( sText, iTab=4 ) :

   wTab = sTab * iTab
   if 'sec_' + sText in asset.msg: sText = asset.msg['sec_' + sText ]

   Out =     [  f'{wTab}<fieldset class="Chart">\n' ]
   Out.append(  f'{wTab}   <div class="GridWrapper">\n' )
   Out.append(  f'{wTab}      <div class="BlueBorder BorderHeightFull">\n' )
   Out.append(  f'{wTab}         <div class="MainConfig">\n' )
   Out.append(  f'{wTab}            <p class="Section">{sText}</p>\n' )
   Out.append(  f'{wTab}            <div class="InlineLeft MenuIcon">' )
   print ( ''.join( Out ) )

# -----------------------------------------------------------------------------------
def SubConfigOpen( sText, iTab=5 ) :

   wTab = sTab * iTab
   if 'sec_' + sText in asset.msg: sText = asset.msg['sec_' + sText ]

   Out  =    [  f'{wTab}   </div> <!-- SubConfigOpen -->\n' ]
   Out.append(  f'{wTab}</div>\n' )
   Out.append(  f'{wTab}<div class="SubConfig">\n' )
   Out.append(  f'{wTab}   <p class="Section">{sText}</p>\n' )
   Out.append(  f'{wTab}   <div id="ShareScroll" class="ScrollBar OverFlow-Y">' )
   print ( ''.join( Out ) )

# -----------------------------------------------------------------------------------
def SubConfigClose( iTab=4 ) :

   wTab = sTab * iTab
   Out =     [  f'{wTab}         <script>rtfScrollDivTo( "ShareScroll", "subMenuIconActive" )</script>\n' ]
   Out.append(  f'{wTab}      </div> <!-- SubConfigClose -->\n' )
   Out.append(  f'{wTab}   </div>\n' )
   Out.append(  f'{wTab}</div>' )
   print ( ''.join( Out ) )

# -----------------------------------------------------------------------------------
def ContentOpen( iTab=4 ) :

   wTab = sTab * iTab
   print (f'{wTab}<div class="Content"> <!-- ContentOpen -->' )

# -----------------------------------------------------------------------------------
def ContentClose( iTab=4 ) :

   wTab = sTab * iTab
   print (f'{wTab}</div> <!-- ContentClose -->' )

# ------------------------------------------------------------------------------
def ContentRow( iTab=5 ) :

   wTab = sTab * iTab
   print (f'{wTab}<div class="ContentRow"> <!-- ContentRowOpen -->' )

# ------------------------------------------------------------------------------
def ContentRowEnd( iTab=5 ) :

   wTab = sTab * iTab
   print (f'{wTab}</div> <!-- ContentRowClose -->' )

# ------------------------------------------------------------------------------
def ContentTab( sWidth=200, sHight=False, iTab=7 ) :

   wTab = sTab * iTab
   if sHight : print (f'{wTab}<div class="BlueBorder InlineLeft" style="width: {sWidth}px; height: {sHight}px;"> <!-- ContentTabOpen -->' )
   else      : print (f'{wTab}<div class="BlueBorder InlineLeft" style="width: {sWidth}px;"> <!-- ContentTabOpen -->' )

# ------------------------------------------------------------------------------
def ContentTabEnd( iTab=7 ) :

   wTab = sTab * iTab
   print (f'{wTab}</div> <!-- ContentTabClose -->' )

# ------------------------------------------------------------------------------
def ClosePage() :

   Out =     [   '            </div> <!-- ClosePage -->\n' ]
   Out.append(   '         </fieldset>\n' )
   Out.append(   '      </form>\n' )
   Out.append(   '   </body>\n' )
   Out.append(   '</html>' )
   print ( ''.join( Out ) )

# ------------------------------------------------------------------------------
def FullPageOpen( iTab=2 ) :

   wTab = sTab * iTab
   Out =     [  f'{wTab}<fieldset class="OpenFullPage">\n' ]
   Out.append(  f'{wTab}   <div class="OpenFullPage">' )
   print ( ''.join( Out ) )

# ------------------------------------------------------------------------------
# --- Print Error-Page ...
# ------------------------------------------------------------------------------
def NoPermissions():

   import Cgi.debug as Debug

   s = lib.InitLanguage( 'admin', asset.avrLangFile )

   Out  =  'Content-type: text/html\n'
   Out +=  '\n'
   Out +=  '<!DOCTYPE html>\n'
   Out +=  '<html xmlns="http://www.w3.org/1999/xhtml">\n'
   Out +=  '   <head>\n'
   Out +=  '      <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>\n'
   Out +=  '      <meta name="viewport" content="width=device-width, initial-scale=1">\n'
   Out += f'      <link rel="shortcut icon" href="images/{asset.SpkName}_32.png" type="image/x-icon"/>\n'
   Out += f'         <link rel="stylesheet" type="text/css" href="css/Normalize.css?v={asset.cssVersion}"/>\n'
   Out += f'         <link rel="stylesheet" type="text/css" href="css/Style.css?v={asset.cssVersion}"/>\n'
   Out += f'         <link rel="stylesheet" type="text/css" href="css/Scrollbars.css?v={asset.cssVersion}"/>\n'
   Out += f'         <script src="js/jQuery.js?v={asset.cssVersion}"></script>\n'
   Out += f'         <script src="js/Scrollbar.js?v={asset.cssVersion}"></script>\n'
   Out +=  '         <script>jQuery(document).ready(function(){{ jQuery(\'.ScrollBar\').scrollbar(); }});</script>\n'
   Out +=  '   </head>\n'
   Out +=  '   <body class="Permission">\n'
   Out +=  '      <div  class="PermissionHeader">\n'
   Out +=  '         <div style="width: 210px;height: 170px;">\n'
   Out +=  '            <img class="Permission" src="images/forbidden.png" alt="forbidden">\n'
   Out +=  '         </div>\n'
   Out +=  '         <div style="width: 100%; height: 170px;">\n'
   Out += f'            <h1 class="Permission">{s["CgiErrPermissions"]}</h1>\n'
   Out += f'            <h2 class="Permission">{s["CgiErrAdministrator"]}</h2>\n'
   Out += f'            <h2 class="Permission">{s["CgiErrRealAdmin"]}</h2>\n'
   Out +=  '            <h2 class="Permission"><br></h2>\n'
   Out +=  '         </div>\n'
   Out +=  '      </div>\n'
   Out +=  '      <hr class="Permission">\n'
   Out += f'      <p class="Permission">{s["CgiErrGetHelp"]}&emsp;\n'
   Out +=  '         <a href="%s" target="_blank">%s</a>\n' % ( s["CgiErrGetHelpUrl"], s["CgiErrGetHelpUrlText"] )
   Out +=  '      </p>\n'
   Out +=  '      <hr class="Permission">\n'

   Out +=  '      <fieldset class="OverFlowDebug">\n'
   Out +=  '<pre class="OverFlowDebug">\n'

   Out +=  '<div id="ShareScroll" class="ScrollBar OverFlowDebug"> <!-- Start DivScroll -->\n'
   print ( Out )
   # ---------------------------------------------------------------------------
   Debug.Content( True )
   # ---------------------------------------------------------------------------
   print ( '               </div> <!-- End DivScroll -->' )
   print ( '            </pre>' )
   print ( '      </fieldset>' )
   print ( '   </body>' )
   print ( '</html>' )
   # ---------------------------------------------------------------------------

# --------------------------------------------------------------------------
# --- Disply "WhatsNew"-Page ...
# --------------------------------------------------------------------------
def WhatsNew( sCgi='AvrConfig' ):

   import __main__ as pMain

   # -----------------------------------------------------------------------
   if os.path.exists( asset.f_AvrDisabled ):
      iID = 0
   else:
      if pMain.AvrPid : iID = 1
      else            : iID = 2

   sIconNice = json.loads( asset.msg['nacCgiStausIcn'] )
   sIcon     = asset.cgiNavIcon[iID]
   sText     = sIconNice[ str(iID) ]

   WhatsNew = lib.InitLanguage( asset.sysCookies['SynoUser'], asset.avrWhatsNewFile, suffix='.html', opt=True )

   onClick  = 'onClick="return window.location.reload();"'

   print ( dedent( f'''\
      Content-type: text/html

      <!DOCTYPE html>
      <html xmlns="http://www.w3.org/1999/xhtml">
         <head>
            <title> -- {asset.SpkName}@Changelog -- </title>
            <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
            <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
            <link rel="shortcut icon" href="images/{asset.SpkName}_32.png" type="image/x-icon"/>
            <link rel="stylesheet" type="text/css" href="css/News.css?v={asset.cssVersion}"/>
            <link rel="stylesheet" type="text/css" href="../../help/help.css?v={asset.cssVersion}"/>
            <link rel="stylesheet" type="text/css" href="../../help/scrollbar/flexcroll.css"/>
            <script type="text/javascript" src="../../help/scrollbar/flexcroll.js"></script>
            <script type="text/javascript" src="../../help/scrollbar/initFlexcroll.js"></script>
         </head>
         <body style="margin-left:0px; margin-top:0px;">
            <div class="callConfig" id="Sticky">
               <img id="callConfig" class="callConfig" href="{sCgi}.cgi?page=status" src="images/{sIcon}" title="{sText}" {onClick}/>
            </div>
            <div class="content">''' ) )

   # --- Print content of File ...
   print ( indent( WhatsNew, prefix='         ' ) )

   print ( dedent( f'''\
            </div>
         </body>
      </html>''' ))

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
