# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import asset
from   Html  import *

from   lib   import switch, case

# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
import __main__  as pMain

FuncName = 'celsius'

FuncDesc = { 'IconName'     : 'Umwelt.png',
             'SubName'      : asset.msg['modSourcesEnvName'],
             'SubText'      : asset.msg['modSourcesEnvText'],
             'iWidth'       : SubMenuWidth,
             'iOnChange'    : f'{asset.PageName}-{FuncName}+OnChange',
           }

# -----------------------------------------------------------------------------------
# --- Pre-Initialization
# -----------------------------------------------------------------------------------
def pre( site, more, sub=False ) :

   while switch( f'{site}+{more}' ) :
      # -----------------------------------------------------------------------------
      if case( 'celsius+save' ) :
         # --------------------------------------------------------------------------
         import main
         main.save()
         break

      # -----------------------------------------------------------------------------
      break

# -----------------------------------------------------------------------------------
# --- Post-Initialization
# -----------------------------------------------------------------------------------
def post( site, more, sub=False ) :

   while switch( f'{site}+{more}' ) :
      # -----------------------------------------------------------------------------
      # --- "OnChange" CallBack-Function ...
      if case( 'celsius+OnChange' ) :
         break

      # -----------------------------------------------------------------------------
      if case( 'celsius+query' ) :
         jBusy ( 'msgCgiWaitInfo', iFlush=True )

         import Snmp
         sRet = Snmp.Test( asset.cgiInput )
         try    : asset.cgiInput[ 'CelsiusQueryRet' ] += sRet + '<br>'
         except : asset.cgiInput[ 'CelsiusQueryRet' ]  = sRet + '<br>'

         break

      # -----------------------------------------------------------------------------
      break

   # --------------------------------------------------------------------------------
   # --- Prepare Values ...
   for sKey in asset.cgiInput :
      asset.Out[ sKey] = asset.cgiInput[ sKey ]

# -----------------------------------------------------------------------------------
# --- Function : "engine"
# -----------------------------------------------------------------------------------
def Func ( sSite, sMore ) :

   iWidth   = iMainOutWidth
   iHeight  = iMainHeight

   s           = asset.msg
   Out         = asset.Out

   # --------------------------------------------------------------------------------
   ContentOpen()
   # --------------------------------------------------------------------------------
   ContentRow ()
   ContentTab ( 315, 260 )
   # --------------------------------------------------------------------------------

   Section ( 'Environmental' )

   for index, sID in enumerate( asset.sTabEnv ):
      try    : GUI += ' "Sensor {0}"'.format( index + 1 )
      except : GUI  =  '"Sensor {0}"'.format( index + 1 )

      try    : RAW += ' {0}'.format( index )
      except : RAW  =  '{0}'.format( index )

   sID = asset.sTabEnv[ int( Out['LogEnvCurrent'] ) ]

   RowOpen       ()
   SelectGUI     ( 'LogEnvCurrent',          'LogEnvCurrent', GUI, RAW, Column_2a, 105, True )
   CheckBox      ( 'LogEnvOn' + sID,         '',                  0, True, sHelp=s['LogEnvOn'] )
   RowClose()

   if Out['LogEnvOn' + sID] == "Yes" :
      RowOpen    ()
      Input      ( 'LogEnvName' + sID,       'LogEnvName',       Column_2a, 186, sHelp=s['LogEnvName'] )
      RowClose   ()

      RowOpen    ()
      Input      ( 'LogEnvIp' + sID,         'IP :',             Column_2a,  186, sHelp=s['LogEnvIp'] )
      RowClose   ()

      RowOpen    ()
      Input      ( 'LogEnvOID' + sID,        'OID:',             Column_2a,  186 )
      RowClose   ()

      RowOpen    ()
      Select     ( 'LogEnvScale' + sID,      'LogEnvScale',      '1 10',     Column_2a,   63 )
      RowClose   ()

      if Out['LogEnvV3On' + sID] == "No" :
         RowOpen ()
         Input   ( 'LogEnvCom' + sID,        'Community :',      Column_2a,  186 )
         RowClose()
      else :
         RowOpen ()
         Input   ( 'LogEnvUser' + sID,       'LogEnvUser',       Column_2a,  186 )
         RowClose()

         RowOpen ()
         Password( 'LogEnvPass' + sID,       'LogEnvPass',       Column_2a,  186, sHelp=s['LogEnvPass'] )
         RowClose()

      RowOpen    ()
      CheckBox   ( 'LogEnvV3On' + sID,       'LogEnvV3On',       Column_2a, True )
      if Out['LogEnvV3On' + sID] == "Yes" :
         Select  ( 'LogEnvEncryption' + sID, 'LogEnvEncryption', 'MD5 SHA',  65, 63 )
      RowClose()

      RowOpen    ()
      Label      ( s['dec_EnvQueryResult'], Column_2a )
      Button     ( 'ButtonQueryEnv',  '- Test -', f'{asset.PageName}-{FuncName}+query', iSize=80 )
      RowClose   ()

   ContentTabEnd()

   # --------------------------------------------------------------------------------
   ContentTab( 465, 260 )
   # --------------------------------------------------------------------------------
   iOutWidth   = iWidth  - 343
   iOutHeight  = iHeight - 382

   Section ( 'EnvQueryResult' )
   divOpenBlk  ( iOutWidth, iOutHeight )

   sRet = Out.get( 'CelsiusQueryRet', '' )
   print ( sRet )

   divCloseBlk ()

   ContentTabEnd()
   ContentRowEnd()

   # --------------------------------------------------------------------------------
   # --- Display "Save" & "Undo" Button ...
   SaveUndo( asset.PageName, FuncName, iSize=80, iWidth=iMainOutWidth )
   # --------------------------------------------------------------------------------
   asset.Out['CelsiusQueryRet'] = escape( sRet )

   # --------------------------------------------------------------------------------
   Hide ( 'CelsiusQueryRet' )

   # --------------------------------------------------------------------------------
   ContentClose()

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
