# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import os
import shutil
import sys
import time
import glob
import pwd

import asset
import basics  as io
import lib

import Svg

# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
os.copy = shutil.copy

# -----------------------------------------------------------------------------------
# --- Save all current data / settings in the backup folder ...
# -----------------------------------------------------------------------------------
def RestoreSettings( sPath ) :

   arr = lib.FindeShare( sPath )

   if not arr :
      lib.Error ( "GUI: Fatal Error: \"" + sPath + "\" not found" )
      return False

   ImportPath = '/'.join( arr )

   sName = 'error'
   sFile = '/'.join(( ImportPath, sName ))

   # --------------------------------------------------------------------------------
   if os.path.exists( sFile ) :
      sErr = io.ioRead( asset.ETC + '/log/' + sName )
      os.copy ( sFile,  asset.ETC + '/log/' + sName )
      io.ioAdd( sErr,   asset.ETC + '/log/' + sName )
      lib.Error ( "GUI: Restored successfully: \"" + sName + "\"" )

   # --------------------------------------------------------------------------------
   if os.path.exists( ImportPath + '/tmp.svg' ) :
      iList = glob.glob( asset.LineChart + '/tmp_*.svg' )
      for sID in iList :
         io.ioRm( sID )

   sName = 'tmp_' + str( int( time.time() ) ) + '.svg'

   RestoreGoCopy ( asset.LineChart,       'tmp.svg',       ImportPath, opt=sName )
   RestoreGoCopy ( asset.ETC + "/tmp",    "AvrLogger.dat", ImportPath )
   RestoreGoCopy ( asset.ETC + "/log",    "enumDrives",    ImportPath )
   RestoreGoCopy ( asset.ETC + "/log",    "enumStorage",   ImportPath )

   return ImportPath

# -----------------------------------------------------------------------------------
def RestoreGoCopy( sDest, sName, sPath, opt=False ) :

   if not os.path.exists( sPath + "/" + sName ) :
      return

   if not opt : dest = sName
   else :       dest = opt

   sFile = '/'.join(( sPath, sName ))
   dFile = '/'.join(( sDest, dest  ))

   io.ioCopy ( sFile, dFile )

   lib.Error ( "GUI: Restored successfully: \"" + sName + "\"" )

# -----------------------------------------------------------------------------------
# --- Save all current data / settings in the backup folder ...
# -----------------------------------------------------------------------------------
def BackupSettings( sPath, sUser='admin' ) :

   arr = lib.FindeShare( sPath )

   if not arr :
      lib.Error ( "GUI: Fatal Error: \"" + sPath + "\" not found" )
      return

   try :
      iUID = pwd.getpwnam( sUser ).pw_uid
      iGID = pwd.getpwnam( sUser ).pw_gid
   except :
      lib.Error ( "GUI: Fatal Error: \"" + sUser + "\" not found" )
      return

   ExportPath = '/'.join( arr )
   sName      = time.strftime('%Y.%m.%d - %H.%M.%S AvrLogger.cnf')

   tmpSVG = glob.glob( asset.LineChart + '/tmp_*.svg' )
   if tmpSVG : tmpSVG = tmpSVG[-1].split( '/' )[-1]
   else      : tmpSVG ='NA'

   BackupGoCopy ( asset.LineChart,       tmpSVG,          ExportPath, iUID, iGID, opt='tmp.svg' )
   BackupGoCopy ( asset.ETC + "/config", "log.conf",      ExportPath, iUID, iGID, opt=sName )
   BackupGoCopy ( asset.ETC + "/config", "log.conf",      ExportPath, iUID, iGID )
   BackupGoCopy ( asset.ETC + "/tmp",    "AvrLogger.dat", ExportPath, iUID, iGID )
   BackupGoCopy ( asset.ETC + "/log",    "enumDrives",    ExportPath, iUID, iGID )
   BackupGoCopy ( asset.ETC + "/log",    "enumStorage",   ExportPath, iUID, iGID )
   BackupGoCopy ( asset.ETC + "/log",    "error",         ExportPath, iUID, iGID )


   Svg.PreView( pre=os.path.join( ExportPath, sName + '.svg' ) )

   # --------------------------------------------------------------------------------
   # --- Move User-Configs from Template-Folder to Archive-Folder ...
   for sFile in sorted( glob.glob( asset.avrPresetFiles + '/Config-?.cnf' ) ) :

      sName = os.path.basename( sFile )
      sDest = os.path.join( ExportPath, sName)

      io.ioMove ( sFile, sDest )

      os.chown  ( dFile, iUID, iGID )
      os.chmod  ( sDest, 0o755 )

      # -----------------------------------------------------------------------------
      lib.Error ( 'GUI: Moved successfully: "{0}" to "{1}"'.format( sName, '/'.join(( arr[1], sName ))  ) )

   # --------------------------------------------------------------------------------

# -----------------------------------------------------------------------------------
def BackupGoCopy( sPath, sName, sDest, iUID, iGID, opt=False ):

   if not os.path.exists( sPath + "/" + sName ) :
      lib.Error ( "GUI: Error: \"" + sName + "\" not found" )
      return
   
   if not opt : dest = sName
   else :       dest = opt

   sFile = '/'.join(( sPath, sName ))
   dFile = '/'.join(( sDest, dest  ))

   lib.Error ( "GUI: Saved successfully: \"" + dest + "\"" )

   os.copy  ( sFile, dFile )
   os.chown ( dFile, iUID, iGID )
   os.chmod ( dFile, 0o755 )

