# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#
import os
import time
import subprocess

import asset
import basics   as io

# -----------------------------------------------------------------------------------
# --- Asynchronous task to get OID-Values.
# --- Data is written as it becomes available.
# -----------------------------------------------------------------------------------
def Pick( iLoop, Conf ):

   # ---  0 = Environmental sensor 1
   # ---  1 = Environmental sensor 2
   # ---  2 = Environmental sensor 3
   # ---  3 = Environmental sensor 4
   # ---  4 = Environmental sensor 5
   # ---  5 = Thermal-Status
   # ---  6 = HDD Max. temperature
   # ---  7 = HDD Min. temperature
   # ---  8 = Average temperature
   # ---  9 = MemInfo["Cached"]
   # --- 10 = Speedtest "UploadMiB"
   # --- 11 = Speedtest "DownloadMiB"
   # --- 12 = Speedtest "Ping"
   # --- 13 = Time required in ms

   TimeOID = time.time()
   Flag    = False
   local   = '127.0.0.1'
   OID     = io.ioRead( asset.avrOID_DataFile ).split(' ')

   # --------------------------------------------------------------------------------
   # --- Get Thermal-Status ...
   walk = walk_2( 'syno', local, '1.3.6.1.4.1.6574.1.2.0' )
   if walk.isdigit() :
      OID[5] = '%.3f' % ( float( walk ) )
      Flag   = True

   if iLoop == 0 :
      for index, sID in enumerate( Conf['LogEnvTabEnv'] ):
         # -----------------------------------------------------------------------------
         # --- Get Environmental sensor ...
         if Conf['LogEnvOn' + sID] != "Yes" : continue

         if Conf['LogEnvV3On' + sID] == "No" :
            walk = walk_2( Conf['LogEnvCom' + sID], Conf['LogEnvIp' + sID], Conf['LogEnvOID' + sID] )
         else :
            walk = walk_3( Conf['LogEnvUser' + sID], Conf['LogEnvEncryption' + sID], Conf['LogEnvPass' + sID], Conf['LogEnvIp' + sID], Conf['LogEnvOID' + sID] )

         if walk.isdigit() :
            OID[index] = '%.3f' % ( float( walk ) / int( Conf['LogEnvScale' + sID] ) )
            Flag   = True

      # -----------------------------------------------------------------------------
      # --- Get Harddisk temperatures ...
      walk = walk_2( 'syno', local, '1.3.6.1.4.1.6574.2.1.1.6' ).strip().replace( '\n', ' ' )
      if ''.join( ( walk.split() ) ).isdigit() :
         walk = [ int(x) for x in walk.split(' ') ]

         mxh = walk[0]
         mnh = walk[0]
         mvh = walk[0]

         for mh in range( 1, len(walk) ) :
            if mxh < walk[mh] : mxh = walk[mh]  # --- Max. temperature
            if mnh > walk[mh] : mnh = walk[mh]  # --- Min. temperature
            mvh += walk[mh]                     # --- Average temperature

         OID[6] = '%.3f' % ( mxh )
         OID[7] = '%.3f' % ( mnh )
         OID[8] = '%.3f' % ( mvh / len( walk ) )
         Flag   = True

   # --------------------------------------------------------------------------------
   # --- Get Memory "Cached" ...
   walk = walk_2( 'syno', local, '1.3.6.1.4.1.2021.4.15.0' ).strip().split(' ')[0]
   if walk.isdigit() :
      OID[9] = '%d' % ( int( walk ) )
      Flag   = True

   # --------------------------------------------------------------------------------
   # --- Get Speedtest "UploadMiB" & "DownloadMiB" ...
   if os.path.exists( asset.f_SpeedFlag ) :
      SPEED = io.ioRead( asset.SpeedOIDFile ).split(' ')

      OID[10] = SPEED[0]   # Speedtest "UploadMiB"
      OID[11] = SPEED[1]   # Speedtest "DownloadMiB"
      OID[12] = SPEED[2]   # Speedtest "Ping"
      Flag   = True

   # --------------------------------------------------------------------------------
   # --- Spend Time ...
   OID[13] = str( int( ( time.time() - TimeOID ) * 1000 ) )

   # --------------------------------------------------------------------------------
   # --- If new values exist, then save ...
   if Flag :
      walk = ' '.join( OID ) + '\n'
      io.ioWrite( walk, asset.avrOID_DataFile + '.tmp' )

      io.touch ( asset.f_OidFlag )
      io.ioMove( asset.avrOID_DataFile + '.tmp', asset.avrOID_DataFile )
      io.ioRm  ( asset.f_OidFlag )

# -----------------------------------------------------------------------------------
# --- System "snmpwalk v2" Call :
# -----------------------------------------------------------------------------------
def walk_2( sCom, sIp, sOID ):

   cmd = f'snmpwalk -r 0 -t 1 -v 2c -c "{sCom}" -O qv "{sIp}" "{sOID}" 2>&1'
   return CMD( cmd ).strip()

# -----------------------------------------------------------------------------------
# --- System "snmpwalk v3" Call :
# -----------------------------------------------------------------------------------
def walk_3( sUser, sEnc, sPass, sIp, sOID ):

   cmd = f'snmpwalk -r 0 -t 1 -v 3 -l authNoPriv -u "{sUser}" -a "{sEnc}" -A "{sPass}" "{sIp}" -O qv "{sOID}" 2>&1'
   return CMD( cmd ).strip()

# --------------------------------------------------------------------------
# --- Eine "DSM"-Funktion aufrufen und das Ergebnis liefern ...
# --------------------------------------------------------------------------
def CMD( cmd ):

   out, err = subprocess.Popen( cmd, shell=True, stdout=subprocess.PIPE, stdin=subprocess.PIPE ).communicate()
   return out.decode( "utf-8" )

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
