# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#
import os
import sys
import time
import re
import threading
import glob
import json
import socket
import random

import asset
import basics   as io
import Oid
import lib
from   lib      import switch, case
import Init     as rel
import Sys
import Smart
# -----------------------------------------------------------------------------------
import __main__ as avr
# -----------------------------------------------------------------------------------

# --------------------------------------------------------------------------
# --- Check ''/sys/class/hwmon/'' for the availability of different sensors :
# --- Determine the number of available CPU-Thermal sensors
# --- Determine the number of available FAN-RPM sensors
# --- Determine the number of available i2c-Thermal sensors
def Hwmon() :

   # ----------------------------------------------------------
   # --- Check ''/sys/class/hwmon/'' for the availability of different sensors :
   try    : hwmon = os.listdir( '/sys/class/hwmon' )
   except : hwmon = ""

   # -----------------------------------------------------------------------
   # --- Determine the number of available CPU-Thermal sensors
   asset.sysPrescaler = 1000
   asset.sys_Cpu = []
   asset.sys_Cpu = sorted( lib.getCMD( 'find "/sys" -not -path "*pci*" -name "temp?_input"' ).strip().split('\n') )

   if not asset.sys_Cpu[0] :
      asset.AvrConf["PlotCPUOn"] = ""
   else :
      if not asset.AvrConf["PlotCPUOn"] :
         asset.AvrConf["PlotCPUOn"] = "No"

   # -----------------------------------------------------------------------
   # --- Check what Prescaler we need ...
   mx = 0
   for file in asset.sys_Cpu :
      mx += int( io.ioRead( file ) )

   if mx / len( asset.sys_Cpu ) < 1000 : asset.sysPrescaler = 1

   # -----------------------------------------------------------------------
   # --- Determine the number of available FAN-RPM & i2c-Thermal sensors
   asset.sys_Fan = []
   asset.sys_i2c = []

   if 'hwmon1' in hwmon:
      if os.path.isdir( '/sys/class/hwmon/hwmon1/device' ) :
         asset.sys_Fan = sorted( glob.glob( '/sys/class/hwmon/hwmon1/device/pwm2') )
         asset.sys_i2c = sorted( glob.glob( '/sys/class/hwmon/hwmon1/device/temp?_input') )

   if not asset.sys_Fan :
      asset.AvrConf["PlotPwmOn"] = ""
   else :
      if not asset.AvrConf["PlotPwmOn"] :
         asset.AvrConf["PlotPwmOn"] = "No"

   if not asset.sys_i2c :
      asset.AvrConf["Ploti2cOn"] = ""
   else :
      if not asset.AvrConf["Ploti2cOn"] :
         asset.AvrConf["Ploti2cOn"] = "No"

# --------------------------------------------------------------------------
# --- Check availability of Synology Sensors:
# --------------------------------------------------------------------------
def init( opt=False ):

   if rel.relSensors == ''.join(( asset.AvrConf["SpeedTestOn"], asset.AvrConf["VpnPPPoEOn"] )) : return
   rel.relSensors     = ''.join(( asset.AvrConf["SpeedTestOn"], asset.AvrConf["VpnPPPoEOn"] ))

   # ----------------------------------------------------------
   Hwmon()        # --- Determine the number of available CPU-Thermal / FAN-RPM and i2c-Thermal sensors
   Vpn()          # --- Check / determine "Server" & "Client" network-interfaces ...
   Speedtest()    # --- Check availability of Speedtest
   Storage()      # --- Determine the number of available storagepool's:
   Network()      # --- Determine the number of available network-interfaces ...
#  Nut()          # --- Determine if UPS support is available ...
   # ----------------------------------------------------------

# --------------------------------------------------------------------------
# --- Set the Teritiary 2 Axis values (UPS Input-Voltage) based on the user settings ...
# --------------------------------------------------------------------------
def initUps () :

   # -----------------------------------------------------------------------
   # --- Init query Nominal Voltage ...
   if asset.AvrConf[ 'UsvInputNominal'   ] in [ '100', '110', '120', '127', '220', '230', '240' ] :
      UsvInputNominal = int( asset.AvrConf[ 'UsvInputNominal' ] )
   else :
      asset.AvrConf[ 'UsvInputNominal'   ] = '230'
      UsvInputNominal = 230

   if asset.AvrConf[ 'UsvInputTolerance' ] in [ '5', '10', '15' ] :
      UsvInputTolerance = int( asset.AvrConf[ 'UsvInputTolerance' ] )
   else :
      asset.AvrConf[ 'UsvInputTolerance' ] = '10'
      UsvInputTolerance = 10

   # -----------------------------------------------------------------------
   UsvInputMin = int( ( UsvInputNominal / 100 ) * ( 100 - UsvInputTolerance ) )
   UsvInputMax = int( ( UsvInputNominal / 100 ) * ( 100 + UsvInputTolerance ) )

   asset.AvrConf[ 'Sec3Min'      ] = f'{UsvInputMin}' # Start-Wert
   asset.AvrConf[ 'Sec3ScaleMin' ] = f'{UsvInputMin}' # Skalierung Min.
   asset.AvrConf[ 'Sec3ScaleMax' ] = f'{UsvInputMax}' # Skalierung Max.

# --------------------------------------------------------------------------
# --- Determine if UPS support is available and Init (examin) Syno UPS-Settings ...
# --------------------------------------------------------------------------
def Nut ( opt=False ) :

   # -----------------------------------------------------------------------
   # --- Set Default-Values ...
   asset.AvrConf['UsvSynoSupport']     = ''
   asset.sys_USV_Support               = False

   asset.AvrConf['UsvSynoEnabled']     = ''
   asset.AvrConf['UsvSynoMode'   ]     = ''
   asset.AvrConf['UsvSlaveServer']     = ''

   asset.AvrConf['UsvLoadOn'     ]     = 'No'
   asset.AvrConf['UsvDischargeOn']     = 'No'
   asset.AvrConf['UsvInputOn'    ]     = 'No'

   asset.sys_USV_Discharge             = False
   asset.sys_USV_Load                  = False
   asset.sys_USV_Input                 = False
   asset.sys_USV_HostIP                = False

   # -----------------------------------------------------------------------
   # --- Get some SynoInfo's ...
   if not bool( asset.SynoInfo ) : asset.SynoInfo = lib.FileConfig( asset.sysSynoConfig )

   sysConf = { key : value[:] for key, value in asset.SynoInfo.items() }
   if io.check( asset.sysUpsConfig ) : sysConf = lib.FileConfig( asset.sysUpsConfig, Conf=sysConf )

   # -----------------------------------------------------------------------
   sysConf[ 'ups_enabled'      ] = sysConf.get( 'ups_enabled',      'no'   )
   sysConf[ 'upsslave_enabled' ] = sysConf.get( 'upsslave_enabled', 'no'   )
   sysConf[ 'ups_mode'         ] = sysConf.get( 'ups_mode',         'none' )

   # -----------------------------------------------------------------------
   if sysConf[ 'ups_enabled' ] == 'no' and sysConf[ 'upsslave_enabled' ] == 'no' or not sysConf[ 'ups_mode' ] in [ 'slave', 'usb' ] :
      # --------------------------------------------------------------------
      # --- Flag values as unavailable and return ... 
      asset.AvrConf[ 'PlotUsvDischargeOn' ] = ''
      asset.AvrConf[ 'PlotUsvLoadOn'      ] = ''
      asset.AvrConf[ 'PlotUsvInputOn'     ] = ''

      return False

   # -----------------------------------------------------------------------
   if sysConf[ 'ups_mode'] == 'usb' :
      # --- USB : Local-Mode (LocalHost) ...
      asset.AvrConf[ 'UsvSynoMode'    ] = 'USB'
      asset.AvrConf[ 'UsvSlaveServer' ] = asset.nutDefaultHostIP
   else :
      # --- Slave : Slave-Mode ( IP ) ...
      asset.AvrConf[ 'UsvSynoMode'    ] = 'Slave'
      asset.AvrConf[ 'UsvSlaveServer' ] = sysConf.get( 'upsslave_server', asset.nutDefaultHostPort )

   # -----------------------------------------------------------------------
   # --- Query 'all' UPS-Values ...
   sRet = Get_UsvSock( asset.AvrConf[ 'UsvSlaveServer' ], asset.nutDefaultHostPort )
   if not sRet :
      # --------------------------------------------------------------------
      # --- Flag values as unavailable and return ... 
      asset.AvrConf[ 'PlotUsvDischargeOn' ] = ''
      asset.AvrConf[ 'PlotUsvLoadOn'      ] = ''
      asset.AvrConf[ 'PlotUsvInputOn'     ] = ''

      return False

   # -----------------------------------------------------------------------
   # --- Check whether the UPS delivers the desired values ...
#  if not sRet.get( 'battery.charge', False ) and not sRet.get( 'ups.load', False ) :
   if not sRet.get( 'battery.charge', False ) and not sRet.get( 'ups.load', False ) and not sRet.get( 'input.voltage', False ):
      # --------------------------------------------------------------------
      # --- Flag values as unavailable and return ... 
      asset.AvrConf[ 'PlotUsvDischargeOn' ] = ''
      asset.AvrConf[ 'PlotUsvLoadOn'      ] = ''
      asset.AvrConf[ 'PlotUsvInputOn'     ] = ''

      return False

   # -----------------------------------------------------------------------
   # ---Check UPS charge status ...
   if sRet.get( 'battery.charge', False ) :
      asset.AvrConf[ 'UsvDischargeOn'     ] = 'Yes'
      if not asset.AvrConf.get( 'PlotUsvDischargeOn', '' ) : asset.AvrConf[ 'PlotUsvDischargeOn' ] = 'No'
   else :
      asset.AvrConf[ 'UsvDischargeOn'     ] = 'No'
      asset.AvrConf[ 'PlotUsvDischargeOn' ] = ''

   # -----------------------------------------------------------------------
   # --- Check UPS output power ...
   if sRet.get( 'ups.load', False ) :
      asset.AvrConf[ 'UsvLoadOn'          ] = 'Yes'
      if not asset.AvrConf.get( 'PlotUsvLoadOn', '' ) : asset.AvrConf[ 'PlotUsvLoadOn' ] = 'No'
   else :
      asset.AvrConf[ 'UsvLoadOn'          ] = 'No'
      asset.AvrConf[ 'PlotUsvLoadOn'      ] = ''

   # -----------------------------------------------------------------------
   # --- Check UPS input voltage ...
   if sRet.get( 'input.voltage', False ) :
      asset.AvrConf[ 'UsvInputOn'         ] = 'Yes'
      if not asset.AvrConf.get( 'PlotUsvInputOn', '' ) : asset.AvrConf[ 'PlotUsvInputOn' ] = 'No'
   else :
      asset.AvrConf[ 'UsvInputOn'         ] = 'No'
      asset.AvrConf[ 'PlotUsvInputOn'     ] = ''

   # -----------------------------------------------------------------------
   asset.AvrConf['UsvSynoSupport'] = 'Yes'

   asset.sys_USV_Support   = asset.AvrConf[ 'UsvSynoSupport'     ]  # Could only be "Yes"
   asset.sys_USV_HostIP    = asset.AvrConf[ 'UsvSlaveServer'     ]
   asset.sys_USV_Discharge = asset.AvrConf[ 'PlotUsvDischargeOn' ]  # "Yes" if available
   asset.sys_USV_Load      = asset.AvrConf[ 'PlotUsvLoadOn'      ]  # "Yes" if available
   asset.sys_USV_Input     = asset.AvrConf[ 'PlotUsvInputOn'     ]  # "Yes" if available

   return True

# --------------------------------------------------------------------------
# --- UPS : Get UPS "battery.charge", "ups.load" & "input.voltage" ...
# --------------------------------------------------------------------------
def Get_Nut() :

   if not asset.sys_USV_Support and asset.sys_USV_Discharge != 'Yes' and asset.sys_USV_Load != 'Yes' and asset.sys_USV_Input != 'Yes' :
      return True

   # -----------------------------------------------------------------------
   sRet = Get_UsvSock( asset.sys_USV_HostIP, asset.nutDefaultHostPort )
   if not sRet :
      # --- Try to reInit UPS support status ...
      lib.Error( 'UPS: A new UPS initialization is carried out ...' )
      Nut()
      return False

   # -----------------------------------------------------------------------
   if asset.sys_USV_Discharge :
      asset.SMA[asset.LastSMA][asset.TabUsvCharge] = round( 100.0 - float( sRet[ 'battery.charge' ] ), asset.rnd_SMA )
   else :
      asset.SMA[asset.LastSMA][asset.TabUsvCharge] = 0.0

   if asset.sys_USV_Load :
      asset.SMA[asset.LastSMA][asset.TabUsvLoad  ] = round(         float( sRet[ 'ups.load'       ] ), asset.rnd_SMA )
   else :
      asset.SMA[asset.LastSMA][asset.TabUsvLoad  ] = 0.0

   if asset.sys_USV_Input :
      asset.SMA[asset.LastSMA][asset.TabUsvInput ] = round(         float( sRet[ 'input.voltage'  ] ), asset.rnd_SMA )
   else :
      asset.SMA[asset.LastSMA][asset.TabUsvInput ] = 0.0

# --------------------------------------------------------------------------
# --- Query "UPS Status" informations via socket
# --------------------------------------------------------------------------
def Get_UsvSock( HostIP, HostPort ) :

   # -----------------------------------------------------------------------
   # --- Create a socket object ...
   try :
      sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
      if not sock :
         lib.Error( 'UPS: Socket creation failed.' )
         return False
   except socket.error as Err :
      lib.Error( f'UPS: Error creating socket: {Err}' )
      return False

   # -----------------------------------------------------------------------
   # --- Connection to Host-IP on the port with TimeOut ...
   try :
      sock.settimeout( asset.nutSockTimeOut )
      sock.connect( ( HostIP, HostPort ) )
   except socket.gaierror as Err :
      lib.Error( f'UPS: Error connecting host: {Err}' )
      return False
   except socket.error as Err :
      lib.Error( f'UPS: Connection error: {Err}' )
      return False

   # -----------------------------------------------------------------------
   # --- Initiate query of the UPS parameter / information
   try :
      sock.sendall( b'LIST VAR ups\nLOGOUT\n' )
   except socket.error as Err:
      lib.Error( f'UPS: Query failed: {Err}' )
      return False

   # -----------------------------------------------------------------------
   # --- Start reading from socket (UPS-Informations) ...
   bRet = bytearray()

   try :
      while True:
         bPack = sock.recv( 128 )
         if not bPack : break

         bRet.extend( bPack )

   except socket.error as Err: 
      # --- Faild to read from Usv-Socket !
      lib.Error( f'UPS: Error receiving data: {Err}' )
      sock.close()
      return False

   # -----------------------------------------------------------------------
   # --- Close socket object ...
   sock.close()

   # --- Decode "Byte"-Responce to 'utf-8' ...
   bRet = bRet.decode( 'utf-8' ).strip().split( '\n' )

   if not bRet : return False

   # -----------------------------------------------------------------------
   # --- Check that the responce was complete and correct ...
   if bRet[0].startswith( 'ERR' ) :
      lib.Error( f'UPS: Responce error: {bRet[0]}' )
      return False

   if not bRet[0].startswith( 'BEGIN LIST VAR' ) :
      lib.Error( f'UPS: Responce error: {bRet[0]}' )
      return False

   if not bRet[-1].startswith( 'OK Goodbye' ) :
      lib.Error( f'UPS: Responce error: {bRet[-1]}' )
      return False

   if not bRet[-2].startswith( 'END LIST VAR' ) :
      lib.Error( f'UPS: Responce error: {bRet[-2]}' )
      return False

   # -----------------------------------------------------------------------
   # --- Remove Headline / Footer / Headers ('VAR ups')  / Separate Responce into Dict ...
   sRet = dict()

   for sLine in [ x.split( 'VAR ups ', 1 )[-1] for x in bRet[1:-2] ] :
      try :
         sKey, sValue = sLine.split( ' ', 1 )

         # --- Reformat / clean Value & Key (Name) ...
         sValue = ' '.join( sValue.strip( '"' ).split() )
         sKey   = ' '.join( sKey.split() )

         sRet[ sKey ] = sValue
      except :
         # --- On Error skip line (continue) ...
         pass

   # -----------------------------------------------------------------------
   return sRet

# --------------------------------------------------------------------------
# --- Check / determine "Server" & "Client" network-interfaces :
# --------------------------------------------------------------------------
def Vpn() :

   # ----------------------------------------------------------
   # --- Set Default Values
   # ----------------------------------------------------------
   asset.sys_VPN_Status  = False

   asset.sys_VPN_PPTP    = False
   asset.sys_VPN_OpenVPN = False
   asset.sys_VPN_IPSec   = False
   asset.sys_VPN_PPPoE   = False

   # ----------------------------------------------------------
   # --- Check: VPN Server is installed ...
   # ----------------------------------------------------------
   if io.check( asset.sysVpnServer + "/synovpn.conf" ) :
      # ----------------------------------------------------------
      # --- Examine "VPN Server Config"
      # ----------------------------------------------------------

      vpn = io.ioRead( asset.sysVpnServer + "/synovpn.conf" ).strip().split('\n')
      # ----------------------------------------------------------
      # --- Check: VPN Server "PPTP"
      # ----------------------------------------------------------
      x = [ x for x in vpn if 'runpptpd' in x ]
      if x and x[0].split('=',1)[1] == "yes" :
         asset.sys_VPN_Status  = True
         asset.sys_VPN_PPTP    = True

      # ----------------------------------------------------------
      # --- Check: VPN Server "OpenVPN"
      # ----------------------------------------------------------
      x = [ x for x in vpn if 'runopenvpn' in x ]
      if x and x[0].split('=',1)[1] == "yes" :
         asset.sys_VPN_Status  = True
         asset.sys_VPN_OpenVPN = True

      # ----------------------------------------------------------
      # --- Check: VPN Server "L2TP/IPSec"
      # ----------------------------------------------------------
      x = [ x for x in vpn if 'runl2tpd' in x ]
      if x and x[0].split('=',1)[1] == "yes" :
         asset.sys_VPN_Status  = True
         asset.sys_VPN_IPSec   = True

   # ----------------------------------------------------------
   # --- Check: VPN Client "PPTP"
   # ----------------------------------------------------------
   if io.check( asset.sysVpnClient + "/pptp/pptpclient.conf" ) :
      asset.sys_VPN_Status  = True
      asset.sys_VPN_PPTP    = True

   # ----------------------------------------------------------
   # --- Check: VPN Client "OpenVPN"
   # ----------------------------------------------------------
   if io.check( asset.sysVpnClient + "/openvpn/ovpnclient.conf" ) :
      asset.sys_VPN_Status  = True
      asset.sys_VPN_OpenVPN = True

   # ----------------------------------------------------------
   # --- Check: VPN Client "L2TP/IPSec"
   # ----------------------------------------------------------
   if io.check( asset.sysVpnClient + "/l2tp/l2tpclient.conf" ) :
      asset.sys_VPN_Status  = True
      asset.sys_VPN_IPSec   = True

   # ----------------------------------------------------------
   # --- Check if "PPPoE" is enabled
   # ----------------------------------------------------------
   if asset.AvrConf["VpnPPPoEOn"] == "Yes" :
      asset.sys_VPN_Status  = True
      asset.sys_VPN_PPPoE   = True

# --------------------------------------------------------------------------
# --- Eth Load : Init first use ...
# --------------------------------------------------------------------------
def FirstEth() :

   pre_eth = dict()
   stats   = dict()

   iList = io.ioRead( '/proc/net/dev' ).split('\n')[2:]  # Get eth devicelist incl. I/O statistics
   CurTimeNetstats = time.time()                         # Current time at which the values were read

   for sLine in iList :
      Item = ' '.join( sLine.split() ).split( ' ' )
      stats[ Item[0].strip( ':' ) ] = [ int( Item[1] ), int( Item[9] ) ]

   # ----------------------------------------------------------
   syno_r  = 0
   syno_w  = 0

   # ----------------------------------------------------------
   # --- Clear EthLoad-Flags ...
   EthList = dict()
   for dev in asset.sys_eth :
      EthList[dev] = 0

   # ----------------------------------------------------------
   # --- Prepare selectet List of Network Devices ....

   for dev in asset.sys_EthGetList :
      while switch( re.sub('[^a-zA-Z ]+', '', dev ) ):
         # ----------------------------------------------------------
         if case( 'ppp', 'tun', 'ovsbond' ):
            iO = stats.get( dev, [ 0, 0 ] )
            pre_eth[dev + "-r"] = iO[0]            # Get RX Bytes
            pre_eth[dev + "-w"] = iO[1]            # Get TX Bytes
            break
         # ----------------------------------------------------------
         if case( 'speed' ):
            pre_eth[dev + "-r"] = asset.sys_SpeedDownloadMiB   # Speedtest "DownloadMiB"
            pre_eth[dev + "-w"] = asset.sys_SpeedUploadMiB     # Speedtest "UploadMiB"
            break
         # ----------------------------------------------------------
         if case( 'eth' ):
            if dev in EthList :
               pre_eth[dev + "-r"] = stats[ dev ][0]  # Get RX Bytes
               pre_eth[dev + "-w"] = stats[ dev ][1]  # Get TX Bytes
               EthList.pop(dev)
            break
         # ----------------------------------------------------------
         if case( 'total' ):
            for dev in asset.sys_eth :
               if dev in EthList :
                  pre_eth[dev + "-r"] = stats[ dev ][0]  # Get RX Bytes
                  pre_eth[dev + "-w"] = stats[ dev ][1]  # Get TX Bytes
                  EthList.pop(dev)
                  syno_r += stats[ dev ][0]
                  syno_w += stats[ dev ][1]
               else :
                  syno_r += pre_eth[dev + "-r"]
                  syno_w += pre_eth[dev + "-w"]
            # --- Total read / write ...
            pre_eth["total-r"] = syno_r
            pre_eth["total-w"] = syno_w
            break
         break

   # ----------------------------------------------------------
   pre_eth["none-r"] = 0
   pre_eth["none-w"] = 0

   asset.pre_eth         = pre_eth
   asset.CurTimeNetstats = CurTimeNetstats

# --------------------------------------------------------------------------
# --- Eth Load : Init first use ...
# --------------------------------------------------------------------------
def Get_Eth() :

   cur_eth = dict()
   stats   = dict()

   PreTimeNetstats = asset.CurTimeNetstats               # Previous time at which the values were read
   iList = io.ioRead( '/proc/net/dev' ).split('\n')[2:]  # Get eth devicelist incl. I/O statistics
   CurTimeNetstats = time.time()                         # Current time at which the values were read

   # ----------------------------------------------------------
   for sLine in iList :
      Item = ' '.join( sLine.split() ).split( ' ' )
      stats[ Item[0].strip( ':' ) ] = [ int( Item[1] ), int( Item[9] ) ]

   # ----------------------------------------------------------
   Tick = CurTimeNetstats - PreTimeNetstats   # / 1000000 * 1048576    # 1.048.576 = 1MiB/ms (1024 x 1024)

   syno_r  = 0
   syno_w  = 0

   # ----------------------------------------------------------
   # --- Clear EthLoad-Flags ...
   EthList = dict()
   for dev in asset.sys_eth :
      EthList[dev] = 0

   # ----------------------------------------------------------
   # --- Prepare selectet List of Storrage Devices ....
   for dev in asset.sys_EthGetList :
      while switch( re.sub('[^a-zA-Z ]+', '', dev ) ):
         # ----------------------------------------------------------
         if case( 'ppp', 'tun', 'ovsbond' ):
            iO = stats.get( dev, [ 0, 0 ] )
            cur_eth[dev + "-r"] = iO[0]            # Get RX Bytes
            cur_eth[dev + "-w"] = iO[1]            # Get TX Bytes
            break
         # ----------------------------------------------------------
         if case( 'speed' ):
            cur_eth[dev + "-r"] = asset.sys_SpeedDownloadMiB   # Speedtest "DownloadMiB"
            cur_eth[dev + "-w"] = asset.sys_SpeedUploadMiB     # Speedtest "UploadMiB"
            break
         # ----------------------------------------------------------
         if case( 'eth' ):
            if dev in EthList :
               cur_eth[dev + "-r"] = stats[ dev ][0]  # Get RX Bytes
               cur_eth[dev + "-w"] = stats[ dev ][1]  # Get TX Bytes
               EthList.pop(dev)
            break
         # ----------------------------------------------------------
         if case( 'total' ):
            for dev in asset.sys_eth :
               if dev in EthList :
                  cur_eth[dev + "-r"] = stats[ dev ][0]  # Get RX Bytes
                  cur_eth[dev + "-w"] = stats[ dev ][1]  # Get TX Bytes
                  EthList.pop(dev)
                  syno_r += stats[ dev ][0]
                  syno_w += stats[ dev ][1]
               else :
                  syno_r += cur_eth[dev + "-r"]
                  syno_w += cur_eth[dev + "-w"]
            # --- Total read / write ...
            cur_eth["total-r"] = syno_r
            cur_eth["total-w"] = syno_w
         break

   cur_eth["none-r"] = 0
   cur_eth["none-w"] = 0

   # --- Calculate the given eth device I/O (MByte/sec) ...
   for i in range( 0, asset.TabEth_Max ) :
      # --- Device-Name
   #  dev = asset.AvrConf['Eth' + str(i) + 'Device']
      dev = asset.AvrConf[ f'Eth{i}Device' ]

      lr = i * 2 + asset.TabEth
      lw = lr + 1

      if dev != "speed" :
         asset.SMA[asset.LastSMA][lr] = round( ( cur_eth[dev + "-r"] - asset.pre_eth[dev + "-r"] ) / 1048576 / Tick, asset.rnd_SMA )
         asset.SMA[asset.LastSMA][lw] = round( ( cur_eth[dev + "-w"] - asset.pre_eth[dev + "-w"] ) / 1048573 / Tick, asset.rnd_SMA )
      else :
         asset.SMA[asset.LastSMA][lr] = cur_eth[dev + "-r"]
         asset.SMA[asset.LastSMA][lw] = cur_eth[dev + "-w"]

   # --- Copy current I/O statistics to prevues ...
   asset.pre_eth = {**cur_eth}
   asset.CurTimeNetstats = CurTimeNetstats

# --------------------------------------------------------------------------
# --- Determine the number of available storagepool's:
# --------------------------------------------------------------------------
def Storage() :

#  avr.AvrSmart = True

   asset.sys_sdl          = []
   asset.sys_mdl          = []
   asset.md_Member        = dict()
   asset.sys_cmpMdGetList = dict()

   iRaw   = [ 'none',   'total' ]
   iGUI   = [ '"None"', '"Total"' ]

   iPool  = asset.sysStorage.get( 'pools', dict() )
   iCache = asset.sysStorage.get( 'cache', dict() )
   iDisk  = asset.sysStorage.get( 'disks', dict() )

   # -----------------------------------------------------------------------
   # --- Prepare StoragePool-List ...
   if avr.AvrSmart : print ( '| SYS: Prepare StoragePool-List :' )

   if iCache : iRaw, iGUI = _addPool( iDisk, iCache, asset.msg[ 'sysCacheName' ], iRaw, iGUI, iCache=True  )
   if iPool  : iRaw, iGUI = _addPool( iDisk, iPool,  asset.msg[ 'sysPoolName'  ], iRaw, iGUI, iCache=False )

   if not iPool and not iCache :
      # --------------------------------------------------------------------
      for sDev in sorted( iDisk ) :

         if iDisk[ sDev ].get( 'isBad', True ) or iDisk[ sDev ].get( 'type', 'NA' ) != 'internal' : continue

         try :
            iRaw.append( sDev )                                               # Add to "Raw"-List
            iGUI.append( '"{name} ({0})"'.format( sDev, **iDisk[ sDev ] ) )  # Add to "GUI"-List
            asset.sys_sdl.append( sDev )                                      # Add to "sdl"-List
         except :
            msg  = time.strftime('%Y.%m.%d %H:%M:%S SYS: Fatal error has occurred. "AvrLogger" will now shutdown ...\n' )
            msg += time.strftime('%Y.%m.%d %H:%M:%S SYS: "StoragePool" or "DiskInfo" of Device [{0}] is malformed :\n'.format( sDev ) )
            msg += 'DiskInfo  :\n'
            msg += '{0}\n'.format( json.dumps( iDisk.get( sDev, dict() ) , sort_keys=True, indent=4 ) )
            msg += 'StoragePool :\n'
            msg += io.ioRead( asset.avrEnumStorage )
            io.ioAdd( msg , asset.ERROR )
            Sys.RotateClear( asset.avrEnumStorage )
            io.ioRm( asset.avrEnumStorage )

            lib.getCMD( 'synopkg stop {0}'.format( asset.SpkName ) )
            sys.exit()

   # -----------------------------------------------------------------------
   for dev in iRaw :
      asset.sys_cmpMdGetList[dev] = dev

   # -----------------------------------------------------------------------
   asset.AvrConf[ 'MdDeviceListRAW' ] = ' '.join( iRaw )
   asset.AvrConf[ 'MdDeviceListGUI' ] = ' '.join( iGUI )

#  print ( asset.AvrConf[ 'MdDeviceListRAW' ] )
#  print ( asset.AvrConf[ 'MdDeviceListGUI' ] )
#  print ( asset.sys_mdl )
#  print ( asset.sys_sdl )

# --------------------------------------------------------------------------
# --- Add a SoragePool (Volume / Cache) ...
# --------------------------------------------------------------------------
def _addPool ( iDisk, iPool, pName, iRaw, iGUI, iCache ) :

   # -----------------------------------------------------------------------
   # --- Prepare StoragePool-List ...
   if iPool :
      for iNum in sorted( iPool ):
      #  print ( iNum )
         if iPool[ iNum ][ 'isBad' ] : continue

         if iCache : sMdl = 'ch' + str( iNum )
         else      : sMdl = 'md' + str( iNum )
         sID  = iPool[ iNum ]
         if avr.AvrSmart :
            print (f'| SYS: Process StoragePool : {sID["desc"]}' )
            print (f'| SYS: Location : {sID["loc"]}' )
            print (f'| SYS: Name     : volume{iNum}' )
            print (f'| SYS: Devices  : {sID["sdl"]}' )

         # -----------------------------------------------------------------
         # --- Store Member's of this Pool ...
         asset.md_Member[ sMdl ] = sID['sdl']

         iRaw.append( f'{sMdl}' )               # Add to "Raw"-List
         iGUI.append( f'"{pName} {iNum}"' )     # Add to "GUI"-List
         asset.sys_mdl.append( sMdl )           # Add to "mdl"-List

         # -----------------------------------------------------------------
         # --- Prepare Disk-List ...
         if avr.AvrSmart : print ( '| SYS: Prepare Disk-List :' )

         rDict = dict()
         for sDev in sID['sdl'] :
            iNum = int( iDisk[ sDev ][ 'number' ] )
            
            rDict[ f'{iNum:02d}' ] = sDev


      #  print ( rDict )
         for iItem in sorted( rDict ) :
            sDev = rDict[ iItem ]
         #  print ( sDev )
            try :
               if avr.AvrSmart :
                  print (f'| SYS: Process Device : {sDev}' )
                  print (f'| SYS: Name     : {iDisk[ sDev ][ "name" ]}' )
                  print (f'| SYS: Model    : {iDisk[ sDev ][ "model" ]}' )

               if iCache : sName = asset.msg[ 'sysSSDName' ] + ' ' + iDisk[ sDev ][ 'number' ]
               else      : sName = iDisk[ sDev ][ 'name' ]

               iRaw.append( sDev )                 # Add to "Raw"-List
               iGUI.append( f'"   - {sName}"' )    # Add to "GUI"-List
               asset.sys_sdl.append( sDev )        # Add to "sdl"-List
            except :
               if not avr.AvrSmart :
                  msg  = time.strftime('%Y.%m.%d %H:%M:%S SYS: Fatal error has occurred. "AvrLogger" will now shutdown ...\n' )
                  msg += time.strftime('%Y.%m.%d %H:%M:%S SYS: "StoragePool" or "DiskInfo" of Device [{0}] is malformed :\n'.format( sDev ) )
                  msg += 'DiskInfo  :\n'
                  msg += '{0}\n'.format( json.dumps( iDisk.get( sDev, dict() ) , sort_keys=True, indent=4 ) )
                  msg += 'StoragePool :\n'
                  msg += io.ioRead( asset.avrEnumStorage )
                  io.ioAdd( msg , asset.ERROR )
                  Sys.RotateClear( asset.avrEnumStorage )
                  io.ioRm( asset.avrEnumStorage )
                  lib.getCMD( 'synopkg stop {0}'.format( asset.SpkName ) )
               else :
                  print (f'| SYS: "StoragePool" or "DiskInfo" of Device [{sDev}] is malformed :' )
                  print (f'| SYS: "StoragePool\'s" :\n {json.dumps( iPool, sort_keys=True, indent=4 )}' )
                  print (f'| SYS: "StorageDisk\'s" :\n {json.dumps( iDisk, sort_keys=True, indent=4 )}' )
               # -----------------------------------------------------------
               sys.exit()

   # -----------------------------------------------------------------------
   return iRaw, iGUI

# --------------------------------------------------------------------------
# --- Disk Load : Init first use ...
# --------------------------------------------------------------------------
def FirstDisk() :

   syno_r    = 0
   syno_w    = 0

   DiskLoad  = dict()
   MdLoad    = dict()
   pre_stats = dict()
   stats     = dict()

   iList            = io.ioRead( '/proc/diskstats' ).split('\n')
   CurTimeDiskstats = time.time() # Current time at which the values were read

   for sLine in iList :
      Item = ' '.join( sLine.split() ).split( ' ' )
      stats[ Item[2] ] = [ int( Item[5] ), int( Item[9] ) ]

   # ----------------------------------------------------------
   # --- Clear Load-Flags ...
   for dev in asset.sys_sdl :
      DiskLoad[dev] = 0

   for dev in asset.sys_mdl :
      MdLoad[dev] = 0

   if MdLoad : MdLoad['total'] = 0

   # ----------------------------------------------------------
   # --- Prepare selectet List of Storrage Devices ....
   for dev in sorted( asset.sys_MdGetList ) :
      while switch( re.sub('[^a-zA-Z ]+', '', dev ) ):
         # ----------------------------------------------------
         if case( 'md', 'ch' ):
            if dev in MdLoad :
               pre_stats[dev + "-r"] = 0
               pre_stats[dev + "-w"] = 0
               for sID in asset.md_Member[ dev ] :
                  if sID in DiskLoad :
                     pre_stats[dev + "-r"] += stats[sID][0]              # Total sectors read    (Field 3)
                     pre_stats[dev + "-w"] += stats[sID][1]              # Total sectors written (Field 7)
                     pre_stats[sID + "-r"]  = stats[sID][0]
                     pre_stats[sID + "-w"]  = stats[sID][1]
                     DiskLoad.pop( sID )
                  else :
                     pre_stats[dev + "-r"] += pre_stats[sID + "-r"]
                     pre_stats[dev + "-w"] += pre_stats[sID + "-w"]
               MdLoad.pop( dev )
            break

         # ----------------------------------------------------
         if case( 'total' ):
            if MdLoad :
               for dev in asset.sys_mdl :
                  if dev in MdLoad :
                     pre_stats[dev + "-r"] = 0
                     pre_stats[dev + "-w"] = 0
                     for sID in asset.md_Member[ dev ] :
                        if sID in DiskLoad :
                           pre_stats[dev + "-r"] += stats[sID][0]
                           pre_stats[dev + "-w"] += stats[sID][1]
                           pre_stats[sID + "-r"]  = stats[sID][0]
                           pre_stats[sID + "-w"]  = stats[sID][1]
                           DiskLoad.pop( sID )
                        else :
                           pre_stats[dev + "-r"] += pre_stats[sID + "-r"]
                           pre_stats[dev + "-w"] += pre_stats[sID + "-w"]
                     MdLoad.pop( dev )
                  syno_r += pre_stats[dev + "-r"]
                  syno_w += pre_stats[dev + "-w"]
            else :
               for dev in asset.sys_sdl :
                  if dev in DiskLoad :
                     pre_stats[dev + "-r"] = stats[dev][0]
                     pre_stats[dev + "-w"] = stats[dev][1]
                     DiskLoad.pop( dev )
                  syno_r += pre_stats[dev + "-r"]
                  syno_w += pre_stats[dev + "-w"]
            pre_stats["total-r"] = syno_r
            pre_stats["total-w"] = syno_w
            break

         # ----------------------------------------------------
         # --- Get I/O Values for a single HDD ...
         if dev in DiskLoad :
            pre_stats[dev + "-r"] = stats[dev][0]
            pre_stats[dev + "-w"] = stats[dev][1]
            DiskLoad.pop(dev)
         break

   # ----------------------------------------------------------
   pre_stats["none-r"] = 0
   pre_stats["none-w"] = 0

   asset.pre_stats        = pre_stats
   asset.CurTimeDiskstats = CurTimeDiskstats

# --------------------------------------------------------------------------
# --- Disk Load : Get Disk Load ...
# --------------------------------------------------------------------------
def Get_Disk() :

   syno_r    = 0
   syno_w    = 0

   DiskLoad  = dict()
   MdLoad    = dict()
   cur_stats = dict()
   stats     = dict()

   PreTimeDiskstats = asset.CurTimeDiskstats                         # Previous time at which the values were read
   iList            = io.ioRead( '/proc/diskstats' ).split('\n')
   CurTimeDiskstats = time.time()                                    # Current time at which the values were read

   for sLine in iList :
      Item = ' '.join( sLine.split() ).split( ' ' )
      stats[ Item[2] ] = [ int( Item[5] ), int( Item[9] ) ]

   # ----------------------------------------------------------
   Tick = CurTimeDiskstats - PreTimeDiskstats     # / 1000000 * 2048   # 2048 = 1 MiB (1024 x 1024)
#  Tick = ( CurTimeDiskstats - PreTimeDiskstats ) * 2048

   # ----------------------------------------------------------
   # --- Clear Load-Flags ...
   for dev in asset.sys_sdl :
      DiskLoad[dev] = 0

   for dev in asset.sys_mdl :
      MdLoad[dev] = 0

   if MdLoad : MdLoad['total'] = 0

   # ----------------------------------------------------------
   # --- Prepare selectet List of Storrage Devices ....
   for dev in asset.sys_MdGetList :
      while switch( re.sub('[^a-zA-Z ]+', '', dev ) ):
         # ----------------------------------------------------
         if case( 'md', 'ch' ):
            if dev in MdLoad :
               cur_stats[dev + "-r"] = 0
               cur_stats[dev + "-w"] = 0

               for sID in asset.md_Member[ dev ] :
                  if sID in DiskLoad :
                     cur_stats[dev + "-r"] += stats[sID][0]              # Total sectors read    (Field 3)
                     cur_stats[dev + "-w"] += stats[sID][1]              # Total sectors written (Field 7)
                     cur_stats[sID + "-r"]  = stats[sID][0]
                     cur_stats[sID + "-w"]  = stats[sID][1]
                     DiskLoad.pop( sID )
                  else :
                     cur_stats[dev + "-r"] += cur_stats[sID + "-r"]
                     cur_stats[dev + "-w"] += cur_stats[sID + "-w"]

               MdLoad.pop( dev )
            break

         # ----------------------------------------------------
         if case( 'total' ):
            if MdLoad :
               for dev in asset.sys_mdl :
                  if dev in MdLoad :
                     cur_stats[dev + "-r"] = 0
                     cur_stats[dev + "-w"] = 0

                     for sID in asset.md_Member[ dev ] :
                        if sID in DiskLoad :
                           cur_stats[dev + "-r"] += stats[sID][0]
                           cur_stats[dev + "-w"] += stats[sID][1]
                           cur_stats[sID + "-r"]  = stats[sID][0]
                           cur_stats[sID + "-w"]  = stats[sID][1]
                           DiskLoad.pop( sID )
                        else :
                           cur_stats[dev + "-r"] += cur_stats[sID + "-r"]
                           cur_stats[dev + "-w"] += cur_stats[sID + "-w"]
                     MdLoad.pop( dev )
                  syno_r += cur_stats[dev + "-r"]
                  syno_w += cur_stats[dev + "-w"]
            else :
               for dev in asset.sys_sdl :
                  if dev in DiskLoad :
                     cur_stats[dev + "-r"] = stats[dev][0]
                     cur_stats[dev + "-w"] = stats[dev][1]
                     DiskLoad.pop( dev )
                  syno_r += cur_stats[dev + "-r"]
                  syno_w += cur_stats[dev + "-w"]

            cur_stats["total-r"] = syno_r
            cur_stats["total-w"] = syno_w
            break

         # ----------------------------------------------------
         # --- Get I/O Values for a single HDD ...
         if dev in DiskLoad :
            cur_stats[dev + "-r"] = stats[dev][0]
            cur_stats[dev + "-w"] = stats[dev][1]
            DiskLoad.pop(dev)
         break

   # ----------------------------------------------------------
   cur_stats["none-r"] = 0
   cur_stats["none-w"] = 0

   # ----------------------------------------------------------
   # --- Calculate the given storage device I/O (MByte/sec) ...
   for i in range( 0, asset.TabMd_Max ) :
      # --- Device-Name
      dev = asset.AvrConf[ f'Md{i}Device']

      lr = i * 2 + asset.TabMD
      lw = lr + 1

      asset.SMA[asset.LastSMA][lr] = round( ( cur_stats[dev + "-r"] - asset.pre_stats[dev + "-r"] ) / 2048 / Tick, asset.rnd_SMA )
      asset.SMA[asset.LastSMA][lw] = round( ( cur_stats[dev + "-w"] - asset.pre_stats[dev + "-w"] ) / 2048 / Tick, asset.rnd_SMA )

   # --- Copy current I/O statistics to prevues ...
   asset.pre_stats        = {**cur_stats}
   asset.CurTimeDiskstats = CurTimeDiskstats

# --------------------------------------------------------------------------
# --- CPU-Load: Init first use ...
# --------------------------------------------------------------------------
def FirstLoad():

   if asset.AvrConf["PlotLoadOn"] != "Yes" and asset.AvrConf["BaseListAlwaysOn"] != "Yes" : return
   stat = io.ioRead( '/proc/stat' ).split('\n')[0].split('cpu  ')[1].split(' ')

   preLoad = 0
   for a in stat :
      preLoad += int(a)

   asset.sys_preLoad = preLoad
   asset.sys_preIdle = int(stat[3])

# ----------------------------------------------------------
# --- CPU-Load: Get Value ...
# ----------------------------------------------------------
def Get_Load():

   if asset.AvrConf["PlotLoadOn"] != "Yes" and asset.AvrConf["BaseListAlwaysOn"] != "Yes" : return
   stat = io.ioRead( '/proc/stat' ).split('\n')[0].split('cpu  ')[1].split(' ')

   curLoad = 0
   for a in stat :
      curLoad += int(a)

   curIdle = int(stat[3])
   
   try :    asset.SMA[asset.LastSMA][asset.TabLoad] = round( 100 * ( ( curLoad - asset.sys_preLoad ) - ( curIdle - asset.sys_preIdle ) ) / ( curLoad - asset.sys_preLoad ), asset.rnd_SMA )
   except : asset.SMA[asset.LastSMA][asset.TabLoad] = 0

   asset.sys_preLoad = curLoad
   asset.sys_preIdle = curIdle

# --------------------------------------------------------------------------
# --- OID Load : Init first use ...
# --------------------------------------------------------------------------
def FirstOid():

   AvrConf = asset.AvrConf

   # ---  0 = Thermal-Status
   # ---  1 = Environmental sensor 1
   # ---  2 = Environmental sensor 2
   # ---  3 = Environmental sensor 3
   # ---  4 = Environmental sensor 4
   # ---  5 = Environmental sensor 5
   # ---  6 = HDD Max. temperature
   # ---  7 = HDD Min. temperature
   # ---  8 = Average temperature
   # ---  9 = Time required in ms
   # --- 10 = MemInfo["Cached"]
   # --- 11 = Speedtest "UploadMiB"
   # --- 12 = Speedtest "DownloadMiB"
   # --- 13 = Speedtest "Ping"

   oid = [ "0.000" ] * 14
   # ----------------------------------------------------------
   # --- Initialization only if "OID_Data" does not exist or
   # --- number of values does not match the expected one.
   if not os.path.exists( asset.avrOID_DataFile ) :
      io.ioWrite( ' '.join( oid ) + '\n' , asset.avrOID_DataFile )
      io.ioRm( asset.f_OidFlag )
   else :
      tmp = io.ioRead( asset.avrOID_DataFile ).split(' ')
      if len( oid ) != len ( tmp ) :
         io.ioWrite( ' '.join( oid ) + '\n', asset.avrOID_DataFile )
         io.ioRm( asset.f_OidFlag )

   # ----------------------------------------------------------
   # --- Prepare OID-Dict :
   oid = dict()

   for sID in asset.sTabEnv :
      oid['LogEnvOn'         + sID] = AvrConf['LogEnvOn'         + sID]
      oid['LogEnvV3On'       + sID] = AvrConf['LogEnvV3On'       + sID]
      oid['LogEnvCom'        + sID] = AvrConf['LogEnvCom'        + sID]
      oid['LogEnvIp'         + sID] = AvrConf['LogEnvIp'         + sID]
      oid['LogEnvOID'        + sID] = AvrConf['LogEnvOID'        + sID]
      oid['LogEnvUser'       + sID] = AvrConf['LogEnvUser'       + sID]
      oid['LogEnvEncryption' + sID] = AvrConf['LogEnvEncryption' + sID]
      oid['LogEnvPass'       + sID] = AvrConf['LogEnvPass'       + sID]
      oid['LogEnvIp'         + sID] = AvrConf['LogEnvIp'         + sID]
      oid['LogEnvOID'        + sID] = AvrConf['LogEnvOID'        + sID]
      oid['LogEnvScale'      + sID] = AvrConf['LogEnvScale'      + sID]

   oid['LogEnvTabEnv'] = asset.sTabEnv

   try :
      while asset.OIDpid.isAlive():
         time.sleep(0.001)
   except :
      pass

   asset.OIDpid = threading.Thread(target=Oid.Pick, args=(0, oid,))
   asset.OIDpid.start()

   asset.dOID   = oid

# ----------------------------------------------------------
# --- OID Load : Get OID-Values ...
# ----------------------------------------------------------
def Get_Oid() :

   while os.path.exists( asset.f_OidFlag ) :
      time.sleep(0.001)

   # ----------------------------------------------------------
   oid = io.ioRead( asset.avrOID_DataFile ).split(' ')           # --- OID : Get the OID-Values ...

   # ---  0 = Environmental sensor 1
   # ---  1 = Environmental sensor 2
   # ---  2 = Environmental sensor 3
   # ---  3 = Environmental sensor 4
   # ---  4 = Environmental sensor 5
   # ---  5 = Thermal-Status
   # ---  6 = HDD Max. temperature
   # ---  7 = HDD Min. temperature
   # ---  8 = Average temperature
   # ---  9 = MemInfo["Cached"]
   # --- 10 = Speedtest "UploadMiB"
   # --- 11 = Speedtest "DownloadMiB"
   # --- 12 = Speedtest "Ping"
   # --- 13 = Time required in ms

   for index, iTab in enumerate( asset.iTabEnv ):                 # --- OID : Environmental sensor
      asset.SMA[asset.LastSMA][ iTab ]       = float(oid[ index ])

   asset.SMA[asset.LastSMA][asset.TabStatus] = float(oid[ 5])     # --- OID : Disk/Rack-Station Thermal-Status
   asset.SMA[asset.LastSMA][asset.TabHmax]   = float(oid[ 6])     # --- OID : HDD max
   asset.SMA[asset.LastSMA][asset.TabHmin]   = float(oid[ 7])     # --- OID : HDD min
   asset.SMA[asset.LastSMA][asset.TabHavr]   = float(oid[ 8])     # --- OID : HDD avr
   asset.sys_MemInfoCached                   = float(oid[ 9])     # --- OID : MemInfo "Cached"
   asset.sys_SpeedUploadMiB                  = float(oid[10])     # --- OID : Speedtest "UploadMiB"
   asset.sys_SpeedDownloadMiB                = float(oid[11])     # --- OID : Speedtest "DownloadMiB"
   asset.SMA[asset.LastSMA][asset.TabPing]   = float(oid[12])     # --- OID : Speedtest "Ping"
   asset.oid5 = float(oid[13])                                    # --- OID : Time required in ms

# --------------------------------------------------------------------------
# --- Check availability of Speedtest
# --------------------------------------------------------------------------
def Speedtest() :

   if asset.AvrConf['SpeedTestRegistered'] and asset.AvrConf['SpeedTestOn'] == "Yes" :
      asset.sys_SpeedRAW =     "speed"
      asset.sys_SpeedGUI = "\"# Speed #\""

      SPEED = io.ioRead( asset.SpeedOIDFile )
      if SPEED :
         SPEED = SPEED.strip().split(' ')
         asset.sys_SpeedUploadMiB   = float(SPEED[0])   # Speedtest "UploadMiB"
         asset.sys_SpeedDownloadMiB = float(SPEED[1])   # Speedtest "DownloadMiB"  
      else :
         asset.sys_SpeedUploadMiB   = 0.0               # Speedtest "UploadMiB"
         asset.sys_SpeedDownloadMiB = 0.0               # Speedtest "DownloadMiB"  
   else :
      asset.sys_SpeedRAW = ""
      asset.sys_SpeedGUI = ""
      asset.sys_SpeedUploadMiB   = None
      asset.sys_SpeedDownloadMiB = None

# ----------------------------------------------------------
# --- Get CPU-Temperatur ...
# ----------------------------------------------------------
def Get_CPU() :

   if not asset.sys_Cpu : return

   mx = 0
   for file in asset.sys_Cpu :
      mx += int( io.ioRead( file ) )

   asset.SMA[asset.LastSMA][asset.TabCpu] = round( mx / len(asset.sys_Cpu) / asset.sysPrescaler, asset.rnd_SMA )

# ----------------------------------------------------------
# --- Get Fan-Speed ...
# ----------------------------------------------------------
def Get_Fan():

   if not asset.sys_Fan : return
   
   mx = 0
   for file in asset.sys_Fan :
      mx += int( io.ioRead( file ) )
   
   asset.SMA[asset.LastSMA][asset.TabPwm] = round( mx / len(asset.sys_Fan) * 100 / 256 , asset.rnd_SMA )

# ----------------------------------------------------------
# --- Get i2c-Values ...
# ----------------------------------------------------------
def Get_i2c() :

   if not asset.sys_i2c : return

   mx = 0
   for file in asset.sys_i2c :
      mx += int( io.ioRead( file ) )

   asset.SMA[asset.LastSMA][asset.Tabi2c] = round( mx / len(asset.sys_i2c) / 1000, asset.rnd_SMA )

# --------------------------------------------------------------------------
# --- Determine the number of available network-interfaces ...
# --------------------------------------------------------------------------
def Network() :

   stats = io.ioRead( '/proc/net/dev' ).split('\n')   # Get eth devicelist incl. I/O statistics
   Eth_List = dict()

   # ----------------------------------------------------------
   # --- Prepare entries for the System & GUI ...
   if asset.sys_SpeedRAW :
      List_Raw = ' '.join((   "none",     "total",   asset.sys_SpeedRAW ))
      List_GUI = ' '.join(( "\"None\"", "\"Total\"", asset.sys_SpeedGUI ))
   else :
      List_Raw = ' '.join((   "none",     "total"   ))
      List_GUI = ' '.join(( "\"None\"", "\"Total\"" ))

   # ----------------------------------------------------------
   # --- Determine the number of available eth-interfaces ...
   sys_eth = [ x.split(':',1)[0].strip()  for x in stats if 'eth' in x ]
   sys_eth.sort()

   for dev in sys_eth :
      Eth_List[dev] = dev

   # ----------------------------------------------------------
   # --- Determine the number of available bond-interfaces ...
   sys_bond = [ x.split(':',1)[0].strip()  for x in stats if 'ovs_bond' in x ]
   sys_bond.sort()

   bond_Member = dict()

   if sys_bond :
      for dev in sys_bond :
         cnt = int( re.sub('[^0-9]+', '', dev ) ) +1

         List_Raw += " " + dev
         List_GUI += " \"# Bond " + str(cnt) + " #\""

         scripts = io.ioRead( '/etc/sysconfig/network-scripts/ifcfg-' + dev ).split('\n')

         tmp = [ x for x in scripts if 'SLAVE_LIST' in x ][0].split('=', 1 )[1].strip('"').strip().split(' ')
         bond_Member[dev] = ' '.join( tmp )

         for dev in tmp :
            lib.unset ( Eth_List, dev )
            cnt = int( re.sub('[^0-9]+', '', dev ) ) + 1
            List_Raw += " " + dev
            List_GUI += " \"   - LAN " + str(cnt) + "\""        # Location & Device Number

   tmp = [ x for x in Eth_List.keys() ]
   tmp.sort()
   for dev in tmp :
      cnt = int( re.sub('[^0-9]+', '', dev ) ) + 1
      List_Raw += " " + dev
      List_GUI += " \"LAN " + str(cnt) + "\""        # Location & Device Number

   # ----------------------------------------------------------
   # --- Determine the number of available VPN-interfaces ...
   if asset.sys_VPN_Status :

      if asset.sys_VPN_PPPoE :
         List_Raw += " ppp1"
         List_GUI += " \"PPPoE\""

      if asset.sys_VPN_PPTP :
         List_Raw += " ppp2"
         List_GUI += " \"PPTP\""

      if asset.sys_VPN_IPSec :
         List_Raw += " ppp3"
         List_GUI += " \"L2TP/IPSec\""

      if asset.sys_VPN_OpenVPN :
         List_Raw += " tun"
         List_GUI += " \"OpenVPN\""

   # ----------------------------------------------------------
   sys_cmpEthGetList = dict()
    
   for dev in List_Raw.split(' ') :
      sys_cmpEthGetList[dev] = dev

   asset.AvrConf["EthDeviceListRAW"] = List_Raw
   asset.AvrConf["EthDeviceListGUI"] = List_GUI
   # ----------------------------------------------------------

   asset.sys_eth           = sys_eth
   asset.sys_bond          = sys_bond
   asset.bond_Member       = bond_Member
   asset.sys_cmpEthGetList = sys_cmpEthGetList

# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
# sys.exit()
# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
