# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#
import os
import sys
import glob
import json
import time

import asset
import basics   as io

import lib
from   lib      import switch, case

import Sys
import Usb

# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
vStorage       = '220401.001'    # >> enumStorage
tStorage       =  3600

iBootUp        =   900
iReScanTime    =  3600
iRawErrorRate  = 25000

vDrives        = '220402.003'    # >> enumDrives
tDrives        = 2592000         # << 30 days

sError         = 'Read Device Identity failed:'

# -----------------------------------------------------------------------------------
# --- Class to handle Device S.M.A.R.T  Informations
# -----------------------------------------------------------------------------------
class Smart( ) :

   # --------------------------------------------------------------------------------
   # --------------------------------------------------------------------------------
   def __init__( self, Debug=False, forceReload=False ) :

      if not bool( asset.SynoInfo ) : asset.SynoInfo = lib.FileConfig( asset.sysSynoConfig )

      if not isinstance( forceReload, bool ) :
         forceReload = False

      self.Debug = Debug
      # -----------------------------------------------------------------------------
      # --- Get '/etc/synoinfo.conf' ...
      self.sysConf      = { key : value[:] for key, value in asset.SynoInfo.items() }
      self.ModelName    = self.sysConf.get( 'upnpmodelname', False )
      self.UpTime       = int( float( io.ioRead( '/proc/uptime' ).split(' ')[0] ) )

      self.rDict        = dict()
      self.apiDict      = dict()

      self.iRaw         = []     # List of "all" found devices
      self.iSdl         = []     # List of valid devices
      self.iAdd         = []     # New devices that need to be added
      self.iDel         = []     # Devices that are no longer available and must be removed
      self.iBad         = []     # Devices that do not provide SMART information

      self.sCurDict     = dict()
      self.sUsbDict     = dict()
      self.sDiskDB      = dict()
      self.iEsc         = []

      self.iLocInt      = dict()
      self.iLocBox      = dict()
      self.iLocUsb      = []
      self.iLocEsata    = []

      self.smartInfo    = dict()
      self.smartAttr    = dict()
      self.smartConf    = dict()
      self.smartError   = dict()
      self.SynoUSB      = dict()
      self.SynoEsata    = dict()

   # --------------------------------------------------------------------------------
   # --- Reset "Predict warning" for this Device and update DataBase ...
   # --------------------------------------------------------------------------------
   def resetDenote( self, sDev ) :

      if not self.rDict   : self.getSynoStorage()
      if not self.sDiskDB : self.getDataBase()

      if self.Debug : 
         print ( f'| SYS: SMART Reset "Predict warning" for Drive : {sDev}' )
         print ( f'| SYS: SMART Update sAttr from Drive : {sDev}' )

      sDisk = self.rDict[ 'storage' ][ 'disks' ][ sDev ]
      sDisk[ 'sAttr'    ] = self.getSmartAttr( sDev )
      sDisk[ 'sPredict' ] = self.setDataBasePredict( sDev )
      sDisk[ 'denote'   ] = False

      # -----------------------------------------------------------------------------
      self.rDict[ 'storage' ][ 'disks' ][ sDev ].update( sDisk )
      io.ioWrite( json.dumps( self.rDict, sort_keys=True, indent=4 ), asset.avrEnumStorage )

   # --------------------------------------------------------------------------------
   # --- Get Device Informations for a singel Drive by name ...
   # --------------------------------------------------------------------------------
   def getDiskInfo( self, sDev ) :

      if not self.rDict : self.getSynoStorage()
      if self.Debug : print ( f'| SYS: SMART Get Device Informations for Drive : {sDev}' )

      fChange = False
      sDisk = self.rDict[ 'storage' ][ 'disks' ][ sDev ]

      # -----------------------------------------------------------------------------
      # --- Check SMART "Error" information for device and if necessary ...
      if not sDisk[ 'sError' ] :
         if self.Debug : print ( f'| SYS: SMART Add sError to Drive : {sDev}' )
         sDisk[ 'sError' ] = self.getSmartError( sDev )
         fChange = True

      # -----------------------------------------------------------------------------
      # --- Check SMART information for device and if necessary ...
      if not sDisk[ 'sInfo' ] :
         if self.Debug : print ( f'| SYS: SMART Add sInfo to Drive : {sDev}' )
         sDisk[ 'sInfo' ] = self.getSmartInfo( sDev )
         fChange = True

      if not sDisk[ 'sConf' ] :
         if self.Debug : print ( f'| SYS: SMART Add sConf to Drive : {sDev}' )
         sDisk[ 'sConf' ] = self.getSmartConf( sDev )
         fChange = True

      fOut = False
      # -----------------------------------------------------------------------------
      # --- Check SMART vendor-specific Attributes and update if necessary ...
      if not sDisk[ 'sAttr' ] :
         if self.Debug : print ( f'| SYS: SMART Add sAttr to Drive : {sDev}' )
         fOut = True
      elif int( time.time() ) - sDisk[ 'sAttr' ][ 'time' ] >= iReScanTime :
         if self.Debug : print ( f'| SYS: SMART Update sAttr from Drive : {sDev}' )
         fOut = True

      if fOut :
         sDisk[ 'sAttr' ] = self.getSmartAttr( sDev )
         fChange = True

      # -----------------------------------------------------------------------------
      # Check SMART Predict Attributes and update if necessary ...
      if not sDisk[ 'sPredict' ] :
         if self.Debug : print ( f'| SYS: SMART Add sPredict to Drive : {sDev}' )
         sDisk[ 'sPredict' ] = self.getPredict( sDev, sDisk )
         if sDisk[ 'sPredict' ] : fChange = True
      else :
         if self.Debug : print ( f'| SYS: SMART Check sPredict of Drive : {sDev}' )
         sDenote = self.getDenoteAttr( sDev, sDisk=sDisk )
         if sDenote :
            if self.Debug :
               print ( f'| SYS: SMART "sPredict warning" for Drive : {sDev}' )
               print ( json.dumps( sDenote, sort_keys=True, indent=4 ) )

            sDisk[ 'denote' ] = sDenote
            fChange = True

      # -----------------------------------------------------------------------------
      if fChange :

         self.rDict[ 'storage' ][ 'disks' ][ sDev ].update( sDisk )

         # --------------------------------------------------------------------------
         io.ioWrite( json.dumps( self.rDict, sort_keys=True, indent=4 ), asset.avrEnumStorage )

      # -----------------------------------------------------------------------------
      return dict( sDisk )

   # --------------------------------------------------------------------------------
   # --- Get (Init) Syno-Storage informations ...
   # --------------------------------------------------------------------------------
   def getSynoStorage( self, scan=True, cache=True ) :
      # -----------------------------------------------------------------------------
      if not isinstance( cache, bool ) :
         cache = True

      # -----------------------------------------------------------------------------
      # --- Load "BaseStorage (enumStorage)" :
      # -----------------------------------------------------------------------------
      if os.path.exists( asset.avrEnumStorage ) and cache :
         if self.Debug : print ( '| SYS: SMART Load "Syno-Storage" from Cache' )
         self.rDict = json.loads( io.ioRead( asset.avrEnumStorage ) )

         sVersion = self.rDict[ 'storage' ].get( 'version', '000000.000' )
         if sVersion != vStorage :
            if self.Debug : print   (  f'| SYS: SMART Update "enumStorage" from v{sVersion} to v{vStorage}' )
            else          : lib.Error ( f'SYS: Update "enumStorage" from v{sVersion} to v{vStorage}' )

            Sys.RotateClear( asset.avrEnumStorage )
            self.rDict = dict()

      # -----------------------------------------------------------------------------
      if not self.rDict :
         self.apiSynoStorage()

      # -----------------------------------------------------------------------------
      # --- Scan for "sdl"-Devices ...
      if scan : self.scanDrives()

      # -----------------------------------------------------------------------------
      return   {
                  'disks'  : self.rDict[ 'storage' ].get( 'disks', dict() ),
                  'pools'  : self.rDict[ 'storage' ].get( 'pools', dict() ),
                  'cache'  : self.rDict[ 'storage' ].get( 'cache', dict() ),
               }

   # --------------------------------------------------------------------------------
   # --- Get current list of "sdl"-Devices ...
   # --------------------------------------------------------------------------------
   def scanDrives( self ) :
      io.touch( asset.f_Heartbeat )

      if os.path.exists( asset.f_VirtualDSM ):
         return False

      if not self.rDict   : self.getSynoStorage( scan=False )

      self.iRaw         = []
      self.iSdl         = []
      self.iAdd         = []
      self.iDel         = []
      self.iBad         = []

      # -----------------------------------------------------------------------------
      # --- Get internal "sd" devices ...
      if self.Debug : print ( '| SYS: SMART Scan System for "sdl"-Devices ...' )
      # -----------------------------------------------------------------------------
      for sID in sorted( glob.glob( '/dev/sd[a-z]' ) ) :
         self.iRaw.append( sID.split('/')[-1] )

      for sID in sorted( glob.glob( '/dev/sd[a-z][a-z]' ) ) :
         self.iRaw.append( sID.split('/')[-1] )

      # -----------------------------------------------------------------------------
      # --- Get internal "sata" devices ...
      if self.Debug : print ( '| SYS: SMART Scan System for "sata"-Devices ...' )
      # -----------------------------------------------------------------------------
      for sID in sorted( glob.glob( '/dev/sata[0-9]' ) ) :
         self.iRaw.append( sID.split('/')[-1] )

      for sID in sorted( glob.glob( '/dev/sata[0-9][0-9]' ) ) :
         self.iRaw.append( sID.split('/')[-1] )

      # -----------------------------------------------------------------------------
      # --- Get internal "usb" (sata) devices ...
      if self.Debug : print ( '| SYS: SMART Scan System for "usb"-Devices ...' )
      # -----------------------------------------------------------------------------
      for sID in sorted( glob.glob( '/dev/usb[0-9]' ) ) :
         self.iRaw.append( sID.split('/')[-1] )

      for sID in sorted( glob.glob( '/dev/usb[0-9][0-9]' ) ) :
         self.iRaw.append( sID.split('/')[-1] )

      # -----------------------------------------------------------------------------
      # --- Get internal NVMe devices ...
      if self.Debug : print ( '| SYS: SMART Scan System for "nvme"-Devices ...' )
      # -----------------------------------------------------------------------------
      for sID in sorted( glob.glob( '/dev/nvme[0-9]n[0-9]' ) ) :
         self.iRaw.append( sID.split('/')[-1] )

      # -----------------------------------------------------------------------------
      # --- Get time stamp for each device ...
      for sDev in self.iRaw :
         self.sCurDict[ sDev ] = { 'time' : int( os.path.getmtime( '/dev/' + sDev ) ) }

      # -----------------------------------------------------------------------------
      sDisks = self.rDict['storage'].get( 'disks', dict() )

      # -----------------------------------------------------------------------------
      iEnum = self.rDict.get( 'enum', False )
      if iEnum :
         iLocUsb   = iEnum.get( 'iLocUsb',   [] )
         iLocEsata = iEnum.get( 'iLocEsata', [] )
      else :
         iLocUsb   = []
         iLocEsata = []

      # -----------------------------------------------------------------------------
      for sDev in sDisks :
         if sDev not in self.iRaw :
            self.iDel.append( sDev )

      # -----------------------------------------------------------------------------
      for sDev in self.iRaw :
         # --------------------------------------------------------------------------
         if sDev not in sDisks :
            # -----------------------------------------------------------------------
            # --- Device is new or unknown, mark it as "must be added" ...
            self.iAdd.append( sDev )
            self.iSdl.append( sDev )
            continue
         # --------------------------------------------------------------------------
         if sDisks[ sDev ].get( 'isBad', True ) : 
            # -----------------------------------------------------------------------
            # --- Device is bad, probably a USB-Stick ... we ignore it ...
            self.iBad.append( sDev )
            continue
         # --------------------------------------------------------------------------
         if self.sCurDict[ sDev ][ 'time' ] == sDisks[ sDev ].get( 'time', 0 ) :
            # -----------------------------------------------------------------------
            # --- Time stamps match, we take note of the device ...
            self.iSdl.append( sDev )
            continue
         elif sDisks[ sDev ].get( 'type', False ) in ['usb', 'esata' ] :
            # -----------------------------------------------------------------------
            # --- The drive is known, but the time stamp does not match ...
            self.iDel.append( sDev )
            self.iAdd.append( sDev )
            self.iSdl.append( sDev )
            continue
         # --------------------------------------------------------------------------
         sType = sDisks[ sDev ].get( 'type', False )
         if sType and sType in [ 'internal', 'ebox' ] and self.UpTime >= iBootUp and not io.check( asset.f_NewsAvrPKG ) :

            # -----------------------------------------------------------------------
            # --- Get "sModel" & "sSerial" for "quick"-compare ...
            sLine = lib.getCMD( 'hdparm -i /dev/' + sDev ).strip().split( '\n' )[2]
            sLine = [ x.strip() for x in sLine.split( ',' ) ]

            sModel  = sLine[0].split( '=', 1 )[-1]
            sSerial = sLine[2].split( '=', 1 )[-1]

            # -----------------------------------------------------------------------
            # --- Remove Vendor ID ...
            if ' ' in sModel : sModel  = sModel.split( ' ', 1 )[-1]
            curID = ' '.join(( sModel, sSerial ))
            preID = ' '.join(( sDisks[ sDev ][ 'model' ], sDisks[ sDev ][ 'serial' ] ))

            if curID != preID :
               # --------------------------------------------------------------------
               # --- Time stamp does not match. This should not happen except during a cold start or reboot ...
               if self.Debug :
                  print  ( '| SYS: SMART Unexpected change of "[{0}] {name}"'.format( sDev, **sDisks[ sDev ] ) )
               else :
                  msg  = time.strftime('%Y.%m.%d %H:%M:%S ')
                  msg += 'SYS: Unexpected change of "[{0}] {name}"\n'.format( sDev, **sDisks[ sDev ] )
                  msg += 'TimeStamp :\n'
                  msg += '{0}\n'.format( json.dumps( self.sCurDict[ sDev ], sort_keys=True, indent=4 ) )
                  msg += 'DiskInfo  :\n'
                  msg += '{0}\n'.format( json.dumps( sDisks[ sDev ]       , sort_keys=True, indent=4 ) )
                  io.ioAdd( msg , asset.ERROR )

            # -----------------------------------------------------------------------
            self.iDel.append( sDev )
            self.iAdd.append( sDev )
            self.iSdl.append( sDev )

            continue
         # --------------------------------------------------------------------------
         if not sType :
            # -----------------------------------------------------------------------
            # --- We probably have a problem with the device or its stored information ...
            msg  = time.strftime('%Y.%m.%d %H:%M:%S SYS: "StoragePool" or "DiskInfo" of Device [{0}] is malformed :\n'.format( sDev ) )
            msg += 'DiskInfo  :\n'
            msg += '{0}\n'.format( json.dumps( sDisks.get( sDev, dict() ) , sort_keys=True, indent=4 ) )
            msg += 'StoragePool :\n'
            msg += io.ioRead( asset.avrEnumStorage )
            io.ioAdd( msg , asset.ERROR )
            Sys.RotateClear( asset.avrEnumStorage )
            io.ioRm( asset.avrEnumStorage )
            sys.exit()

         # --------------------------------------------------------------------------
         # --- At this point, the device can only be a USB or eSATA drive ... we take note of it ...
         self.iSdl.append( sDev )

      # -----------------------------------------------------------------------------
      fChange = False

      # -----------------------------------------------------------------------------
      # --- Remove expired or no longer existing drives ...
      for sDev in self.iDel :
         if self.Debug : print ( f'| SYS: SMART Remove expired or no longer existing drive : {sDev}' )
         del sDisks[ sDev ]
         if sDev in iLocUsb   : iLocUsb.remove  ( sDev )
         if sDev in iLocEsata : iLocEsata.remove( sDev )

         fChange = True

      # -----------------------------------------------------------------------------
      # --- Try to map a new hard drive :
      for sDev in self.iAdd :
         fChange = True
         sTyp    = False
         serial  = False

         if not self.sUsbDict :
            self.sUsbDict = Usb.getUsbDevices( iNice=True )

         # --------------------------------------------------------------------------
         # --- Test if Drive-Type is "USB" ...
         if sDev in self.sUsbDict :
            sTyp   = 'usb'
            serial = self.getSmartInfoName( sDev, 'Serial Number' )

         # --------------------------------------------------------------------------
         # --- Test if Drive-Type is "eSATA" ...
         if not sTyp :
            if not self.SynoEsata : self.getSynoEsata()

            if sDev in self.SynoEsata :
               sTyp   = 'esata'
               serial = self.getSmartInfoName( sDev, 'Serial Number' )

         # --------------------------------------------------------------------------
         # --- Test if Drive-Type is "internal" or "ebox" ...
         if not sTyp :
            if not self.apiDict : self.apiSynoStorage()

            if sDev in self.apiDict :
               sTyp   = self.apiDict[ sDev ].get( 'type',   False )
               serial = self.apiDict[ sDev ].get( 'serial', False )

         # --------------------------------------------------------------------------
         iTime = self.sCurDict[ sDev ][ 'time' ]

         if not serial or not sTyp :
            sDisks[ sDev ] =  {
                                 'isBad'  : True,
                                 'time'   : iTime,
                              }

            self.iBad.append( sDev )
            if sDev in self.iSdl : self.iSdl.remove( sDev )
            continue

         if self.Debug : print ( f'| SYS: SMART Add "new" device : {sDev}' )
         # --------------------------------------------------------------------------
         iTime = int( os.path.getmtime( '/dev/' + sDev ) )

         while switch( sTyp ) :
            # -----------------------------------------------------------------------
            if case( 'internal', 'ebox', 'nvme' ) :

               sID = self.apiDict[ sDev ]

               if ' ' in sID[ 'model'  ] :
                  sModel = sID[ 'model'  ].split( ' ', 1 )[-1]
                  if self.Debug : print ( f'| SYS: SMART Adjusted model name : {sModel} [{sID[ "model" ]}] for drive : {sDev}' )
                  sID[ 'model' ] = sModel

               if sTyp == 'ebox' : sNumber = sID[ 'name' ].split( ' ' )[ 1]
               else              : sNumber = sID[ 'name' ].split( ' ' )[-1]

               sDisks[ sDev ] =  {
                                    'isBad'     : False,
                                    'isSSD'     : sID[ 'isSSD'  ],         # false / true
                                    'sInfo'     : {},                      # /usr/bin/smartctl "-i" --info       : Show identity information for device
                                    'sConf'     : {},                      # /usr/bin/smartctl "-g" --get        : Get device setting
                                    'sAttr'     : {},                      # /usr/bin/smartctl "-A" --attributes : Show device SMART vendor-specific Attributes and values
                                    'sError'    : {},                      # /sbin/nvme error-log /dev/nvme0n1   : Get Error Log Entries (only NVMe)
                                    'name'      : sID[ 'name'   ],         # Laufwerk 9 / Drive 9
                                    'number'    : sNumber,                 # logical drive number
                                    'vendor'    : sID[ 'vendor' ],         # Seagate
                                    'loc'       : sID[ 'loc'    ],         # RS3617xs
                                    'type'      : sID[ 'type'   ],         # internal / ebox / nvme =>> usb / esata
                                    'serial'    : sID[ 'serial' ],         # "1EGX4VYZ"
                                    'model'     : sID[ 'model'  ],         # "WD100EFAX-68LHPN0",
                                    'size'      : sID[ 'size'   ],
                                    'isUSB'     : {},
                                    'sPredict'  : {},
                                    'time'      : iTime,
                                    'denote'    : False,
                              }
               break

            # -----------------------------------------------------------------------
            if case( 'usb', 'esata' ) :

               sLoc    = self.ModelName
               sVendor = self.getSmartInfoName( sDev, 'Model Family'  )  # Western Digital Red   or    Seagate IronWolf Pro or    Maxtor DiamondMax 10 (ATA/133 and SATA/150)
               sModel  = self.getSmartInfoName( sDev, 'Device Model'  )  # WDC WD40EFRX-68WT0N0  or    ST10000NE0004-2GT11L or    Maxtor 6L250R0

               if ' ' in sModel :
                  model  = sModel.split( ' ', 1 )[-1]
                  vendor = sModel.split( ' ', 1 )[0]
                  if self.Debug : print ( f'| SYS: SMART Adjusted model name : {model} [{sModel}] for drive : {sDev}' )
                  if self.Debug : print ( f'| SYS: SMART Manufacturer "{vendor}" taken from model name [{sModel}] for drive : {sDev}' )
               else :
                  model  = sModel
                  vendor = sVendor.split( ' ', 1 )[0]


               isSsd  = self.getSmartInfoName( sDev, 'Rotation Rate' )
               sSize  = self.getSmartInfoName( sDev, 'User Capacity' )

               if sSize : sSize = sSize.split( '[', 1 )[-1].strip( ']' )
               else     : sSize = 'NA'

               if isSsd :
                  if 'Solid' in isSsd : isSsd = True
                  else                : isSsd = False

               if sTyp == 'usb' :
                  if not self.SynoUSB : self.getSynoUsb()
                  sName = self.SynoUSB.get( sDev, 'USB Disk not mounted' )
                  sUSB  = self.sUsbDict[ sDev ]
                  iLocUsb.append( sDev )
               else :
                  sName = self.SynoEsata.get( sDev, 'eSATA Disk not mounted' )
                  sUSB  = {}
                  iLocEsata.append( sDev )

               sDisks[ sDev ] =  {
                                    'isBad'     : False,
                                    'isSSD'     : isSsd,                   # false / true
                                    'sInfo'     : self.smartInfo[ sDev ],  # /usr/bin/smartctl "-i" --info       : Show identity information for device
                                    'sConf'     : {},                      # /usr/bin/smartctl "-g" --get        : Get device setting
                                    'sAttr'     : {},                      # /usr/bin/smartctl "-A" --attributes : Show device SMART vendor-specific Attributes and values
                                    'sError'    : {},                      # /sbin/nvme error-log /dev/nvme0n1   : Get Error Log Entries (only NVMe)
                                    'name'      : sName,                   # Laufwerk 9 / Drive 9
                                    'number'    : sName.split( ' ' )[-1],  # logical drive number
                                    'vendor'    : vendor,                  # Seagate
                                    'loc'       : sLoc,                    # RS3617xs
                                    'type'      : sTyp,                    # internal / ebox =>> usb / esata
                                    'serial'    : serial,                  # "1EGX4VYZ"
                                    'model'     : model,                   # "WD100EFAX-68LHPN0",
                                    'size'      : sSize,                   # 4,000,787,030,016 bytes [4.00 TB]
                                    'isUSB'     : sUSB,
                                    'sPredict'  : {},
                                    'time'      : iTime,
                                    'denote'    : False,
                                 }
               break

            # --- Default's ---------------------------------------------------------
            break

      # -----------------------------------------------------------------------------
      if fChange :
         self.rDict[ 'enum' ].update   (
                                          {
                                             'iLocUsb'   : iLocUsb,
                                             'iLocEsata' : iLocEsata,
                                          }
                                       )

         self.rDict[ 'storage' ].update(
                                         {
                                             'disks'     : sDisks,
                                          }
                                       )
      # -----------------------------------------------------------------------------
      # --- Check storage pools for "isBad" drives and mark them if necessary ...
      for iID in sorted( self.rDict[ 'storage' ]['pools'] ) :
         isBad = False

         # --------------------------------------------------------------------------
         for sDev in sorted( self.rDict[ 'storage' ]['pools'][ iID ][ 'sdl' ] ) :
            if self.rDict[ 'storage' ][ 'disks' ][ sDev ][ 'isBad' ] :
               isBad = True
               if self.Debug : print   ( f'| SYS: SMART Unsupported Drive [{sDev}] found in Storage Pool [{iID}]' )
               else          : lib.Error ( f'SYS: Unsupported Drive [{sDev}] found in Storage Pool [{iID}]' )

         # --------------------------------------------------------------------------
         if isBad :
            self.rDict[ 'storage' ]['pools'][ iID ][ 'isBad' ] = True
            if self.Debug : print   ( f'| SYS: SMART Storage Pool [{iID}] has been marked as unsupported and will be ignored' )
            else          : lib.Error ( f'SYS: Storage Pool [{iID}] has been marked as unsupported and will be ignored' )

         # --------------------------------------------------------------------------
         io.ioWrite( json.dumps( self.rDict, sort_keys=True, indent=4 ), asset.avrEnumStorage )

      # -----------------------------------------------------------------------------
      # -----------------------------------------------------------------------------
      if self.Debug :
         print ( f'| SYS: SMART iAdd : {self.iAdd}' )
         print ( f'| SYS: SMART iBad : {self.iBad}' )
         print ( f'| SYS: SMART iDel : {self.iDel}' )
         print ( f'| SYS: SMART iSdl : {self.iSdl}' )
         print ( f'| SYS: SMART iRaw : {self.iRaw}' )

      return { 'iAdd' : self.iAdd, 'iDel' : self.iDel, 'iSdl' : self.iSdl, 'iBad' : self.iBad, 'iRaw' : self.iRaw }

   # --------------------------------------------------------------------------------
   # --- Init Base Storage informations ...
   # --------------------------------------------------------------------------------
   def apiSynoStorage( self ) :

      if self.Debug : print ( '| SYS: SMART Init : --exec "api=SYNO.Storage.CGI.Storage"' )

      # -----------------------------------------------------------------------------
      self.apiDict = lib.getCMD( '/usr/syno/bin/synowebapi --exec api=SYNO.Storage.CGI.Storage method=load_info version=1 2>/dev/null' )
      self.apiDict = json.loads( self.apiDict )[ 'data' ]

      # -----------------------------------------------------------------------------
      self.rDict[ 'storage' ] =  {
                                    'version'   : vStorage,
                                    'time'      : int( time.time() ),
                                 }

      # -----------------------------------------------------------------------------
      rTemp = dict()
      for sID in self.apiDict[ 'storagePools' ] :

         rTemp[ sID['num_id'] ] =   {
                                       'isBad'  : False,
                                       'loc'    : sID[ 'container' ],   # "internal" / "ebox"
                                       'desc'   : sID[ 'desc'      ],   # IronWolf (Raid 6 - btrfs)
                                       'sdl'    : sID[ 'disks'     ],   # [ "sdc", "sdd", "sde", "sdf", "sdg", "sdh", "sdi", "sdj", "sdk", "sdl" ]
                                       'time'   : int( time.time() ),
                                    }

      self.rDict[ 'storage' ].update( { 'pools'     : rTemp } )

      # -----------------------------------------------------------------------------
      rTemp = dict()
      for sID in self.apiDict.get( 'ssdCaches', [] ) :

         sID['num_id'] += 1
         rTemp[ sID['num_id'] ] =   {
                                       'isBad'  : False,
                                       'loc'    : sID[ 'container' ],   # "internal" / "ebox"
                                       'desc'   : sID[ 'id'        ],   # "ssd_1",
                                       'sdl'    : sID[ 'disks'     ],   # [ "sda", "sdb" ]
                                       'time'   : int( time.time() ),
                                    }

      self.rDict[ 'storage' ].update( { 'cache' : rTemp } )

      # -----------------------------------------------------------------------------
      rTemp   = dict()
      iLocInt = dict()
      iLocBox = dict()
      iLocM2  = []

      for sID in self.apiDict[ 'disks' ] :
         sDev  = sID.get( 'id', 'NA' )

         # --------------------------------------------------------------------------
         # --- Skip (ignore) all "unknown" SATA-Devices ...
         if not sDev.startswith( 'sd' ) and not sDev.startswith( 'sata' ) and not sDev.startswith( 'nvme' ): continue

         sLoc  = sID[ 'container' ][ 'str'  ]
         sTyp  = sID[ 'container' ][ 'type' ]

         iSize = int( sID[ 'size_total' ] ) / 1000000000

         if iSize < 1000 : iSize = '%d GB'   % int  ( iSize )
         else            : iSize = '%.2f TB' % float( iSize / 1000 )

         if sDev.startswith( 'nvme' ) :
            sLoc = 'M.2 Slot'
            sTyp = 'nvme'

         rTemp[ sDev ] =   {
                                 'isSSD'     : sID[ 'isSsd' ],                # false / true
                                 'name'      : sID[ 'longName' ],             # Laufwerk 9 / Drive 9
                                 'vendor'    : sID[ 'vendor' ].strip(),       # Seagate
                                 'loc'       : sLoc,                          # RS3617xs
                                 'type'      : sTyp,                          # internal / ebox =>> usb / esata
                                 'serial'    : sID[ 'serial' ],               # "1EGX4VYZ"
                                 'model'     : sID[ 'model' ].strip(),        # "WD100EFAX-68LHPN0",
                                 'size'      : iSize,
                              }

         # --------------------------------------------------------------------------
         while switch( sTyp ) :
            # -----------------------------------------------------------------------
            if case( 'internal' ) :
               try    : iLocInt[ sLoc ].append( sDev )
               except : iLocInt[ sLoc ] =     [ sDev ]
               break
            # -----------------------------------------------------------------------
            if case( 'nvme' ) :
               iLocM2.append( sDev )
               break
            # -----------------------------------------------------------------------
            if case( 'ebox' ) :
               try    : iLocBox[ sLoc ].append( sDev )
               except : iLocBox[ sLoc ] =     [ sDev ]
               break
            # --- Default's ---------------------------------------------------------
            break

      # -----------------------------------------------------------------------------
      self.rDict[ 'enum' ] =  {
                                 'iLocInt'   : iLocInt,
                                 'iLocM2'    : iLocM2,
                                 'iLocBox'   : iLocBox,
                              }
      self.apiDict = rTemp

      # -----------------------------------------------------------------------------
      io.ioWrite( json.dumps( self.rDict, sort_keys=True, indent=4 ), asset.avrEnumStorage )

   # --------------------------------------------------------------------------------
   # --- Get Device "Error"-Informations (only NVMe ) by Device :
   # --------------------------------------------------------------------------------
   def getSmartError( self, sDev, opt=False ) :
      if not sDev.startswith( 'nvme' ) : return False

      if sDev not in self.smartError or opt :
         io.touch( asset.f_Heartbeat )

         # --------------------------------------------------------------------------
         # --- Error Log Entries :
         if self.Debug : print ( f'| SYS: SMART /sbin/nvme error-log /dev/{sDev}' )
         sList = lib.getCMD( f'/sbin/nvme error-log /dev/{sDev}' ).strip().split( '\n' )
         sList = sList[1:]
         iList = int( ( len( sList ) - 1 ) / 11 )

         # --------------------------------------------------------------------------
         iKey   = []
         sError = dict()
         # --------------------------------------------------------------------------

         for cnt in range( 0, iList ) :

            sEntry = sList[3:11]
            sList  = sList[11:]

            iAttr  = []

            for line in sEntry :
               iAttr.append( line.split( ':', 1 )[1].strip() )
               if '(' in iAttr[-1] : iAttr[-1] = iAttr[-1].split( '(', 1 )[0]

            iKey.append( f'Entry {cnt}' )
            sError[ f'Entry {cnt}' ] = iAttr

         # --------------------------------------------------------------------------
         self.smartError[ sDev ] =  {
                                       'iKey'   : iKey,
                                       'sValue' : sError,
                                    }

      # -----------------------------------------------------------------------------
      return self.smartError[ sDev ]

   # --------------------------------------------------------------------------------
   # --- Get Syno "eSATA" Device Informations :
   # --------------------------------------------------------------------------------
   def getSynoEsata( self ) :
      io.touch( asset.f_Heartbeat )

      if self.Debug : print ( '| SYS: SMART Init : --exec "api=SYNO.Core.ExternalDevice.Storage.eSATA"' )

      rTemp = lib.getCMD( '/usr/syno/bin/synowebapi --exec api=SYNO.Core.ExternalDevice.Storage.eSATA version=1 method=list 2>/dev/null' )
      rTemp = json.loads( rTemp )[ 'data' ][ 'devices' ]

      for sID in rTemp :
         sDev = sID[ 'dev_id' ]
      #  self.SynoEsata[ sDev ] = sID[ 'dev_title' ]
         self.SynoEsata[ sDev ] = f'eSATA Disk {sID[ "dev_title" ][-1]}'

      if not self.SynoEsata :
         self.SynoEsata = { 'none' : 'NA' }

   # --------------------------------------------------------------------------------
   # --- Get Syno "USB" Device Informations :
   # --------------------------------------------------------------------------------
   def getSynoUsb( self ) :
      io.touch( asset.f_Heartbeat )

      if self.Debug : print ( '| SYS: SMART Init : --exec "api=SYNO.Core.ExternalDevice.Storage.USB"' )

      rTemp = lib.getCMD( '/usr/syno/bin/synowebapi --exec api=SYNO.Core.ExternalDevice.Storage.USB version=1 method=list 2>/dev/null' )
      rTemp = json.loads( rTemp )[ 'data' ][ 'devices' ]

      # -----------------------------------------------------------------------------
      # --- Need to sort by 'dev_title' !!
      for sID in rTemp :
         sDev = sID[ 'dev_id' ]
         self.SynoUSB[ sDev ] = sID[ 'dev_title' ]

      if not self.SynoUSB :
         self.SynoUSB = { 'none' : 'NA' }

   # --------------------------------------------------------------------------------
   # --- Get Device Informations (SMART) by Device :
   # --------------------------------------------------------------------------------
   def getSmartInfo( self, sDev, opt=False ) :
      if sDev not in self.smartInfo or opt :
         io.touch( asset.f_Heartbeat )

         if sDev.startswith( 'nvme' ) :
            # -----------------------------------------------------------------------
            # --- NVME Identify Controller :
            if self.Debug : print ( f'| SYS: SMART /sbin/nvme id-ctrl /dev/{sDev}' )
            sList = lib.getCMD( '/sbin/nvme id-ctrl /dev/' + sDev ).strip().split( '\n' )
            sList = sList[1:]
         else :
            # -----------------------------------------------------------------------
            # --- Get identity information for device
            if self.Debug : print ( f'| SYS: SMART smartctl -d sat -T permissive -i /dev/{sDev}' )
            sList = lib.getCMD( '/usr/bin/smartctl -d sat -T permissive -i /dev/' + sDev ).strip().split( '\n' )

            if sList[3].startswith( sError ) :
               self.smartInfo[ sDev ] = False
               return False
            sList = sList[4:]

         # --------------------------------------------------------------------------
         iKey   = []
         sInfo  = dict()
         iPower = False
         # --------------------------------------------------------------------------
         for sLine in sList :

            if not iPower and sLine.startswith( 'ps ' ) :
               iPower = sLine
               continue

            if iPower :
               sLine = f'{iPower} {sLine}'
               iPower = False

               # --- On "split"-error ... skip this line ...
               try    : sKey, sValue = sLine.split( ':', 1 )
               except : continue

               sKey   = ' '.join( sKey.split() )
               sValue = ' '.join( sValue.split() )

            else :
               # --- On "split"-error ... skip this line ...
               try    : sKey, sValue = sLine.split( ':', 1 )
               except : continue

            iKey.append( sKey.strip() )
            sInfo[ sKey.strip() ] = sValue.strip()

      # -----------------------------------------------------------------------------
      self.smartInfo[ sDev ] =   {
                                    'iKey'   : iKey,
                                    'sValue' : sInfo,
                                 }

      return self.smartInfo[ sDev ]

   # --------------------------------------------------------------------------------
   # --- Get a Single Device Information (SMART) by Name :
   # --------------------------------------------------------------------------------
   def getSmartInfoName( self, sDev, sName, sDef=False ) :
      if sDev not in self.smartInfo : self.getSmartInfo( sDev )

      if not self.smartInfo[ sDev ] : return sDef

      return self.smartInfo[ sDev ][ 'sValue' ].get( sName, sDef )

   # --------------------------------------------------------------------------------
   # --- Get Device Configuration (SMART) by Device :
   # --------------------------------------------------------------------------------
   def getSmartConf( self, sDev, opt=False ) :
      if sDev not in self.smartConf or opt :
         io.touch( asset.f_Heartbeat )

         if sDev.startswith( 'nvme' ) :
            # -----------------------------------------------------------------------
            # --- NVME Identify Namespace :
            if self.Debug : print ( f'| SYS: SMART /sbin/nvme id-ns /dev/{sDev}' )
            sList = lib.getCMD( f'/sbin/nvme id-ns /dev/{sDev}' ).strip().split( '\n' )
            sList = sList[1:]
         else :
            # -----------------------------------------------------------------------
            # --- Get device setting :
            if self.Debug : print ( f'| SYS: SMART smartctl -d sat -T permissive -g all /dev/{sDev}' )
            sList = lib.getCMD( f'/usr/bin/smartctl -d sat -T permissive -g all /dev/{sDev}' ).strip().split( '\n' )
            sList = sList[4:]

         iKey  = []
         sInfo = dict()
         # --------------------------------------------------------------------------
         for sLine in sList :
            # --- On "split"-error ... skip this line ...
            try    : sKey, sValue = sLine.split( ':', 1 )
            except : continue

            sKey = ' '.join( sKey.split() )

            iKey.append( sKey.strip() )
            sInfo[ sKey.strip() ] = sValue.strip()

      # -----------------------------------------------------------------------------
      self.smartConf[ sDev ] =   {
                                    'iKey'   : iKey,
                                    'sValue' : sInfo,
                                 }

      return self.smartConf[ sDev ]

   # --------------------------------------------------------------------------------
   # --- Get Device Attribute (SMART) by Device :
   # --------------------------------------------------------------------------------
   def getSmartAttr( self, sDev, opt=False ) :

      fAttr = False
      sDisk = self.rDict[ 'storage' ][ 'disks' ][ sDev ]

      if int( time.time() ) - sDisk[ 'sAttr' ].get( 'time', 0 ) >= iReScanTime or sDev not in self.smartAttr or opt :
         io.touch( asset.f_Heartbeat )
         fAttr = True

         # --------------------------------------------------------------------------
         # -->> Smart Log for NVME device :
         # -->> nvme --smart-log /dev/nvme0
         # -->> nvme --smart-log-add /dev/nvme0 << This query sometimes returns no result
         if sDev.startswith( 'nvme' ) :
            if self.Debug : print ( f'| SYS: SMART /sbin/nvme --smart-log /dev/{sDev}' )
            sList = lib.getCMD( f'/sbin/nvme --smart-log /dev/{sDev}' ).strip().split( '\n' )
            sList = sList[1:]

            iKey  = []
            sInfo = dict()
            # -----------------------------------------------------------------------
            for sLine in sList :
               # --- On "split"-error ... skip this line ...
               try    : sKey, sValue = sLine.split( ':', 1 )
               except : continue

               sKey= sKey[:1].upper() + sKey[1:].strip()
               iKey.append( sKey )
               sInfo[ sKey ] = [ sValue.strip(), '' ]

            if self.Debug : print ( f'| SYS: SMART /sbin/nvme --smart-log-add /dev/{sDev}' )
            sList = lib.getCMD( f'/sbin/nvme --smart-log-add /dev/{sDev} 2>&1' ).strip().split( '\n' )
            try    : sList = sList[2:]
            except : sList = sList[1:]

            # -----------------------------------------------------------------------
            for sLine in sList :
               # --- On "split"-error ... skip this line ...
               try    : sKey, sValue = sLine.split( ':', 1 )
               except : continue

               sKey= sKey[:1].upper() + sKey[1:].strip()
               iKey.append( sKey )

               sItems = sValue.strip().split( ' ' )
               sInfo[ sKey ] = [ sItems[0], ' '.join( sInfo[1:] ) ]

         else :
            # -----------------------------------------------------------------------
            # --- Vendor Specific SMART Attributes with Thresholds :
            if self.Debug : print ( f'| SYS: SMART smartctl -d sat -T permissive -A /dev/{sDev}' )
            sList = lib.getCMD( '/usr/bin/smartctl -d sat -T permissive -A /dev/' + sDev ).strip().split( '\n' )

            iKey  = []
            sInfo = dict()
            # -----------------------------------------------------------------------
            for sLine in sList[7:] :
               iAttr = ' '.join( sLine.split() ).split( ' ' )

               iID = iAttr[0]
               iKey.append( iID )
               try    : sInfo[ iID ] = [ iAttr[1], iAttr[3], iAttr[4], iAttr[5], iAttr[9], ' '.join( iAttr[10:] ) ]
               except : sInfo[ iID ] = [ iAttr[1], iAttr[3], iAttr[4], iAttr[5], iAttr[9], ' ' ]

      # -----------------------------------------------------------------------------
      self.smartAttr[ sDev ] =   {
                                    'iKey'   : iKey,
                                    'sValue' : sInfo,
                                    'time'   : int( time.time() ),
                                 }

      # -----------------------------------------------------------------------------
      if fAttr :
         if self.Debug : print ( f'| SYS: SMART Update sAttr from Drive : {sDev}' )

         sDisk[ 'sAttr' ] = self.smartAttr[ sDev ]
         self.rDict[ 'storage' ][ 'disks' ][ sDev ].update( sDisk )
         io.ioWrite( json.dumps( self.rDict, sort_keys=True, indent=4 ), asset.avrEnumStorage )

      # -----------------------------------------------------------------------------
      return self.rDict[ 'storage' ][ 'disks' ][ sDev ][ 'sAttr' ]

   # --------------------------------------------------------------------------------
   # --- Get a SMART-Attribut by #ID ...
   # --------------------------------------------------------------------------------
   def getSmartAttrID( self, sDev, sID, sDef=False ) :

      if not self.rDict   : self.getSynoStorage()
      if not self.sDiskDB : self.getDataBase()


      sAttr = self.rDict[ 'storage' ][ 'disks' ][ sDev ][ 'sAttr' ].get( 'sValue', False )
      sType = self.rDict[ 'storage' ][ 'disks' ][ sDev ][ 'type' ]

      if not sAttr : sAttr = self.getSmartAttr( sDev )[ 'sValue' ]

      sRet = sAttr.get( sID, sDef )
      if not sRet : return False

      # -----------------------------------------------------------------------------
      while switch( sType ) :
         # --------------------------------------------------------------------------
         if case( 'nvme' ) :

            while switch( sID ) :
               # --------------------------------------------------------------------
               if case( 'Available_spare', 'Available_spare_threshold', 'Percentage_used' ) :
                  return sRet[0][0:-1]
                  break
               # --------------------------------------------------------------------
               if case( 'Temperature' ) :
                  return sRet[0].split( ' ', 1 )[0]
                  break
               # --------------------------------------------------------------------
               break
            # -----------------------------------------------------------------------
            return sRet[0]

         # --------------------------------------------------------------------------
         if case( 'internal', 'ebox', 'esata', 'usb' ) :
            return sRet[4]
            break
         # --------------------------------------------------------------------------
         break

      # -----------------------------------------------------------------------------
      return False

   # --------------------------------------------------------------------------------
   # --- Get List of Drive "Location / Name" ...
   # --------------------------------------------------------------------------------
   def getDriveLocation( self ) :
      rTemp = self.rDict[ 'storage' ][ 'disks' ]
      sDisk = dict()

      for sDev in rTemp :
         sDrive = rTemp[ sDev ]

         if sDrive[ 'isBad' ] : continue

         # --------------------------------------------------------------------------
         while switch( sDrive[ 'type' ] ) :
            # -----------------------------------------------------------------------
            if case( 'internal', 'ebox' ) :
               sName = ' '.join( sDrive[ 'name' ].split( ' ' )[0:2] )
               break
            # -----------------------------------------------------------------------
            if case( 'nvme' ) :
               sName = sDrive[ 'name' ]
               break
            # -----------------------------------------------------------------------
            if case( 'usb', 'esata' ) :
               sName = sDrive[ 'name' ]
               break
            # --- Default's ---------------------------------------------------------
            break

         sDisk[ sDev ] =   {
                              'isSSD'  : sDrive[ 'isSSD'  ],
                              'name'   : sName,
                              'vendor' : sDrive[ 'vendor' ],
                              'model'  : sDrive[ 'model'  ],
                              'serial' : sDrive[ 'serial' ],
                              'size'   : sDrive[ 'size'   ],
                           }

      # -----------------------------------------------------------------------------
      return self.rDict[ 'enum' ], sDisk

   # --------------------------------------------------------------------------------
   # --- Init "Predict (Critical)" SMART Attributes ...
   # --------------------------------------------------------------------------------
   def initSmartEsc( self ) :
      import smartEsc
      for sID in smartEsc.iSmartImportant :
         if smartEsc.iSmartImportant[ sID ] == 'Denote' :
            self.iEsc.append( sID )

   # --------------------------------------------------------------------------------
   # --- Get / Load Device DataBase (enumDrives) ...
   # --------------------------------------------------------------------------------
   def getDataBase( self ) :

      if self.Debug : print( '| SYS: SMART Loading "enumDrives"-DataBase' )

      rTemp   = io.ioRead( asset.avrEnumDrives )

      if rTemp : self.sDiskDB = json.loads( rTemp )
      else     : self.sDiskDB = { 
                                    'time' : int( time.time() ),
                                    'version' : vDrives,
                                 }

      # -----------------------------------------------------------------------------
      sVersion = self.sDiskDB.get( 'version', '000000.000' )
      iTime    = self.sDiskDB.get( 'time',    0 )

      if sVersion == vDrives and int( time.time() ) - iTime < tDrives :
         return

      # -----------------------------------------------------------------------------
      if self.Debug : print   ( '| SYS: SMART Start "Garbage Collection" of "History DataBase"' )
      else          : lib.Error ( 'SYS: Start "Garbage Collection" of "History DataBase"' )

      # -----------------------------------------------------------------------------
      self.sDiskDB.update  (
                              {
                                 'time' : int( time.time() ),
                                 'version' : vDrives,
                              }
                           )

      # -----------------------------------------------------------------------------
      sEnum   = list( self.sDiskDB.keys() )
      iEnum   = len( sEnum ) - 2
      iVendor = 0
      iMerge  = 0
      iClean  = 0

      # -----------------------------------------------------------------------------
      for sID in sorted( self.sDiskDB ) :
         # --------------------------------------------------------------------------
         if sID in [ 'time', 'version' ] or sID not in sEnum : continue

         # --------------------------------------------------------------------------
         if self.Debug : print   ( f'| SYS: SMART    Examine ID  : {sID}' )
         else          : lib.Error ( f'SYS:    Examine ID  : {sID}' )

         rTemp  = dict()
         iParts = sID.split( ' ' )

         # --------------------------------------------------------------------------
         if len( iParts ) > 3 :
            # -----------------------------------------------------------------------
            # --- Skip ID if it contains more than 3 parts ...
            if self.Debug : print   (  '| SYS: SMART       Cannot process ... Skipped' )
            else          : lib.Error ( 'SYS:       Cannot process ... Skipped' )

            sEnum.remove ( sID )
            continue               

         # --------------------------------------------------------------------------
         if len( iParts ) > 2 :
            # -----------------------------------------------------------------------
            # --- Clean up ID (remove Vendor) ...
            if self.Debug : print   ( f'| SYS: SMART       Remove Vendor-ID : {iParts[0]}' )
            else          : lib.Error ( f'SYS:       Remove Vendor-ID : {iParts[0]}' )

            iVendor += 1
            rTemp = dict( self.sDiskDB[ sID ] )
            del self.sDiskDB[ sID ]
            sEnum.remove ( sID )

            sID = ' '.join(( iParts[1], iParts[2] ))

            if sID in sEnum : 
               # --------------------------------------------------------------------
               # --- Merge ID duplicate ...
               if self.Debug : print   (  '| SYS: SMART       Merge duplicate IDs' )
               else          : lib.Error ( 'SYS:       Merge duplicate IDs' )

               iMerge += 1
               rTemp.update( self.sDiskDB[ sID ] )
               sEnum.remove ( sID )

            # -----------------------------------------------------------------------
            # --- Update ID ...
            if self.Debug : print   (  '| SYS: SMART       Save updated ID' )
            else          : lib.Error ( 'SYS:       Save updated ID' )

            try    : self.sDiskDB[ sID ].update( rTemp )
            except : self.sDiskDB[ sID ] =       rTemp

      # -----------------------------------------------------------------------------
      for sID in sorted( self.sDiskDB ) :
         # --------------------------------------------------------------------------
         if sID in [ 'time', 'version' ] : continue

         if self.Debug : print   ( f'| SYS: SMART    Clean up ID : {sID}' )
         else          : lib.Error ( f'SYS:    Clean up ID : {sID}' )

         # --------------------------------------------------------------------------
         sEnum = sorted( list( self.sDiskDB[ sID ].keys() ) )
         if len( sEnum ) <= 1 :
            continue

         # --------------------------------------------------------------------------
         rTemp = dict()
         fClean = False
         for iTime in sEnum[0:-1] :
            if not self.sDiskDB[ sID ][ iTime ].get( 'Denote', False ) :
               if self.Debug : print   ( f'| SYS: SMART       Delete empty timestamp : {self.sDiskDB[ sID ][ iTime ][ "Date" ]}' )
               else          : lib.Error ( f'SYS:       Delete empty timestamp : {self.sDiskDB[ sID ][ iTime ][ "Date" ]}' )
               fClean  = True
               iClean += 1
               continue

            # -----------------------------------------------------------------------
            rTemp[ iTime ] = self.sDiskDB[ sID ][ iTime ]

         # --------------------------------------------------------------------------
         rTemp[ sEnum[-1] ] = self.sDiskDB[ sID ][ sEnum[-1] ]

         # --------------------------------------------------------------------------
         if fClean :
            if self.Debug : print   (  '| SYS: SMART       Save cleaned ID' )
            else          : lib.Error ( 'SYS:       Save cleaned ID' )

         self.sDiskDB[ sID ] = rTemp

      # -----------------------------------------------------------------------------
      if self.Debug :
         print   ( f'| SYS: SMART Examined IDs             : {iEnum}'  )
         print   ( f'| SYS: SMART Removed Vendor IDs       : {iVendor}')
         print   ( f'| SYS: SMART Removed Duplicate IDs    : {iMerge}' )
         print   ( f'| SYS: SMART Deleted empty timestamps : {iClean}' )
         print   (  '| SYS: SMART -------------------------------------------------------' )
      else :
         lib.Error (f'SYS: Examined IDs             : {iEnum}'  )
         lib.Error (f'SYS: Removed Vendor IDs       : {iVendor}')
         lib.Error (f'SYS: Removed Duplicate IDs    : {iMerge}' )
         lib.Error (f'SYS: Deleted empty timestamps : {iClean}' )
         lib.Error ( 'SYS: -------------------------------------------------------' )

      # -----------------------------------------------------------------------------
      Sys.RotateClear( asset.avrEnumDrives )
      io.ioWrite( json.dumps( self.sDiskDB, sort_keys=True, indent=4 ), asset.avrEnumDrives )

   # --------------------------------------------------------------------------------
   # --- Get latest Predict Attributes from DataBase ...
   # --------------------------------------------------------------------------------
   def getDataBasePredict( self, sDev, sID ) :

      if not self.sDiskDB : self.getDataBase()

      # -----------------------------------------------------------------------------
      rTemp = self.sDiskDB.get( sID, False )

      if rTemp : return rTemp[ sorted( rTemp )[-1] ][ 'Predict' ]
      else     : return False

   # --------------------------------------------------------------------------------
   # --- Add or Update "Predict" SMART Attributes to / in DataBase ...
   # --------------------------------------------------------------------------------
   def setDataBasePredict( self, sDev, sDisk=False ) :

      if not self.sDiskDB : self.getDataBase()
      if not sDisk : sDisk = self.rDict[ 'storage' ][ 'disks' ][ sDev ]

      # -----------------------------------------------------------------------------
      # --- Get "current-Predict's" from SMART ...
      sPredict = self.getPredictAttr( sDev, sDisk=sDisk )
      sID      = '{model} {serial}'.format( **sDisk )

      if not sPredict : return sPredict

      if self.sDiskDB.get( sID, False ) :
         # --------------------------------------------------------------------------
         # --- Update "Predict" ....
         if self.Debug : print ( f'| SYS: SMART Update "Predict" DataBase for Drive : {sDev}' )
         sDenote  = self.getDenoteAttr( sDev, sPredict, sDisk=sDisk )
         self.sDiskDB[ sID ].update(  {
                                          str( int( time.time() ) ) :   {
                                                                           'Date'      : time.strftime( '%Y.%m.%d %H:%M:%S' ),
                                                                           'Predict'   : sPredict,
                                                                           'Denote'    : sDenote,
                                                                        },
                                       } )
      else :
         # --------------------------------------------------------------------------
         # --- Add "New Predict" ....
         if self.Debug : print ( f'| SYS: SMART Add "New Predict" to DataBase for Drive : {sDev}' )
         self.sDiskDB[ sID ] =         {
                                          str( int( time.time() ) ) :   {
                                                                           'Date'      : time.strftime( '%Y.%m.%d %H:%M:%S' ),
                                                                           'Predict'   : sPredict,
                                                                           'Denote'    : dict(),
                                                                        },
                                       }

      # -----------------------------------------------------------------------------
      io.ioWrite( json.dumps( self.sDiskDB, sort_keys=True, indent=4 ), asset.avrEnumDrives )

      # -----------------------------------------------------------------------------
      return sPredict

   # --------------------------------------------------------------------------------
   # --- Get Predict Attributes and update if necessary ...
   # --------------------------------------------------------------------------------
   def getPredict( self, sDev, sDisk=False ) :

      if not sDisk : sDisk = self.rDict[ 'storage' ][ 'disks' ].get( sDev, False )

      if not sDisk :
         if self.Debug : print  ( f'| SYS: SMART Fatal Error : Unexpected error in "Syno-Storage" - Missing Drive : {sDev}' )
         else          : lib.Error(f'SYS: Fatal Error : Unexpected error in "Syno-Storage" - Missing Drive : {sDev}' )
         return False

      # -----------------------------------------------------------------------------
      # --- Check whether this drive is present in the DataBase ...
      sID   = '{model} {serial}'.format( **sDisk )
      rTemp = self.getDataBasePredict( sDev, sID )

      # -----------------------------------------------------------------------------
      # --- Add / update DataBase with current Predict Attributes ...
      if not rTemp :
         rTemp = self.setDataBasePredict( sDev, sDisk=sDisk )

      # -----------------------------------------------------------------------------
      return rTemp

   # --------------------------------------------------------------------------------
   #  --- Get current Predict SMART attributes for this drive ...
   # --------------------------------------------------------------------------------
   def getPredictAttr( self, sDev, sDisk=False ) :

      if not self.iEsc : self.initSmartEsc()
      if not sDisk : sDisk = self.rDict[ 'storage' ][ 'disks' ][ sDev ]
      if self.Debug : print ( f'| SYS: SMART Current Predicts for device : {sDev}' )

      # -----------------------------------------------------------------------------
      # --- Get SMART vendor-specific Attributes and values : Either from the cache or directly ...
      smartAttr = sDisk[ 'sAttr' ].get( 'sValue', False )

      if not smartAttr :
         smartAttr = self.getSmartAttr( sDev )[ 'sValue' ]

      sRet = dict()

      for sID in self.iEsc :
         sRaw = smartAttr.get( sID, False )
         if sRaw : sRet[ sID ] = [ sRaw[0], sRaw[4] ]

      # -----------------------------------------------------------------------------
      # --- Check "Raw_Read_Error_Rate" is greater than "iRawErrorRate" and ignore / scip if necessary
      RawRead = sRet.get( '1', False )
      if RawRead and int( RawRead[1] ) >= iRawErrorRate :
         del sRet[ '1' ]

      if self.Debug : print ( json.dumps( sRet, sort_keys=True, indent=4 ) )

      return sRet

   # --------------------------------------------------------------------------------
   # --- Compare "previous-Predict" with "current-Predict" and return "Denoted" SMART-attributes ...
   # --------------------------------------------------------------------------------
   def getDenoteAttr( self, sDev, sCurDict=False, sDisk=False ) :

      if not self.rDict   : self.getSynoStorage()
      if not self.sDiskDB : self.getDataBase()
      if not sDisk : sDisk = self.rDict[ 'storage' ][ 'disks' ][ sDev ]

      sID   = '{model} {serial}'.format( **sDisk )

      if sDisk[ 'denote' ] : sRet  = sDisk[ 'denote' ]
      else                 : sRet  = dict()

      rTemp = self.sDiskDB.get( sID, False )

      if not sCurDict : sCurDict = self.getPredictAttr( sDev, sDisk=sDisk )

      try    : sPreDict = rTemp[ sorted( rTemp )[-1] ][ 'Predict' ]
      except : sPreDict = sCurDict

      # -----------------------------------------------------------------------------
      # --- Compare "previous" with "current"-Values ...
      for sID in sCurDict :
         curName, curAttr = sCurDict[ sID ]

         if not sID in sPreDict : continue

         preName, preAttr = sPreDict[ sID ]

         if curName != preName : continue

         if curAttr != preAttr :
            sRet[ sID ] = [ preName, preAttr, curAttr ]

      # -----------------------------------------------------------------------------
      return sRet

   # --------------------------------------------------------------------------------
   # --- Get Attribut-History from DataBase ...
   # --------------------------------------------------------------------------------
   def getAttrHistory( self, sDev ) :

      if not self.rDict   : self.getSynoStorage()
      if not self.sDiskDB : self.getDataBase()

      # -----------------------------------------------------------------------------
      sDisk = self.rDict[ 'storage' ][ 'disks' ][ sDev ]
      sID   = '{model} {serial}'.format( **sDisk )
      sRet  = dict()

      try :
         for iTime in sorted( self.sDiskDB[ sID ] ) :
            sDate   = self.sDiskDB[ sID ][ iTime ][ 'Date' ]
            sDenote = self.sDiskDB[ sID ][ iTime ][ 'Denote' ]

            for iID in sDenote :
               iAttr = sDenote[ iID ]

               try    : sRet[ iID ][ 'List' ].update( { sDate : iAttr[1:] } )
               except : sRet[ iID ] = { 'Name' : iAttr[0], 'List' : { sDate : iAttr[1:], }, }
      except :
         pass

      # -----------------------------------------------------------------------------
      if self.Debug :
         print ( f'| SYS: SMART Get Attribut-History of device : {sDev}' )
         print ( json.dumps( sRet, sort_keys=True, indent=4 ) )

      return sRet

   # --------------------------------------------------------------------------------
   # --- Get History from DataBase ...
   # --------------------------------------------------------------------------------
   def getDiskHistory( self, sDev ) :

      if not self.rDict   : self.getSynoStorage()
      if not self.sDiskDB : self.getDataBase()

      # -----------------------------------------------------------------------------
      sDisk = self.rDict[ 'storage' ][ 'disks' ][ sDev ]
      sID   = '{model} {serial}'.format( **sDisk )
      sRet  = dict()

      try :
         for iID in sorted( self.sDiskDB[ sID ] ) :
            sDenote = self.sDiskDB[ sID ][ iID ][ 'Denote' ]

            # --- Skip empty entry ...
            if not sDenote : continue

            sDate   = self.sDiskDB[ sID ][ iID ][ 'Date' ]
            sRet[ sDate ] = sDenote
      except :
         pass

      # -----------------------------------------------------------------------------
      if self.Debug :
         print ( f'| SYS: SMART Get Disk-History of device : {sDev}' )
         print ( json.dumps( sRet, sort_keys=True, indent=4 ) )

      return sRet


# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
