# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import os
import sys
import time
from   shutil   import copy, chown
from   math     import log10 as log
import glob

import asset
import basics   as io
import lib

from   lib      import xrange, switch, case
import Flow     as flow

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
# --- Global Options
# -----------------------------------------------------------------------------------
svg = sys.modules[ __name__ ]

sStyle = [  '',                                  \
            ' stroke-dasharray="5,3"',           \
            ' stroke-dasharray="8,3,3,3,3,3"',   \
            ' stroke-dasharray="10,3,3,6"',      \
            ' stroke-dasharray="8,3,2,3"',       \
            ' stroke-dasharray="2,2"' ]

# -----------------------------------------------------------------------------------
# --- Prepare "PreView"-SVG ...
# -----------------------------------------------------------------------------------
def PreView ( base=False, pre=False ) :

   import Init     as AvrInit
   import __main__ as avr

   avr.AvrDebug = True
   avr.AvrSmart = False

   if base : sDat = base
   else    : sDat = asset.avrLogDataFile
   
   # --------------------------------------------------------------------------------
   # --- Init Language for AvrLogger ...
   asset.msg = lib.InitLanguage( 'admin', asset.avrLangFile )

   # --------------------------------------------------------------------------------
   AvrInit.init( smallInit=True )

   # --------------------------------------------------------------------------------
   # --- Get the DataBase-File ...
   if os.path.exists( sDat ) :
      DB = io.ioRead( sDat ).split('\n')

   try:    Lines = len( DB )
   except: Lines = False

   # ----------------------------------------------------------
   # --- Shorten DataBase to 200 values (lines) ...
   if Lines :
      if Lines >= asset.sys_MaxLines :
         DB = DB[ Lines - asset.sys_MaxLines: ]

      iStep = round( asset.sys_MaxLines / asset.SvgPreMax, 2)
      asset.DB = []

      for a in xrange( 0, len ( DB ), iStep, opt='<' ) :
         asset.DB.append( DB[ int( round( a, 0 ) ) ] )

   # ----------------------------------------------------------
   Init ()
   Prio ()
   # ----------------------------------------------------------
   oMaxLines = asset.sys_MaxLines
   asset.sys_MaxLines = asset.SvgPreMax
   flow.init    ()
   # ----------------------------------------------------------
   PlotInit ()
   asset.sys_MaxLines = oMaxLines

   # ----------------------------------------------------------
   SVG  ( opt=pre )


# --------------------------------------------------------------------------
# --- Prepare SVG-Values
# --------------------------------------------------------------------------
def Init() :

   Avr = asset.AvrConf
   svg.Plot              = dict()

   svg.PageSizeWidht     = int( Avr['PageSizeWidht' ] )
   svg.PageSizeHeight    = int( Avr['PageSizeHeight'] )

   svg.BorderEdge        = int( Avr['BorderEdge'    ] ) / 10
   svg.BorderX           = svg.BorderEdge
   svg.BorderY           = svg.BorderEdge
   svg.BorderWidth       = svg.PageSizeWidht  - ( 2 * svg.BorderEdge )
   svg.BorderHeight      = svg.PageSizeHeight - ( 2 * svg.BorderEdge )
   # --- Plotting Area ---
   svg.PlotAreaFill      = Avr["PlotAreaFill"]
   # ----------------------------------------
   # --- Prepare Borders
   # ----------------------------------------
   svg.PlotEdgeLeft      = int( Avr['PlotEdgeLeft'       ] ) / 10
   svg.PlotEdgeRight     = int( Avr['PlotEdgeRight'      ] ) / 10
   svg.PlotEdgeTop       = int( Avr['PlotEdgeTop'        ] ) / 10
   svg.PlotEdgeBottom    = int( Avr['PlotEdgeBottom'     ] ) / 10

   svg.PlotLabelWidth    = int( Avr['PlotLabelWidth'     ] ) / 10
   svg.PlotLabelHeight   = int( Avr['PlotLabelHeight'    ] ) / 10

   svg.LegAxisHeadOffset = int( Avr[ 'LegAxisHeadOffset' ] ) / 10
   svg.LegAxisOffset     = int( Avr[ 'LegAxisOffset'     ] ) / 10

   svg.svg_LabelColor    = [ Avr["Pri1LabelColor"], Avr["Sec1LabelColor"], Avr["Pri2LabelColor"], Avr["Sec2LabelColor"], Avr["Pri3LabelColor"], Avr["Sec3LabelColor"] ]
   svg.pw_l              = 0
   svg.pw_r              = 0
   svg.pw_v              = 0

   svg.PriCount          = 0
   svg.SecCount          = 0

   # -----------------------------------------------------------------------

   if Avr["Pri1LabelOn"] == "Yes" : svg.PriCount += 1
   if Avr["Pri2LabelOn"] == "Yes" : svg.PriCount += 1
   if Avr["Pri3LabelOn"] == "Yes" : svg.PriCount += 1

   svg.PlotEdgeLeft = svg.PlotEdgeLeft + ( svg.PriCount * svg.PlotLabelWidth )
   svg.pw_l         = svg.pw_l         + ( svg.PriCount * svg.PlotLabelWidth )

   # -----------------------------------------------------------------------
   if Avr["Sec1LabelOn"] == "Yes" : svg.SecCount += 1
   if Avr["Sec2LabelOn"] == "Yes" : svg.SecCount += 1
   if Avr["Sec3LabelOn"] == "Yes" : svg.SecCount += 1

   svg.PlotEdgeRight = svg.PlotEdgeRight + ( svg.SecCount * svg.PlotLabelWidth )
   svg.pw_r          = svg.pw_r          + ( svg.SecCount * svg.PlotLabelWidth )

   # -----------------------------------------------------------------------
   if Avr["VerLabelOn"] == "Yes" or Avr["LegAxisHeadPos"] == "Bottom" :
      svg.PlotEdgeBottom = svg.PlotEdgeBottom + svg.PlotLabelHeight
      svg.pw_v = svg.PlotLabelHeight

   svg.LegSignSize  = int( Avr["LegSignSize"])  / 10
   svg.LegAlignment =      Avr["LegAlignment"]    # Orientation "Hor" (horizontal) "Ver" (vertical)
   svg.LegZeroX     =      Avr["LegZeroX"]        # X-Zero "Left" / "Right" / "Split"
   svg.LegZeroY     =      Avr["LegZeroY"]        # Y-Zero "Top" / "Bottom"
   svg.LegOffsetX   = int( Avr["LegOffsetX"] ) / 10
   svg.LegOffsetY   = int( Avr["LegOffestY"] ) / 10
   svg.LegTabX      = int( Avr["LegTabX"] )    / 10
   svg.LegTabY      = int( Avr["LegTabY"] )    / 10

   # ----------------------------------------------------------
   # --- Calculate the margins / distances based on position,
   # --- orientation and number of plots ...
   # ----------------------------------------------------------
   if Avr["LegLabelOn"] == "Yes" :
      if svg.LegZeroX == "Split" :
         # --- Legend "Mode"-Split: Only "Top" and "Bottem"
         svg.LegendCount  = 1
         svg.LegAlignment = "Hor"
         svg.LegZeroY     = "Top"

         if asset.ScalingAxis_Names[2] or asset.ScalingAxis_Names[3] : svg.LegendCount += 1
         if asset.ScalingAxis_Names[4] or asset.ScalingAxis_Names[5] : svg.LegendCount += 1

         svg.PlotEdgeTop = svg.PlotEdgeTop + ( svg.LegTabY * svg.LegendCount ) + ( svg.LegOffsetY / 2 )
      else :
         svg.LegBreakCount = len( asset.BaseList )

         if Avr["LegBreakOn"] == "No" :
            # --- Calculate the number of necessary "LegendCount" ...
            svg.LegendCount = int( svg.LegBreakCount / int(Avr["LegBreakCount"]) )
            if svg.LegBreakCount % int(Avr["LegBreakCount"]) >= 1 : svg.LegendCount += 1
         else :
            svg.LegendCount = 1
            #        ScalingAxis_Names[0]                         # <<< (Pri1 links ) Temperature °C
            #        ScalingAxis_Names[1]                         # <<< (Sec1 rechts) Load percent %
            if asset.ScalingAxis_Names[2] : svg.LegendCount += 1  # <<< (Pri2 links ) Network I/O MiB/s
            if asset.ScalingAxis_Names[3] : svg.LegendCount += 1  # <<< (Sec2 rechts) Storage I/O MiB/s
            #        ScalingAxis_Names[4]                         # <<< (Pri3 links ) Ping ms (Ookla)
            #        ScalingAxis_Names[5]                         # <<< (Sec3 rechts) USV Inut Volt

         if svg.LegZeroY  == "Top" and svg.LegAlignment == "Hor" :
            svg.PlotEdgeTop = svg.PlotEdgeTop + (svg.LegTabY * svg.LegendCount) + (svg.LegOffsetY / 2)

         if svg.LegZeroY == "Bottom" and svg.LegAlignment == "Hor" :
            svg.PlotEdgeBottom = svg.PlotEdgeBottom + (svg.LegTabY * svg.LegendCount) + (svg.LegOffsetY / 2)

         if svg.LegZeroX == "Left" and svg.LegAlignment == "Ver" :
            svg.PlotEdgeLeft = svg.PlotEdgeLeft + svg.LegTabX + (svg.LegOffsetX * 2)

         if svg.LegZeroX == "Right" and svg.LegAlignment == "Ver" :
            svg.PlotEdgeRight = svg.PlotEdgeRight + svg.LegTabX + (svg.LegOffsetX * 2)

         if svg.LegZeroX == "Left" :
            svg.LegLabelAnchor = "start"

         if svg.LegZeroX == "Right" :
            svg.LegLabelAnchor = "end"
   # ----------------------------------------------------------

   # ----------------------------------------------------------
   if Avr[ 'LegAxisHeadlineOn' ] == 'Yes' :
      if svg.LegZeroY != "Top" or svg.LegAlignment != "Hor" or Avr[ 'LegLabelOn' ] != "Yes" :
         svg.PlotEdgeTop = svg.PlotEdgeTop + svg.LegAxisHeadOffset

   if Avr[ 'LegAxisUnitOn'     ] == "Yes" :
      if svg.LegZeroY != "Top" or svg.LegAlignment != "Hor" or Avr[ 'LegLabelOn' ] != "Yes" :
         svg.PlotEdgeTop = svg.PlotEdgeTop + svg.LegAxisOffset
   # ----------------------------------------------------------

   svg.pw = round( svg.PageSizeWidht  - svg.PlotEdgeLeft - svg.PlotEdgeRight,  1 )
   svg.ph = round( svg.PageSizeHeight - svg.PlotEdgeTop  - svg.PlotEdgeBottom, 1 )

# --------------------------------------------------------------------------
# --- Prepare the Display-Order (z-Index) ...
# --------------------------------------------------------------------------
def Prio() :

   svgPrio         = dict()
   svg.svgBaseList = []

   # -----------------------------------------------------------------------
   for iID in asset.BaseList :
      PlotPrio = f'Plot{asset.BaseName[ iID ]}Prio'
      sID = str( iID )

      if asset.AvrConf[ PlotPrio ] == "" :
         asset.AvrConf[ PlotPrio ] = sID

      Prio = asset.AvrConf[ PlotPrio ]
      try :   svgPrio[ Prio ] += " " + sID
      except: svgPrio[ Prio ]  =       sID

   # -----------------------------------------------------------------------
   for iID in reversed( range( 0, asset.iTabLast ) ) :
      sID = str( iID )
      if sID not in svgPrio : continue

      try :   Raw += ' ' + svgPrio[ sID  ]
      except: Raw  =       svgPrio[ sID  ]

   # -----------------------------------------------------------------------
   try :
      asset.AvrConf['SvgBaseList'] = Raw
      svg.svgBaseList = [ int( x ) for x in Raw.split(' ') ]
   except :
      asset.AvrConf['SvgBaseList'] = ''
      svg.svgBaseList = []

# --------------------------------------------------------------------------
# --- Prepare SVG Plot-Array ...
# --------------------------------------------------------------------------
def PlotInit() :

   Avr  = asset.AvrConf
   sKey = [ 'Pri1', 'Sec1', 'Pri2', 'Sec2', 'Pri3', 'Sec3' ]

   PreHeader()
   PreLegend()
   PreGrid  ()
   PreLabel ()

   for i in asset.BaseList :
      sBaseName   = asset.BaseName[ i ]
      sNum        = str( i )

      LineWidth   = int( Avr[ f'Plot{ sBaseName }Stroke' ] ) / 10
      LineStyle   = sStyle[ int( Avr[ f'Plot{ sBaseName }Style'] ) ]
      LineStroke  = Avr[ f'Plot{ sBaseName }Color'   ]
      LineOpacity = Avr[ f'Plot{ sBaseName }Opacity' ]

      if LineStyle : svg.Plot[ f'PlotColor{sNum}' ]  = f'\t\t\t\t<path id="Plot{ sBaseName }" vector-effect="non-scaling-stroke" stroke="{LineStroke}" stroke-opacity="{LineOpacity}" stroke-width="{LineWidth:.2f}" {LineStyle}\n'
      else         : svg.Plot[ f'PlotColor{sNum}' ]  = f'\t\t\t\t<path id="Plot{ sBaseName }" vector-effect="non-scaling-stroke" stroke="{LineStroke}" stroke-opacity="{LineOpacity}" stroke-width="{LineWidth:.2f}"\n'

   for Axis in range( 0, 6 ) :
      # --------------------------------------------------------------------
      if Avr[ sKey[ Axis ] + 'ScaleType' ] == 'Lin' :
         y = f'{svg.ph / ( asset.ScalingAxis_Max[Axis] - asset.ScalingAxis_Min[Axis]  ):.4f}'
      else :
         y = f'{svg.ph / ( log( asset.ScalingAxis_Max[Axis] - asset.ScalingAxis_Min[Axis] ) ):.4f}'
      # --------------------------------------------------------------------

      x = f'{svg.pw / asset.sys_MaxLines:.4f}'

      svg.Plot['PlotAxis' + str(Axis)] = f'\t\t\t<g id="Axis-{Axis}" transform="scale({x},{y})">\n'

# --------------------------------------------------------------------------
# --- Build SVG ...
# --------------------------------------------------------------------------
def SVG( opt=False ) :

   if opt : asset.SvgTmp = opt

   # --- Timeline update axis label ...
   TimeLabel()

   # --- Create the SVG plot ...
   echo = [ svg.Plot["header"]                         ]
   echo.append( '\t\t<!-- $svgBaseList -->\n'          )
   echo.append( svg.Plot["legend"]                     )
   # ----------------------------------------
   echo.append( '\t\t<g id="PrimaryAxis">\n'           )
   echo.append( svg.Plot["Pri1MainGrid"]               )
   echo.append( svg.Plot["Pri1HelpGrid"]               )
   echo.append( svg.Plot["Pri1Label"]                  )
   echo.append( svg.Plot["Pri2Label"]                  )
   echo.append( svg.Plot["Pri3Label"]                  )
   echo.append( '\t\t</g> <!-- PrimaryAxisEnd -->\n'   )
   # ----------------------------------------
   echo.append( '\t\t<g id="SecondaryAxis">\n'         )
   echo.append( svg.Plot["Sec1Label"]                  )
   echo.append( svg.Plot["Sec2Label"]                  )
   echo.append( svg.Plot["Sec3Label"]                  )
   echo.append( '\t\t</g> <!-- SecondaryAxisEnd -->\n' )
   # ----------------------------------------
   echo.append( '\t\t<g id="VerticalAxis">\n'          )
   echo.append( svg.Plot["VerMainGrid"]                )
   echo.append( svg.Plot["VerLabel"]                   )
   echo.append( '\t\t</g> <!-- VerticalAxisEnd -->\n'  )
   # ----------------------------------------

   x = svg.PlotEdgeLeft
   y = ( svg.PlotEdgeTop + svg.ph ) * -1

   echo.append( f'\t\t<g id="DataPlots" style="clip-path: url(#PlotArea);" transform="scale(1,-1) translate({x} {y})" fill="none">\n' )

   # ----------------------------------------------------------
   if flow.minFlag :
      for j in svg.svgBaseList :
         echo.append( svg.Plot['PlotAxis' + str(asset.BaseNum[j]) ] )
         # ----------------------------------------------------------
         echo.append( svg.Plot['PlotColor' + str(j) ] )
         echo.append( f'{flow.startFlow[j]}\n' )
         echo.append( f'{flow.plotFlow[j]}\n' )
         echo.append( f'\t\t\t\t\"/> <!-- Plot{asset.BaseName[j]}End -->\n' )
         # ----------------------------------------------------------
         echo.append( f'\t\t\t</g> <!-- Axis-{asset.BaseNum[j]}End -->\n' )
   # ----------------------------------------

   echo.append( '\t\t</g> <!-- DataPlotsEnd -->\n' )
   # ----------------------------------------------------------
   echo.append( '\t</g> <!-- PageEnd -->\n' )
   echo.append( '</svg> <!-- AvrLoggerEnd -->\n' )

   # ----------------------------------------------------------
   with open( asset.SvgTmp, 'w+', encoding='utf-8' ) as f:
      f.write( ''.join( echo ) )

   # ----------------------------------------------------------
   if opt :
      chown( asset.SvgTmp, user='admin', group='users' )
      os.chmod( asset.SvgTmp, 0o755 )
      return
   # ----------------------------------------------------------

   io.touch ( asset.f_SvgLock )                 # --- Lock SVG (SVG is busy and the graphic is invalid)
   io.ioMove( asset.SvgTmp, asset.SvgPlot )
   io.ioRm  ( asset.f_SvgLock )                 # --- Clear SVG-Lock

   # ----------------------------------------------------------
   # --- Copy LineChart with random suffix to circumvent the caching problem in some browsers.
   iList = glob.glob( asset.LineChart + '/tmp_*.svg' )

   for sID in iList :
      io.ioRm( sID )

   svgName = 'tmp_' + str( int( time.time() ) ) + '.svg'
   io.ioCopy ( asset.SvgPlot, asset.LineChart + '/' + svgName )

   # ----------------------------------------------------------
   # --- Copy "SVG-File" to users defined destination and overwrite earlier File
   if asset.AvrConf["SvgCopyOn"] != "Yes" : return

   # ----------------------------------------------------------
   SvgPath = os.path.dirname ( asset.AvrConf['SvgCopyPath'] )
   SvgName = os.path.basename( asset.AvrConf['SvgCopyPath'] )

   arr = lib.FindeShare ( SvgPath )

   if arr :
      SvgFile = os.path.join( *arr, SvgName )

      io.ioCopy ( asset.SvgPlot, SvgFile )
      chown( SvgFile, user='admin', group='users' )
      os.chmod( SvgFile, 0o755 )
   else :
      asset.AvrConf["SvgCopyOn"] = "No"
      lib.Error( asset.msg['errSvgCopyPath'] + ' "' + asset.AvrConf["SvgCopyPath"] + '"' )

# --------------------------------------------------------------------------
# --- Print Time-Axis Labels (Time-Line) ...
# --------------------------------------------------------------------------
def TimeLabel() :

   Avr = asset.AvrConf

   svg.Plot[ 'VerLabel' ] = ''
   if Avr["VerLabelOn"] != "Yes" : return

   c              = 0
   VerLabelOffset = int( Avr[ "VerLabelOffset"] ) / 10
   VerLabelPos    = int( Avr[ "VerLabelPos"   ] ) / 10
   FontSize       = int( Avr[ "VerLabelSize"  ] ) / 10
   AxisLine       = int( Avr[ "LegAxisLine"   ] ) / 10
   LabelColor     =      Avr[ "VerLabelColor" ]

   VerLine = [ f'\t\t\t<g id="VerticalyLine" stroke-width="0.9" stroke="{LabelColor}">\n' ]
   VerText = [ f'\t\t\t<g id="VerticalyLabel" text-anchor="middle" font-family="\'ArialMT\'" font-size="{FontSize}" fill="{LabelColor}">\n' ]

   Inc = svg.pw / ( asset.sys_MaxLines / asset.svg_TimeStep )
   y   = svg.PlotEdgeTop + svg.ph + VerLabelPos
   y1  = svg.PlotEdgeTop + svg.ph
   xz  = svg.PlotEdgeLeft

   svgTimeline = int( int(time.mktime( time.localtime() )) - int( flow.PlotRow * asset.sys_Tick * asset.sys_Div ) )

   for i in xrange( 0, asset.sys_MaxLines, asset.svg_TimeStep, '<=' ) :
      t  = time.strftime('%H:%M', time.localtime( svgTimeline + ( c * asset.svg_TimeBase ) ))

      x1 = xz + ( Inc * c )
      x  = x1 - VerLabelOffset

      VerLine.append( f'\t\t\t\t<path d="M{x1:.2f},{y1:.2f} v{AxisLine}"/>\n' )
      VerText.append( f'\t\t\t\t<text x="{x:.2f}" y="{y:.2f}">{t}</text>\n' )

      c += 1

   svg.Plot["VerLabel"]  = f'{ "".join( VerLine ) }\t\t\t</g> <!-- VerticalyLineEnd -->\n'
   svg.Plot["VerLabel"] += f'{ "".join( VerText ) }\t\t\t</g> <!-- VerticalyLabelEnd -->\n'

# --------------------------------------------------------------------------
# --- Prepare SVG-Labels ...
# --------------------------------------------------------------------------
def PreLabel() :

   svg.Plot[ 'Pri1Label' ] = Label( "Pri1", "Primary1"    )
   svg.Plot[ 'Pri2Label' ] = Label( "Pri2", "Primary2"    )
   svg.Plot[ 'Pri3Label' ] = Label( "Pri3", "Primary3"    )
   svg.Plot[ 'Sec1Label' ] = Label( "Sec1", "Secondary1"  )
   svg.Plot[ 'Sec2Label' ] = Label( "Sec2", "Secondary2"  )
   svg.Plot[ 'Sec3Label' ] = Label( "Sec3", "Secondary3"  )

# --------------------------------------------------------------------------
# --- Print Vertical-Axis Labels
# --------------------------------------------------------------------------
def Label( sName, sID ) :

   Avr  = asset.AvrConf
   iRnd = asset.rnd_SVG

   Plot = []
   if Avr[sName + 'LabelOn'] != "Yes" : return ''

   while switch( sName ):
      if case( 'Pri1' ):
         LabelMax = asset.ScalingAxis_Max[0]
         LabelMin = asset.ScalingAxis_Min[0]
         break
      if case( 'Sec1' ):
         LabelMax = asset.ScalingAxis_Max[1]
         LabelMin = asset.ScalingAxis_Min[1]
         break
      if case( 'Pri2' ):
         LabelMax = asset.ScalingAxis_Max[2]
         LabelMin = asset.ScalingAxis_Min[2]
         break
      if case( 'Sec2' ):
         LabelMax = asset.ScalingAxis_Max[3]
         LabelMin = asset.ScalingAxis_Min[3]
         break
      if case( 'Pri3' ):
         LabelMax = asset.ScalingAxis_Max[4]
         LabelMin = asset.ScalingAxis_Min[4]
         break
      if case( 'Sec3' ):
         LabelMax = asset.ScalingAxis_Max[5]
         LabelMin = asset.ScalingAxis_Min[5]
         break
      break

   Suffix      = Avr[ sName + 'LabelSuffix'   ]
   Headline    = Avr[ sName + 'LabelHeadline' ]
   Range       = LabelMax - LabelMin

   c           = 0
   Label_Line  = 0
   LabelPos    = int( Avr[sName + 'LabelPos']    ) / 10
   LabelOffset = int( Avr[sName + 'LabelOffset'] ) / 10

   if sName in [ 'Pri1', 'Pri2', 'Pri3' ] :
      x  = svg.PlotEdgeLeft - LabelPos
      x1 = svg.PlotEdgeLeft - ( int(Avr["LegAxisLine"]) / 10 )

   if sName in [ 'Sec1', 'Sec2', 'Sec3' ] :
      x  = svg.PlotEdgeLeft + svg.pw + LabelPos
      x1 = svg.PlotEdgeLeft + svg.pw

   while switch( sName ):
      # -----------------------------------------------------------------------------
      if case( 'Pri1' ):
         Label_Line    = 1
         break
      # -----------------------------------------------------------------------------
      if case( 'Pri2' ):
         if Avr["Pri1LabelOn"] == "Yes" :
            x         -= svg.PlotLabelWidth
         else :
            Label_Line = 1
         break
      # -----------------------------------------------------------------------------
      if case( 'Pri3' ):

         if Avr["Pri1LabelOn"] != "Yes" and Avr["Pri2LabelOn"] != "Yes" :
            Label_Line = 1

         if Avr["Pri1LabelOn"] == "Yes" : x -= svg.PlotLabelWidth
         if Avr["Pri2LabelOn"] == "Yes" : x -= svg.PlotLabelWidth
         break
      # -----------------------------------------------------------------------------
      if case( 'Sec1' ):
         Label_Line    = 1
         break
      # -----------------------------------------------------------------------------
      if case( 'Sec2' ):
         if Avr["Sec1LabelOn"] == "Yes" :
            x         += svg.PlotLabelWidth
         else :
            Label_Line = 1
         break
      # -----------------------------------------------------------------------------
      if case( 'Sec3' ):

         if Avr["Sec1LabelOn"] != "Yes" and Avr["Sec2LabelOn"] != "Yes" :
            Label_Line = 1

         if Avr["Sec1LabelOn"] == "Yes" : x += svg.PlotLabelWidth
         if Avr["Sec1LabelOn"] == "Yes" : x += svg.PlotLabelWidth
         break
      # -----------------------------------------------------------------------------
      break

   if Label_Line == 1 :
      HorLine = [ f"\t\t\t<g id=\"{sID}Line\" stroke-width=\"0.9\" stroke=\"{Avr[sName + 'LabelColor']}\">\n" ]

   HorText = [ f"\t\t\t<g id=\"{sID}Label\" text-anchor=\"middle\" font-family=\"'ArialMT'\" font-size=\"{int(Avr[sName + 'LabelSize']) / 10}\" fill=\"{Avr[sName + 'LabelColor']}\">\n"  ]

   # --- Fixed divider of 10
   MainInterval = round( ( abs( LabelMax - LabelMin ) ) / 10 , iRnd )
   Inc          = round( svg.ph / ( Range / MainInterval ), iRnd )
   lStep        = round( log( LabelMax - LabelMin ) / 10, 5 )
   iCount       = 0

   yz  = svg.PlotEdgeTop + svg.ph

   if Avr[ 'LegAxisHeadlineOn' ] == 'Yes' and Avr[ 'LegAxisHeadPos' ] == "Bottom" :
      y  = yz + svg.LegAxisHeadOffset + LabelOffset
      
      HorText.append( f'\t\t\t\t<text x="{x:.2f}" y="{y:.2f}">{Headline}</text>\n' )

   # -----------------------------------------------------------------------
   PathHight = int(Avr["LegAxisLine"]) / 10

   for i in xrange( LabelMin, LabelMax, MainInterval, '<=' ) :
      y1 = yz - c
      y  = y1 + LabelOffset

      if Label_Line == 1 :
         HorLine.append( f'\t\t\t\t<path d="M{x1:.2f},{y1:.2f} h{PathHight}"/>\n' )

      # --------------------------------------------------------------------
      if Avr[ sName + 'ScaleType' ] == 'Log' :
         iValue  = round( ( 10 ** ( iCount * lStep ) ) + LabelMin, 1 )
         if iCount == 0 : iValue -= 1
         iCount += 1
      else :
         iValue = i

      # --------------------------------------------------------------------
      if iValue >= 10 : LabelValue = f'{int(iValue)}'
      else :            LabelValue = f'{iValue:.1f}'

      HorText.append( f'\t\t\t\t<text x="{x:.2f}" y="{y:.2f}">{LabelValue}</text>\n' )
      c = c + Inc

   # -----------------------------------------------------------------------
   if Avr[ 'LegAxisUnitOn'     ] == 'Yes' :
      y1 = y1 - svg.LegAxisOffset
      y  = y1 + LabelOffset
      HorText.append( f'\t\t\t\t<text x="{x:.2f}" y="{y:.2f}">{Suffix}</text>\n' )

   if Avr[ 'LegAxisHeadlineOn' ] == 'Yes' and Avr[ 'LegAxisHeadPos' ] == "Top" :
      y1 = y1 - svg.LegAxisHeadOffset
      y  = y1 + LabelOffset
      HorText.append( f'\t\t\t\t<text x="{x:.2f}" y="{y:.2f}">{Headline}</text>\n' )

   if Label_Line == 1 :
      Plot.append( f'{ "".join( HorLine ) }\t\t\t</g> <!-- {sID} LineEnd -->\n' )

   Plot.append( f'{ "".join( HorText) }\t\t\t</g> <!-- {sID} LabelEnd -->\n' )

   # -----------------------------------------------------------------------
   return ''.join( Plot )

# --------------------------------------------------------------------------
# --- Prepare SVG-Grid(s) ....
# --------------------------------------------------------------------------
def PreGrid() :

   svg.Plot[ 'Pri1MainGrid' ] = Grid( "Pri1", "Main", "h" )
   svg.Plot[ 'Pri1HelpGrid' ] = Grid( "Pri1", "Help", "h" )
   svg.Plot[ 'VerMainGrid'  ] = Grid( "Ver",  "Main", "v" )

# --------------------------------------------------------------------------
# --- Draw Grid
# --------------------------------------------------------------------------
def Grid( sAxis, sName, Alignment ) :

   Avr  = asset.AvrConf
   iRnd = asset.rnd_SVG

   Plot = []
   if Avr[sAxis + sName + 'GridOn'] != "Yes" : return ''

   gName = sAxis + sName

   c = 0
   a = 0
   b = 0

   while switch( sAxis + '-' + sName ):
      if case( 'Pri1-Main' ):
         Max = asset.ScalingAxis_Max[0] / 10
         Min = asset.ScalingAxis_Min[0] / 10
         # --- Fixed divider of 10
         Step = round( ( Max - Min ) / 10 , iRnd )
         Main = Step
         break
      if case( 'Pri1-Help' ):
         Max = asset.ScalingAxis_Max[0] / 10
         Min = asset.ScalingAxis_Min[0] / 10
         # --- Fixed divider of 50
         Step = round( ( Max - Min ) / 10 / 5, iRnd )
         # --- Fixed divider of 10
         Main = round( ( Max - Min ) / 10, iRnd )
         break
      if case( 'Ver-Main' ):
         Max  = asset.sys_MaxLines
         Min  = 0
         Step = asset.svg_TimeStep
         Main = Step
         break
      break

   if Alignment == "h" :
      Inc   = round( svg.ph / ( ( Max - Min ) / Step ), iRnd )
      h     = svg.pw
      x     = svg.PlotEdgeLeft
      yz    = svg.PlotEdgeTop + svg.ph
   else :
      Inc   = round( svg.pw / ( asset.sys_MaxLines / asset.svg_TimeStep ), iRnd )
      h     = svg.ph
      y     = svg.PlotEdgeTop
      xz    = svg.PlotEdgeLeft

   StrokeWidth = int ( Avr[gName + 'GridStroke'] ) / 10
   Plot.append( f"\t\t\t<g id=\"{gName}Grid\" fill=\"none\" stroke-width=\"{StrokeWidth}\" stroke=\"{Avr[gName + 'GridColor']}\">\n" )


   for i in xrange( Min, Max, Step, '<=' ) :
      if a == b and sName == "Help" :
         a = a + Main
      else :
         if Alignment == "h" : y = round( yz - c, iRnd )
         else :                x = round( xz + c, iRnd )

         Plot.append( f'\t\t\t\t<path d="M{x:.2f},{y:.2f} {Alignment}{h:.2f}"/>\n' )

      b = round( b + Step, iRnd )
      c = round( c + Inc,  iRnd )

   Plot.append( f'\t\t\t</g> <!-- {gName} GridEnd -->\n' )

   # -----------------------------------------------------------------------
   return ''.join( Plot )

# --------------------------------------------------------------------------
# --- Prepare SVG-Header
# --------------------------------------------------------------------------
def PreHeader() :

   Avr = asset.AvrConf

   x1 = svg.PlotEdgeLeft
   y1 = svg.PlotEdgeTop
   x2 = svg.PageSizeWidht  - svg.PlotEdgeLeft - svg.PlotEdgeRight
   y2 = svg.PageSizeHeight - svg.PlotEdgeTop  - svg.PlotEdgeBottom

   BorderStroke = int( Avr["BorderStroke"] ) / 10
   BorderFill   = Avr["BorderFill"]
   if BorderFill == "#FFFFFF" : BorderFill = "none"

   # ----------------------------------------------------------
   Plot = [      '<?xml version="1.0" encoding="utf-8"?>\n' ]
   Plot.append( f'<!-- Generator: {asset.SvgEngine} -->\n' )
   Plot.append(  '<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd" [\n' )
   Plot.append(  '\t<!ENTITY ns_svg "http://www.w3.org/2000/svg">\n' )
   Plot.append(  '\t<!ENTITY ns_xlink "http://www.w3.org/1999/xlink">\n' )
   Plot.append(  ']>\n' )
   Plot.append( f'<svg version="1.2" baseProfile="tiny" id="{asset.SvgPageID}" xmlns="&ns_svg;" xmlns:xlink="&ns_xlink;"\n' )
   Plot.append( f'\twidth="{svg.PageSizeWidht}" height="{svg.PageSizeHeight}"\n' )
   Plot.append( f'\tviewBox="0 0 {svg.PageSizeWidht} {svg.PageSizeHeight}"\n' )
   Plot.append(  '\toverflow="visible"\n' )
   Plot.append( f'\tenable-background="new 0 0 {svg.PageSizeWidht} {svg.PageSizeHeight}"\n' )

   Plot.append(  '\txml:space="preserve">' )
   Plot.append(  '\n' )
    # --- Clipping Area ---------------------------------------
   Plot.append(  '<defs>\n' )
   Plot.append(  '\t<clipPath id="PlotArea" clipPathUnits="userSpaceOnUse">\n' )
   Plot.append( f'\t\t<rect x="-1.0" y="-1.0" width="{x2 + 2.0}" height="{y2 + 2.0}"/>\n' )
   Plot.append(  '\t</clipPath>\n' )

   Plot.append(  '</defs>\n' )
   # ---------------------------------------------------------
   Plot.append(  '\t<g id="Page">\n' )
   Plot.append( f"\t\t<g id=\"Border\" fill=\"{BorderFill}\" stroke=\"{Avr['BorderColor']}\" stroke-width=\"{BorderStroke}\" >\n" )
   Plot.append( f'\t\t\t<rect x="{svg.BorderX}" y="{svg.BorderY}" width="{svg.BorderWidth}" height="{svg.BorderHeight}"/>\n' )
   Plot.append(  '\t\t</g> <!-- BorderEnd -->\n' )

   if svg.PlotAreaFill != "none" :
      Plot.append( f'\t\t<g id="PlotFrame" fill="{svg.PlotAreaFill}" stroke="none" stroke-width="0.1">\n' )
      Plot.append( f'\t\t\t<rect x="{x1}" y="{y1}" width="{x2}" height="{y2}"/>\n' )
      Plot.append(  '\t\t</g> <!-- PlotFrameEnd -->' )

   # ---------------------------------------------------------
   svg.Plot[ 'header' ] = ''.join( Plot )
# --------------------------------------------------------------------------
# --- Prepare SVG-Legend
# --------------------------------------------------------------------------
def PreLegend() :

   Avr = asset.AvrConf

   svg.Plot["legend"] = ""
   if Avr["LegLabelOn"] != "Yes" : return

   LegLabelSize = int ( Avr["LegLabelSize"] ) / 10
   Plot         = []

   # ----------------------------------------------------------
   if svg.LegZeroX == "Split" :
   #  OrgLegZeroX  = svg.LegZeroX
   #  Axis = [ 1, 3, 0, 2 ]
      Axis = [ 1, 3, 5, 0, 2, 4 ]

      svg.Eth_Flag = False
      svg.Md_Flag  = False

      for j in Axis :

         svg.LegColCount = 0
         svg.LegRowCount = 0

         while switch( j ):
            if case( 0 ):
               svg.LegZeroX       = "Left"
               svg.LegLabelAnchor = "start"
               if asset.ScalingAxis_Names[2] : svg.LegRowCount = 1    # <<< (Pri1 links)  Temperature °C
               break
            if case( 1 ):
               svg.LegZeroX       = "Right"
               svg.LegLabelAnchor = "end"
               if asset.ScalingAxis_Names[3] : svg.LegRowCount = 1    # <<< (Sec1 rechts) Load percent %
               break
            if case( 2 ):
               svg.LegZeroX       = "Left"
               svg.LegLabelAnchor = "start"
               if asset.ScalingAxis_Names[4] : svg.LegRowCount = 2    # <<< (Pri2 links)  Network I/O MiB/s
               break
            if case( 3 ):
               svg.LegZeroX       = "Right"
               svg.LegLabelAnchor = "end"
               if asset.ScalingAxis_Names[5] : svg.LegRowCount = 2    # <<< (Sec2 rechts) Storage I/O MiB/s
               break
            if case( 4 ):
               svg.LegZeroX       = "Left"
               svg.LegLabelAnchor = "start"
            #  if asset.ScalingAxis_Names[2] : svg.LegRowCount = 1    # <<< (Pri3 links)  Ookla Ping ms
               break
            if case( 5 ):
               svg.LegZeroX       = "Right"
               svg.LegLabelAnchor = "end"
            #  if asset.ScalingAxis_Names[3] : svg.LegRowCount = 1    # <<< (Sec3 rechts) USV AC-V
               break
            break

         AlignmentLabel()

         Plot.append( f'\t\t<g id="Legend" text-anchor="{svg.LegLabelAnchor}" font-family="\'ArialMT\'" font-size="{LegLabelSize}" stroke-width="0.1">\n' )

         # ---------------------------------------------------------
         if asset.ScalingAxis_Names[j] :
            for dev in asset.ScalingAxis_Names[j] :
               Plot.append( LegendLabel( asset.BaseName[dev], j, dev ) )
         # ---------------------------------------------------------
         Plot.append( '\t\t</g> <!-- LegendEnd -->\n' )

      svg.LegZeroX = "Split"
   # ----------------------------------------------------------
   else :
      svg.LegColCount = 0
      svg.LegRowCount = 0

      AlignmentLabel()

      if svg.LegAlignment == "Hor" and Avr["LegBreakOn"] == "Yes" :
         svg.Eth_Flag = True
         svg.Md_Flag  = True
      else :
         svg.Eth_Flag = False
         svg.Md_Flag  = False

      Plot.append( f'\t\t<g id="Legend" text-anchor="{svg.LegLabelAnchor}" font-family="\'ArialMT\'" font-size="{LegLabelSize}" stroke-width="0.1">\n' )

      # ---------------------------------------------------------
      for dev in asset.BaseList :
         Plot.append( LegendLabel( asset.BaseName[dev], asset.BaseNum[dev], dev ) )

      # ---------------------------------------------------------
      Plot.append( '\t\t</g> <!-- LegendEnd -->\n' )

   # ----------------------------------------------------------
   svg.Plot[ 'legend' ] = ''.join( Plot )

# --------------------------------------------------------------------------
# --- Create Legend of Plot-Sensors (Label & Sign) ...
# --------------------------------------------------------------------------
def LegendLabel( sName, iAxis, iTab ) :

   Avr = asset.AvrConf

   while switch( iAxis ):
      if case( 2 ):
         if svg.Eth_Flag :
            svg.LegColCount  = 0
            svg.LegRowCount += 1
            svg.Eth_Flag     = False
         break
      if case( 3 ):
         if svg.Md_Flag :
            svg.LegColCount  = 0
            svg.LegRowCount += 1
            svg.Md_Flag      = False
         break
      break

   LegendColor = Avr['Plot' + sName + 'Color']
   Plot = [ f'\t\t\t<g id="Legend {sName}" fill="{LegendColor}" stroke="{LegendColor}">\n' ]

   # ----------------------------------------------------------
   if svg.LegZeroY == "Top" :
      # --- Zero-Y : Top
      if svg.LegAlignment == "Hor" : y = svg.LegColCount * svg.LegendTab_y + svg.LegendZero_y - svg.LegOffsetY - (( svg.LegendCount - svg.LegRowCount - 1 ) * svg.LegTabY )
      else :                         y = svg.LegColCount * svg.LegendTab_y + svg.LegendZero_y
   else :
      # --- Zero-Y : Bottom
      if svg.LegAlignment == "Hor" : y = svg.LegColCount * svg.LegendTab_y + svg.LegendZero_y + svg.LegOffsetY + svg.pw_v + (svg.LegRowCount * svg.LegTabY )
      else :                         y = svg.LegColCount * svg.LegendTab_y + svg.LegendZero_y

   # ----------------------------------------------------------
   x = svg.LegColCount * svg.LegendTab_x + svg.LegendZero_x
   Scale = f'{ ( svg.LegSignSize + 2.0 ) / 100:.3f}'

   # ----------------------------------------------------------
   # --- Ein Legenden-Label untersteichen in Farbe und Stil der Chart-Line ...
   if Avr['LegUndelineOn'] == 'Yes' :
      LineStyle = sStyle[ int( Avr['Plot' + asset.BaseName[iTab] + 'Style'] ) ]

      iLineWidth  = svg.LegTabX * float( Avr[ 'LegUndelineSize' ] )
      if svg.LegZeroX != "Left" : xLine = x + svg.LegTabX - iLineWidth
      else                      : xLine = x 

      sOpt =   {  'iXpos'           : '%.2f' % ( xLine ),
                  'iYpos'           : '%.2f' % ( y  + ( svg.LegTabY / 2 ) ),
                  'iWidth'          : '%.2f' % ( iLineWidth ),
                  'iHeight'         : '%.2f' % ( 0 ),
                  'stroke'          : Avr['Plot' + sName + 'Color'],
                  'stroke-opacity'  : Avr['Plot' + asset.BaseName[iTab] + 'Opacity'],
                  'stroke-width'    : '%.2f' % ( int( Avr['Plot' + asset.BaseName[iTab] + 'Stroke'] ) / 10  ),
                  'style'           : LineStyle,
               }

      Plot.append( '\t\t\t\t<path stroke-opacity="{stroke-opacity}" {style} stroke-width="{stroke-width}" stroke="{stroke}" d="M{iXpos},{iYpos} l{iWidth},{iHeight}"/>\n'.format( **sOpt ) )

   # ----------------------------------------------------------
   # Signs: Line / Circle / Square / UpArrow / DownArrow
   while switch( asset.BaseSign[iTab] ):
      # -------------------------------------------------------
      if case( 'Line' ) :
         h = 2.0
         w = svg.LegSignSize

         if svg.LegZeroX == "Left" : x1 = x
         else :                      x1 = x + svg.LegTabX - svg.LegSignSize

         y1 = y - ( h / 2 )

         Plot.append( f'\t\t\t\t<rect x="{x1}" y="{y1}" width="{w}" height="{h}"/>\n' )
         break

      # -------------------------------------------------------
      if case( 'Circle' ) :
         r = svg.LegSignSize / 2    # = Radius

         if svg.LegZeroX == "Left" : x1 = x + r
         else :                      x1 = x + svg.LegTabX - r

         y1 = y

         Plot.append( f'\t\t\t\t<circle cx="{x1}" cy="{y1}" r="{r}"/>\n' )
         break

      # -------------------------------------------------------
      if case( 'Square' ) :
         h = svg.LegSignSize # / 2
         w = h

         if svg.LegZeroX == "Left" : x1 = x
         else :                      x1 = x + svg.LegTabX - h

         y1 = y - ( h / 2 )

         Plot.append( f'\t\t\t\t<rect x="{x1}" y="{y1}" width="{w}" height="{h}"/>\n' )
         break

      # -------------------------------------------------------
      if case( 'UpArrow' ) :
         h = svg.LegSignSize / 2

         if svg.LegZeroX == "Left" : x1 = x + h
         else :                      x1 = x + svg.LegTabX - h

         y1 = y
         Plot.append( f'\t\t\t\t<path d="M 0 0 m -26 50 l 0 -50 l -24 0 l 43 -50 l 43 50 l -24 0 l 0 50 l -38 0" transform="translate({x1} {y1}) scale({Scale})"/>\n' )
         break

      # -------------------------------------------------------
      if case( 'DownArrow' ) :
         h = svg.LegSignSize / 2

         if svg.LegZeroX == "Left" : x1 = x + h
         else :                      x1 = x + svg.LegTabX - h

         y1 = y

         Plot.append( f'\t\t\t\t<path d="M 0 0 m -26 -50 l 0 50 l -24 0 l 43 50 l 43 -50 l -24 0 l 0 -50 l -38 0" transform="translate({x1} {y1}) scale({Scale})"/>\n' )
         break
      # -------------------------------------------------------
      break

   LegLabelOffset = int ( Avr["LegLabelOffset"] ) / 10

   if svg.LegZeroX == "Left" :
      x1 = x + svg.LegSignSize + ( svg.LegSignSize / 2 )
      y1 = y + LegLabelOffset
   else :
      x1 = x + svg.LegTabX - svg.LegSignSize - ( svg.LegSignSize / 2 )
      y1 = y + LegLabelOffset

   if Avr["LegColorOn"] == "No" : LabelColor = Avr["LegLabelColor"]
   else                         : LabelColor = svg.svg_LabelColor[ iAxis ]

   Plot.append( f'\t\t\t\t<text fill="{LabelColor}" stroke="{LabelColor}" x="{x1:.2f}" y="{y1:.2f}">{Avr[f"Plot{sName}Label"]}</text>\n' )
   Plot.append(  '\t\t\t</g> <!-- Legend' + sName + 'End -->\n' )

   svg.LegColCount += 1

   if svg.LegAlignment == "Hor" and Avr["LegBreakOn"] == "No" and Avr["LegZeroX"] != "Split" :
      if svg.LegColCount >= int(Avr["LegBreakCount"]) :
         svg.LegColCount  = 0
         svg.LegRowCount += 1

   # ----------------------------------------------------------
   return ''.join( Plot )

# --------------------------------------------------------------------------
# --- Prepare Origin of X / Y - Position ...
# --------------------------------------------------------------------------
def AlignmentLabel() :

   # ----------------------------------------------------------
   if svg.LegAlignment == "Hor" :
      # Alignment : Hor
      svg.LegendTab_x = svg.LegTabX
      svg.LegendTab_y = 0
   else :
      # Alignment : Ver
      svg.LegendTab_x = 0
      svg.LegendTab_y = svg.LegTabY

   # ----------------------------------------------------------
   if svg.LegZeroX == "Left" : 
      # -------------------------------------------------------
      # --- Zero-X : Left
      if svg.LegAlignment == "Hor" : svg.LegendZero_x = svg.PlotEdgeLeft + svg.LegOffsetX                            # Left - Hor
      else :                         svg.LegendZero_x = svg.PlotEdgeLeft - svg.LegTabX - svg.LegOffsetX - svg.pw_l   # Left - Ver
   else :
      # -------------------------------------------------------
      # --- Zero-X : Right
      svg.LegendTab_x = svg.LegendTab_x * -1
      if svg.LegAlignment == "Hor" : svg.LegendZero_x = svg.PlotEdgeLeft + svg.pw - svg.LegTabX - svg.LegOffsetX     # Right - Hor
      else :                         svg.LegendZero_x = svg.PlotEdgeLeft + svg.pw + svg.pw_r    + svg.LegOffsetX     # Right - Ver

   # ----------------------------------------------------------
   if svg.LegZeroY == "Top" :
      # -------------------------------------------------------
      # --- Zero-Y : Top
      if svg.LegAlignment == "Hor" : svg.LegendZero_y = svg.PlotEdgeTop - ( svg.LegTabY / 2 )                        # Top - Hor
      else :                         svg.LegendZero_y = svg.PlotEdgeTop + ( svg.LegTabY / 2 )                        # Top - Ver
   else :
      # -------------------------------------------------------
      # --- Zero-Y : Bottom
      svg.LegendTab_y = svg.LegendTab_y * -1
      if svg.LegAlignment == "Hor" : svg.LegendZero_y = svg.PlotEdgeTop + svg.ph + ( svg.LegTabY / 2 )               # Bottom - Hor
      else :                         svg.LegendZero_y = svg.PlotEdgeTop + svg.ph - ( svg.LegTabY / 2 )               # Bottom - Ver

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
