# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#
# --- Usbview.py based on usbutils/lsusb.py.in (usbutils 012) :
# -->    https://github.com/gregkh/usbutils
# ---    Copyright (c) 2009 Kurt Garloff <garloff@suse.de>
# ---    Copyright (c) 2013,2018 Kurt Garloff <kurt@garloff.de>
# --- Various changes made to get the best fit for 'AvrLogger'
# -----------------------------------------------------------------------------------#

import getopt
import os
import re
import sys

import asset

# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
#  HUB_ICLASS     = 0x09
HUB_ICLASS     = '09'

f_Debug        = False

f_showIFs      = False
f_showHubIFs   = False
f_noEmptyHub   = False
f_noHub        = False
f_showEPs      = False
f_useColors    = None
f_showWarn     = True
f_showTree     = False
f_preColor     = None

prefix      = '/sys/bus/usb/devices/'

iUsbIDs      = [ asset.avrUsbIdsFiles,
                 '/usr/syno/etc/usb.map', ]

devlst      = [
               'host',              # usb-storage
               'video4linux/video', # uvcvideo et al.
               'sound/card',        # snd-usb-audio
               'net/',              # cdc_ether, ...
               'input/input',       # usbhid
               'usb:hiddev',        # usb hid
               'bluetooth/hci',     # btusb
               'ttyUSB',            # btusb
               'tty/',              # cdc_acm
               'usb:lp',            # usblp
            #  'usb/lp',            # usblp
               'usb/',              # hiddev, usblp
            #  'usbhid',            # hidraw
               ]

iColors     = { 'escNormal'   : '\033[0;0m' ,      # 0
                'cssNormal'   : '',                # 0
                'escBold'     : '\033[0;1m' ,      # 1
                'cssBold'     : 'Bold',            # 1
                'escRed'      : '\033[0;31m',      # 2
                'cssRed'      : 'Red',             # 2
                'escGreen'    : '\033[0;32m',      # 3
                'cssGreen'    : 'Lime',            # 3
                'escAmber'    : '\033[0;33m',      # 4
                'cssAmber'    : 'Yellow',          # 4
                'escBlue'     : '\033[0;34m',
                'cssBlue'     : 'Blue',
                'escMagenta'  : '\033[0;35m',      # 5
                'cssMagenta'  : 'MediumVioletRed', # 5
                'escCyan'     : '\033[0;36m',
                'cssCyan'     : 'MediumBlue',
                  }

UsbVendors     = {}
UsbProducts    = {}
UsbClasses     = {}

sIntent     = '%-18s %s\n'
sNoIntend   = '%s\n'
sTree       = '|__'
sTrEP       = '\\__'

deDescriptors  = (   'bDeviceClass',
                     'bDeviceSubClass',
                     'bDeviceProtocol',
                     'bMaxPacketSize0',
                     'idVendor',
                     'idProduct',
                     'bcdDevice',
                     'manufacturer',
                     'product',
                     'serial',
                     'bNumConfigurations',
                     'bNumInterfaces',
                     'bConfigurationValue',
                     'bmAttributes',
                     'bMaxPower',
                     'authorized',
                     'busnum',
                     'dev',
                     'devnum',
                     'devpath',
                     'maxchild',
                     'speed',
                     'urbnum',
                     'version',
                     'quirks',
                     'avoid_reset_quirk',
                     'configuration',
                     'ltm_capable',
                     'removable',
                     'syno_quirks', )

# -----------------------------------------------------------------------------------
# --- Get and return USB-Device Descriptor :
# -----------------------------------------------------------------------------------
def getDeviceDescriptor( sPath ) :

   sRet = dict()

   # -----------------------------------------------------------------------
   for Name in deDescriptors :
      sRet[ Name ] = getAttr( sPath, Name )

   # -----------------------------------------------------------------------
   sRet[ 'idVendor'     ] = getAttr( sPath, 'idVendor'  ).upper()
   sRet[ 'idProduct'    ] = getAttr( sPath, 'idProduct' ).upper()

   # -----------------------------------------------------------------------
   if not sRet[ 'manufacturer' ] or not sRet[ 'product' ] :
      sRet[ 'manufacturer' ] = FindUsbVendor ( sRet[ 'idVendor' ], True )
      sRet[ 'product' ]      = FindUsbProduct( sRet[ 'idVendor' ], sRet[ 'idProduct' ], True )

   # -----------------------------------------------------------------------
   # --- Some USB Card readers have a better name then Generic ...
   if sRet[ 'manufacturer' ].startswith( 'Generic' ) :

      sTemp = FindUsbVendor( sRet[ 'idVendor' ], True )
      if sTemp : sRet[ 'manufacturer' ] = sTemp

      sTemp = FindUsbProduct( sRet[ 'idVendor' ], sRet[ 'idProduct' ], True )
      if sTemp : sRet[ 'product' ] = FindUsbProduct( sRet[ 'idVendor' ], sRet[ 'idProduct' ], True )

   # -----------------------------------------------------------------------
   return sRet

# -----------------------------------------------------------------------------------
# --- Return a Attribut :
# -----------------------------------------------------------------------------------
def getAttr ( sFile, sName ) :

   sAttr = sFile + '/' + sName
   try :
      with open( sAttr, 'r', encoding='utf-8' ) as f:
         return ' '.join( f.read().split() )
   except :
      return False

# -----------------------------------------------------------------------------------
def Colorize( sColor, sText, iColor=False, iSytle=False ):

   global f_useColors, f_preColor

   if iColor : f_useColors = iColor
   if iSytle : f_preColor  = iSytle

   if not f_useColors : return sText

   Color = iColors.get( f_preColor + sColor, False )

   if f_preColor == 'esc' :
      return '{0}{1}{2}'.format( iColors[ 'esc' + sColor ], sText, iColors[ 'escNormal' ] )
   else :
      if not Color or Color == 'Normal' : return sText
      elif            Color == 'Bold'   : return '<b>{0}</b>'.format( sText )
      else                              : return '<b class="info" style="color: {0};">{1}</b>'.format( Color, sText )

# -----------------------------------------------------------------------------------
# --- return True if all digits are valid hex digits ...
# -----------------------------------------------------------------------------------
def isHexDigit( str ) :

   for dg in str :
      if not dg.isdigit() and not dg in 'aAbBcCdDeEfF' : return False

   return True

# -----------------------------------------------------------------------------------
# --- Parse /usr/share/usb.ids and fill UsbVendors, UsbProducts, UsbClasses ...
# -----------------------------------------------------------------------------------
def ParseUsbIDs( opt=False ) :

   global iUsbIDs, UsbVendors, UsbProducts, UsbClasses

   vid      = ''
   did      = ''
   mode     = 'Vendor'
   strg     = ''
   cstrg    = ''

   # --------------------------------------------------------------------------------
   # --- If 'opt' : Act as Import-Module ...
   if opt :
      iUsbIDs        = [ opt ]
      UsbVendors     = {}
      UsbProducts    = {}
      UsbClasses     = {}

   # --------------------------------------------------------------------------------
   # --- Search for usb.ids ...
   for sFile in iUsbIDs :
      sID = os.path.exists( sFile )
      if sID : break

   # --------------------------------------------------------------------------------
   if not sID : return False

   # --------------------------------------------------------------------------------
   with open( sFile, 'r', encoding='utf-8' ) as f:
      for line in f:

         # --------------------------------------------------------------------------
         # --- Parse usb.ids line by line ...
         ln = line.rstrip()

         # --------------------------------------------------------------------------
         # --- Ignore blank or comment lines ...
         if not ln or ln[0] == '#' : continue

         # --------------------------------------------------------------------------
         if isHexDigit( ln[0:4].upper() ) :
            mode              = 'Vendor'
            vid               = ln[:4].upper()
            UsbVendors[ vid ] = ln[6:].strip()
            continue

         # --------------------------------------------------------------------------
         if ln[0] == '\t' and isHexDigit( ln[1:3] ) :
            did = ln[1:5].strip().upper()

            # -----------------------------------------------------------------------
            if mode == 'Vendor' :
               UsbProducts[ vid, did ] = ln[7:].strip()
               continue
            elif mode == 'Class' :
               nm = ln[5:].strip()
               if nm != 'Unused' :
                  strg = cstrg + ':' + nm
               else :
                  strg = cstrg + ':'

               UsbClasses[ cid, did, 'FF' ] = strg
            #  UsbClasses[ vid, did, 'FF' ] = strg
               continue

         # --------------------------------------------------------------------------
         if ln[0] == 'C' :
            mode = 'Class'
            cid   = ln[2:4].strip().upper()
            cstrg = ln[6:].strip()
            UsbClasses[ cid, 'FF', 'FF' ] = cstrg
            continue

         # --------------------------------------------------------------------------
         if mode == 'Class'     and ln[0] == '\t' and ln[1] == '\t' and isHexDigit( ln[2:4] ) :
            prid = ln[2:4].strip().upper()
            UsbClasses[cid, did, prid] = ln[6:].strip()
            continue

         mode = 'Misc'

      # -----------------------------------------------------------------------------
      UsbClasses[ 'FF', 'FF', 'FF' ] = 'Vendor Specific'

   # --------------------------------------------------------------------------------
   if not opt : return True
   else       : return UsbVendors, UsbProducts, UsbClasses

# -----------------------------------------------------------------------------------
# --- Return USB Vendor
# -----------------------------------------------------------------------------------
def FindUsbVendor( vID, opt=False ) :

   sRet = UsbVendors.get( vID.upper() )
   if sRet : return sRet
   else :
      if opt : return False 
      else   : return 'NA'

# -----------------------------------------------------------------------------------
# --- Return USB Product by Vendor
# -----------------------------------------------------------------------------------
def FindUsbProduct( vID, pID, opt=False ) :

   sRet = UsbProducts.get( ( vID.upper(), pID.upper() ) )
   if sRet : return sRet
   else :
      if opt : return False 
      else   : return 'NA'

# -----------------------------------------------------------------------------------
# --- Return device name from USB Vendor:Product list ...
# -----------------------------------------------------------------------------------
def FindUsbProd( vID, pID ) :

   sRet = ''

   sVendor = UsbVendors.get( vID.upper() )
   if sVendor :
      sRet = sVendor
   else :
      return ''

   sProduct = UsbProducts.get( ( vID.upper(), pID.upper() ) )
   if sProduct :
   #  return strg + ' ' + str( product )
      return sRet + ' @ ' + sProduct

   return sRet

# -----------------------------------------------------------------------------------
# --- Return USB protocol from usbclasses list ...
# -----------------------------------------------------------------------------------
def FindUsbClass( cID, sID, pID ) :

   sRet = UsbClasses.get( ( cID,  sID,  pID ) ) \
      or  UsbClasses.get( ( cID,  sID, 'FF' ) ) \
      or  UsbClasses.get( ( cID, 'FF', 'FF')  )

   if sRet : return sRet
   else    : return 'unknown'

# -----------------------------------------------------------------------------------
# --- Return SCSI block dev names for host ...
# -----------------------------------------------------------------------------------
def FindStorage( hostno ) :

   res = ''
   for ent in os.listdir( '/sys/class/scsi_device/' ) :
      ( host, bus, tgt, lun ) = ent.split( ':' )
      if host == hostno :
         try :
            for ent2 in os.listdir( '/sys/class/scsi_device/%s/device/block' % ent ) :
               res += ent2 + ' '
         except :
            pass

   if not res : return 'unmounted '
   else       : return res

# -----------------------------------------------------------------------------------
def AddDriver( path, drvnm ) :

   res = ''
   try :
      for e2 in os.listdir( path + '/' + drvnm ) :
         if e2[0:len( drvnm )] == drvnm :
            res += e2 + ' '
      try :
         if res :
            res += '(' + os.path.basename( os.readlink( path + '/driver' ) ) + ') '
      except :
         pass
   except :
      pass

   return res

# -----------------------------------------------------------------------------------
# --- Return pseudo devname that's driven by driver ...
# -----------------------------------------------------------------------------------
def FindDevName( driver, usbname ) :

   res = ''
   for nm in devlst :
      dirnm = prefix + usbname
      prep = ''
      idx = nm.find( '/' )
      if idx != -1 :
         prep = nm[:idx+1]
         dirnm += '/' + nm[:idx]
         nm = nm[idx+1:]

      ln = len( nm )
      try :
         for ent in os.listdir( dirnm ) :
            if ent[:ln] == nm :
               res += prep + ent + ' '
               if nm == 'host' :
                  res += '(' + FindStorage( ent[ln:] )[:-1] + ')'
      except :
         pass

   if driver == 'usbhid' :
      rg = re.compile( r'[0-9A-F]{4}:[0-9A-F]{4}:[0-9A-F]{4}\.[0-9A-F]{4}' )
      for ent in os.listdir( prefix + usbname ) :
         m = rg.match( ent )
         if m :
            res += AddDriver( prefix + usbname + '/' + ent, 'hidraw' )
            add  = AddDriver( prefix + usbname + '/' + ent, 'input'  )
            if add :
               res += add
            else :
               for ent2 in os.listdir( prefix + usbname + '/' + ent ) :
                  m = rg.match( ent2 )
                  if m :
                     res += AddDriver( prefix + usbname + '/' + ent + '/' + ent2, 'input' )

   return res

# -----------------------------------------------------------------------------------
class UsbObject :

   def Attr( self, name ) :
      path = prefix + self.path + '/' + name
      return ' '.join( open( path, 'r', encoding='utf-8' ).readline().strip().split() )

   def Link( self, name ) :
      path = prefix + self.path + '/' + name
      return os.path.basename( os.readlink( path ) )

# -----------------------------------------------------------------------------------
# --- Container for USB endpoint info ...
# -----------------------------------------------------------------------------------
class UsbEndpoint( UsbObject ) :

   def __init__( self, parent, fname, level ) :
      self.parent    = parent
      self.level     = level
      self.fname     = fname
      self.path      = ''
      self.epaddr    = 0
      self.len       = 0
      self.ival      = ''
      self.type      = ''
      self.attr      = 0
      self.max       = 0
      if self.fname :
         self.read( self.fname )

   def read( self, fname ) :
      self.fname     = fname
      self.path      = self.parent.path + '/' + fname
      self.epaddr    = int( self.Attr( 'bEndpointAddress' ), 16 )
      ival           = int( self.Attr( 'bInterval' ), 16 )
      if ival :
         self.ival   = ' (%s)' % self.Attr( 'interval' )

      self.len       = int( self.Attr( 'bLength' ), 16 )
      self.type      = self.Attr( 'type' )
      self.attr      = int( self.Attr( 'bmAttributes' ), 16 )
      self.max       = int( self.Attr( 'wMaxPacketSize' ), 16 )

   def __repr__( self ) :
      return '<UsbEndpoint[%r]>' % self.fname

   def __str__( self ) :
      indent = '  ' * self.level
      name   = self.fname

      body   = '(EP) %02x: %s%s attr %02x len %02x max %03x' % \
         ( self.epaddr, self.type, self.ival, self.attr, self.len, self.max )
      body   = Colorize( 'Magenta', body )

      if f_showTree : return sIntent   % ( indent + sTrEP + name, body )
      else          : return sNoIntend % ( body )

# -----------------------------------------------------------------------------------
# --- Container for USB interface info ...
# -----------------------------------------------------------------------------------
class UsbInterface( UsbObject ) :

   def __init__( self, parent, fname, level=1 ) :
      self.parent    = parent
      self.level     = level
      self.fname     = fname
      self.path      = ''
      self.iclass    = ''
      self.isclass   = ''
      self.iproto    = ''
      self.noep      = 0
      self.driver    = ''
      self.devname   = ''
      self.protoname = ''
      self.eps       = []
      if self.fname :
         self.read( self.fname )

   def read( self, fname ) :

      self.fname     = fname
      self.path      = self.parent.path + '/' + fname
      self.iclass    =      self.Attr( 'bInterfaceClass'    ).upper()
      self.isclass   =      self.Attr( 'bInterfaceSubClass' ).upper()
      self.iproto    =      self.Attr( 'bInterfaceProtocol' ).upper()
      self.noep      = int( self.Attr( 'bNumEndpoints'      ) )

      try :
         self.driver  = self.Link( 'driver' )
         self.devname = FindDevName( self.driver, self.path )
      except :
         pass

      self.protoname = FindUsbClass( self.iclass, self.isclass, self.iproto )
      if f_showEPs :
         for dirent in os.listdir( prefix + self.path ) :
            if dirent.startswith( 'ep_' ) :
               ep = UsbEndpoint( self, dirent, self.level + 1 )
               self.eps.append( ep )

   def __repr__( self ) :
      return '<UsbInterface[%r]>' % self.fname

   def __str__( self ) :
      if f_showTree : indent = '  ' * self.level
      else          : indent = ''
      name     = self.fname

      plural   = ( ' ' if self.noep == 1 else 's' )
      body     = '(IF) %02s:%02s:%02s %iEP%s (%s) %s %s' % \
                 (self.iclass, self.isclass, self.iproto, self.noep, plural,
                  self.protoname, Colorize( 'Green', self.driver ), Colorize( 'Amber', self.devname ) )
      if f_showTree : strg = sIntent   % ( indent + sTree + name, body )
      else          : strg = sNoIntend % ( body )
      if f_showEPs and self.eps :
         for ep in self.eps :
            strg += str( ep )

      return strg

# -----------------------------------------------------------------------------------
# --- Container for USB (root) device info ...
# -----------------------------------------------------------------------------------
class UsbDevice( UsbObject ) :

   # --------------------------------------------------------------------------------
   def __init__( self, parent, fname, level=0 ) :
      self.parent       = parent
      self.level        = level
      self.fname        = fname
      self.path         = ''
      self.iclass       = ''
      self.isclass      = ''
      self.iproto       = ''
      self.vid          = ''
      self.pid          = ''
      self.name         = ''
      self.usbver       = ''
      self.speed        = ''
      self.maxpower     = ''
      self.noports      = 0
      self.nointerfaces = 0
      self.driver       = ''
      self.devname      = ''
      self.interfaces   = []
      self.children     = []
      if self.fname :
         self.read( self.fname )
         self.readchildren()

   # --------------------------------------------------------------------------------
   def read( self, fname ) :
      self.fname     = fname
      self.path      = fname
      self.iclass    =      self.Attr( 'bDeviceClass'    ).upper()
      self.isclass   =      self.Attr( 'bDeviceSubClass' ).upper()
      self.iproto    =      self.Attr( 'bDeviceProtocol' ).upper()
      self.vid       =      self.Attr( 'idVendor'        ).upper()
      self.pid       =      self.Attr( 'idProduct'       ).upper()

      try    : self.name   = self.Attr( 'manufacturer' ) + ' @ ' + self.Attr( 'product' )
      except : self.name   = FindUsbProd( self.vid, self.pid )

      # -----------------------------------------------------------------------------
      # --- Some USB Card readers have a better name then Generic ...
      if self.name.startswith( 'Generic' ) :
         oldnm = self.name
         self.name = FindUsbProd( self.vid, self.pid )
         if not self.name :
            self.name = oldnm

      try :
         ser = self.Attr( 'serial' )
         # --------------------------------------------------------------------------
         # --- Some USB devs report 'serial' as serial no. suppress
         if (ser and ser != 'serial' ) :
            self.name += ' @ ' + ser
      except :
         pass

      self.usbver    =      self.Attr( 'version' )
      self.speed     =      self.Attr( 'speed' )
      self.maxpower  =      self.Attr( 'bMaxPower' )
      self.noports   = int( self.Attr( 'maxchild' ) )

      try :
         self.nointerfaces = int( self.Attr( 'bNumInterfaces' ) )
      except :
         self.nointerfaces = 0

      try :
         self.driver = self.Link( 'driver' )
         self.devname = FindDevName( self.driver, self.path )
      except :
         pass

   # --------------------------------------------------------------------------------
   def readchildren( self ) :
      if self.fname[0:3] == 'usb' :
         fname = self.fname[3:]
      else :
         fname = self.fname

      # -----------------------------------------------------------------------------
      # --- 
      for dirent in os.listdir( prefix + self.fname ) :
         if not dirent[0:1].isdigit() : continue

         if os.access( prefix + dirent + '/bInterfaceClass', os.R_OK ) :
            iface = UsbInterface( self, dirent, self.level + 1 )
            self.interfaces.append( iface )
         else :
            usbdev = UsbDevice( self, dirent, self.level + 1 )

            self.children.append( usbdev )

      usbsortkey = lambda obj: [ int( x ) for x in re.split( r'[-:.]', obj.fname ) ]
      self.interfaces.sort( key=usbsortkey )
      self.children.sort  ( key=usbsortkey )

   # --------------------------------------------------------------------------------
   def __repr__( self ) :
      return '<UsbDevice[%r]>' % self.fname

   # --------------------------------------------------------------------------------
   def __str__( self ) :
      is_hub = ( self.iclass == HUB_ICLASS )
      if is_hub :
         if f_noEmptyHub and len( self.children ) == 0 :
            return ''

      strg = ''
      if not ( f_noHub and is_hub ) :
         if f_showTree : indent = '  ' * self.level
         else          : indent = ''
         name   = self.fname
         plural = ( ' ' if self.nointerfaces == 1 else 's' )

         body = '%s %02s  %iIF%s [USB %s, %4s Mbps, %5s] (%s) %s' % ( 
                  Colorize( 'Bold', '%04s:%04s' % ( self.vid, self.pid ) ),
                  self.iclass, self.nointerfaces, plural,
                  self.usbver.strip(), self.speed, self.maxpower,
                  Colorize( 'Red' if is_hub else 'Bold', self.name ),
                  Colorize( 'Red', 'hub' ) if is_hub else '' )

         if self.level > 0 :
            if f_showTree : strg = sIntent   % ( indent + sTree + name, body )
            else          : strg = sNoIntend % ( body )
         else : 
            if f_showTree : strg = sIntent   % ( indent         + name, body )
            else          : strg = sNoIntend % ( body )

         if not ( is_hub and not f_showHubIFs ) :
            if f_showEPs :
               ep = UsbEndpoint( self, 'ep_00', self.level + 1 )
               strg += str( ep )

            if f_showIFs :
               for iface in self.interfaces :
                  strg += str( iface )

      for child in self.children :
         strg += str( child )

      return strg

# -----------------------------------------------------------------------------------
# --- Read toplevel USB entries and print ...
# -----------------------------------------------------------------------------------
def ReadUsb() :

   root_hubs = []
   for dirent in os.listdir( prefix ) :
      if not dirent[0:3] == 'usb' : continue

      usbdev = UsbDevice( None, dirent, 0 )
      root_hubs.append( usbdev )

   root_hubs.sort( key=lambda x: int( x.fname[3:] ) )

   # --------------------------------------------------------------------------------
   for usbdev in root_hubs :
      print( usbdev, end='' )

# -----------------------------------------------------------------------------------
# --- main entry point ...
# -----------------------------------------------------------------------------------
def main( argv ) :

   global f_showIFs, f_showHubIFs, f_noEmptyHub, f_noHub, f_useColors, f_showEPs, f_showWarn, f_showTree, f_Debug
   global iUsbIDs, f_preColor
   global cols, iCols

   # --------------------------------------------------------------------------------
   try :
      ( optlist, args ) = getopt.gnu_getopt( argv[1:], 'tiIuUwcCef:d' )
   except getopt.GetoptError as exc :
      print( 'Error:', exc, file=sys.stderr )
      sys.exit(2)

   # --------------------------------------------------------------------------------
   for opt in optlist :
      # --- Debug-Mode
      if opt[0]         == '-d' :
         f_Debug        = True

      # --- Display hierarchy as a tree
      elif opt[0]       == '-t' :
         f_showTree      = True

      # --- Display interface information (IFs)
      elif opt[0]       == '-i' :
         f_showIFs      = True

      # --- Display interface information (IFs), even for hubs
      elif opt[0]       == '-I' :
         f_showIFs      = True
         f_showHubIFs   = True

      # --- Suppress empty hubs
      elif opt[0]       == '-u' :
         f_noEmptyHub   = True

      # --- Suppress all hubs
      elif opt[0]       == '-U' :
         f_noEmptyHub   = True
         f_noHub        = True

      # --- Use colors (default)
      elif opt[0]       == '-c' :
         f_useColors    = True

      # --- Disable colors
      elif opt[0]       == '-C' :
         f_useColors    = False

       # --- Display warning if usb.ids is not sorted correctly or missing
      elif opt[0]       == "-w":
         f_showWarn     = False

      # --- override filename "usb.ids"
      elif opt[0]       == '-f' :
         iUsbIDs        = [opt[1]]

      # --- Display endpoint info (EPs)
      elif opt[0]       == '-e' :
         f_showEPs      = True

   # --------------------------------------------------------------------------------
   if len( args ) > 0 :
      print( 'Error: excess args %s ...' % args[0], file=sys.stderr )
      sys.exit(2)

   if f_useColors is None :
      f_useColors = sys.stdout.isatty() and os.environ.get( 'TERM', 'dumb' ) != 'dumb'

   if f_Debug : f_preColor = 'esc'
   else       : f_preColor = 'css'


   # --------------------------------------------------------------------------------
   if iUsbIDs[0] :
      try :
         ParseUsbIDs()
      except:
         if f_showWarn : print( 'WARNING: Failure to read usb.ids', file=sys.stderr )

   # --------------------------------------------------------------------------------
   ReadUsb()

# -----------------------------------------------------------------------------------
# --- AvrLogger-Entry point :
# -----------------------------------------------------------------------------------
def View( arg=False ) :

   if arg : sys.argv = [ sys.argv[0], arg ]
   else   : sys.argv = [ sys.argv[0], '-eIc' ]

   main( sys.argv )

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
# sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------


